# trait class

import pygame
from pygame.locals import *
from sys import exit
from lostsky.core.linalg import Vector2
from math import floor, ceil
from random import randint

class TraitLearningSystem(object):

    def __init__(self,engine):
        
        """
        # Function Name: __init__
        # Purpose: Creates the trait learning system object
        """
    
        self.engine = engine
    
    def trait_learning_menu(self,unit):
        """
        # Function Name: trait learning menu
        # Purpose: trait learning system menu
        """
    
    
        text_menu_header = self.engine.bfont.render('Learn Traits: '+unit.name,True,(0,0,0))
        text_cost = self.engine.bfont.render('Cost',True,(0,0,0))
        text_empty = self.engine.sfont.render('Empty',True,(0,0,0))
        text_name = self.engine.bfont.render('Trait Name',True,(0,0,0))
        
        menu_flag = True
        menu_pos = 0
        
        # Generate List of available traits
        all_learnable_traits = self.get_learnable_traits(unit)
        all_trait_data = self.refresh_trait_list(unit,all_learnable_traits)
        
        if all_learnable_traits:
            text_selected_trait = self.engine.get_text_selected_trait(self.engine.trait_catalog[all_learnable_traits[menu_pos][0]])
        text_TP = self.engine.bfont.render("Trait Points: "+str(unit.trait_points),True,(0,0,0))
        
        
        while menu_flag:
        
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == KMOD_ALT|K_F4):
                    exit()
                if event.type == KEYDOWN:
                    
                    if ( event.key == K_UP or event.key == K_LEFT ):
                        if menu_pos > 0:
                            menu_pos -= 1
                        elif menu_pos == 0:
                            menu_pos = len(all_learnable_traits)-1
                        if all_learnable_traits:
                            text_selected_trait = self.engine.get_text_selected_trait(self.engine.trait_catalog[all_learnable_traits[menu_pos][0]])
                    if ( event.key == K_DOWN or event.key == K_RIGHT ):
                        if menu_pos < len(all_learnable_traits)-1:
                            menu_pos += 1
                        elif menu_pos == len(all_learnable_traits)-1:
                            menu_pos = 0
                        if all_learnable_traits:
                            text_selected_trait = self.engine.get_text_selected_trait(self.engine.trait_catalog[all_learnable_traits[menu_pos][0]])
                    if event.key == K_z or event.key == K_RETURN:
                        if all_learnable_traits:
                            
                            #Checks if there are enough TP available to learn this trait
                            if unit.trait_points >= all_learnable_traits[menu_pos][1]:
                                
                                learned_trait = self.engine.trait_catalog[all_learnable_traits[menu_pos][0]]
                                
                                confirmed = self.trait_confirm_menu(unit, learned_trait)
                                
                                if confirmed:
                                
                                    if learned_trait.variation == "Action":
                                        trait_index = 0
                                    elif learned_trait.variation == "Support":
                                        trait_index = 1
                                    # First tries to assign to available empty slots
                                    assign = unit.add_trait(learned_trait)
                                    # Places trait in reserve
                                    if not assign:
                                        
                                        # Sends the trait to reserve
                                        unit.reserve_traits[trait_index].append(learned_trait)
                                    
                                    # Updates unit's trait points
                                    unit.trait_points -= all_learnable_traits[menu_pos][1]
                                    
                                    # Refreshes displayed data
                                    all_learnable_traits = self.get_learnable_traits(unit)
                                    all_trait_data = self.refresh_trait_list(unit,all_learnable_traits)
                                    text_TP = self.engine.bfont.render("Trait Points: "+str(unit.trait_points),True,(0,0,0))
        
                                    # Fixes cursor position if beyond range
                                    if menu_pos > len(all_learnable_traits)-1:
                                        menu_pos = len(all_learnable_traits)-1
                                
                                    # Updates trait detail display
                                    if all_learnable_traits:
                                        text_selected_trait = self.engine.get_text_selected_trait(self.engine.trait_catalog[all_learnable_traits[menu_pos][0]])
                    
                    if event.key == K_x:
                        menu_flag = False

            if menu_flag:
                # Draw the learnable traits on the left side
                self.engine.surface.blit(self.engine.stats_bg,(0,0))
                self.engine.surface.blit(text_menu_header,(50,50))
                self.engine.surface.blit(text_name,(60,70))
                self.engine.surface.blit(text_cost,(250,70))
                self.engine.surface.blit(text_TP,(250,50))
                
                # Draw the information about the selected trait on the right side
                if all_learnable_traits:
                    for count,trait_data in enumerate(all_trait_data):
                        self.engine.surface.blit(trait_data[0],(70,90+count*20))
                        self.engine.surface.blit(trait_data[1],(270,90+count*20))
                    
                    self.engine.draw_trait_data(text_selected_trait)
                    
                    pygame.draw.circle(self.engine.surface, (0,0,0), (50,98+20*menu_pos),5)
        
                pygame.display.flip()
                self.engine.clock.tick(60)
                
    def trait_confirm_menu(self,unit,trait):
        """
        # Function Name: trait confirm menu
        # Purpose: trait asks the user to verify that they want to learn the trait
        """
        text_menu_header = self.engine.bfont.render('Learn Traits: '+unit.name,True,(0,0,0))
        text_cost = self.engine.bfont.render('Cost',True,(0,0,0))
        text_empty = self.engine.sfont.render('Empty',True,(0,0,0))
        text_name = self.engine.bfont.render('Trait Name',True,(0,0,0))
        
        text_confirm = self.engine.bfont.render('Confirm learning trait:',True,(0,0,0))
        text_trait_name = self.engine.bfont.render(trait.name,True,(0,0,0))
        text_yes = self.engine.bfont.render('Yes',True,(0,0,0))
        text_no = self.engine.bfont.render('No',True,(0,0,0))
        text_trait_name = self.engine.bfont.render(trait.name,True,(0,0,0))
        text_unit_name = self.engine.sfont.render("Character: "+unit.name,True,(0,0,0))
        text_trait_type = self.engine.sfont.render("Type: "+trait.variation,True,(0,0,0))
        text_trait_cost = self.engine.sfont.render("Cost: %s TP"%(trait.cost),True,(0,0,0))
        
        menu_flag = True
        menu_pos = True
        
        # Generate List of available traits
        all_learnable_traits = self.get_learnable_traits(unit)
        all_trait_data = self.refresh_trait_list(unit,all_learnable_traits)
                
        text_TP = self.engine.bfont.render("Trait Points: "+str(unit.trait_points),True,(0,0,0))
        
        while menu_flag:
        
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == KMOD_ALT|K_F4):
                    exit()
                if event.type == KEYDOWN:
                    
                    if event.key in (K_UP,K_LEFT,K_RIGHT,K_DOWN):
                        menu_pos = not menu_pos
                        
                    if event.key == K_z or event.key == K_RETURN:
                        return menu_pos
                            
                    if event.key == K_x:
                        return False

            if menu_flag:
                # Draw the learnable traits on the left side
                self.engine.surface.blit(self.engine.stats_bg,(0,0))
                self.engine.surface.blit(text_menu_header,(50,50))
                self.engine.surface.blit(text_name,(60,70))
                self.engine.surface.blit(text_cost,(250,70))
                self.engine.surface.blit(text_TP,(250,50))
                
                
                # Draw the information about the selected trait on the right side
                if all_learnable_traits:
                    for count,trait_data in enumerate(all_trait_data):
                        self.engine.surface.blit(trait_data[0],(70,90+count*20))
                        self.engine.surface.blit(trait_data[1],(270,90+count*20))
                    
                self.engine.surface.blit(self.engine.results_panel,(280,140))
                self.engine.surface.blit(text_confirm,(300,160))
                self.engine.surface.blit(text_yes,(360,180))
                self.engine.surface.blit(text_no,(460,180))

                self.engine.surface.blit(text_trait_name,(300,300))
                self.engine.surface.blit(text_unit_name,(320,320))
                self.engine.surface.blit(text_trait_type,(320,340))
                self.engine.surface.blit(text_trait_cost,(320,360))
                
                # Draws the cursor
                pygame.draw.circle(self.engine.surface, (0,0,0), (453-100*menu_pos,188),5)
                
                pygame.display.flip()
                self.engine.clock.tick(60)
        
    def get_learnable_traits(self,unit):
        """
        # Function Name: get learnable traits
        # Purpose: get a list of learnable traits
        # Input: unit - unit to get traits for
        # Output: all_learnable_traits - a list of all learnable traits
        """
    
        all_learnable_traits = []
    
        # Appends [Trait name, cost, T/F if unit has enough TP to learn this] if trait can be learned
        [all_learnable_traits.append([trait.name,trait.cost, unit.trait_points >= trait.cost])for trait in self.engine.trait_catalog.values() if trait.check_restrictions(unit)]
        
        all_learnable_traits.sort()
    
        return all_learnable_traits

    def refresh_trait_list(self,unit,all_learnable_traits):
        
        """
        # Function Name: refresh_trait_list
        # Purpose: renders the trait list into appropriate text surface images.
        # Input: unit - unit to get traits for
        #        all_learnable_traits - a list of all traits unit can learn
        # Output: all_trait_data = list of all text objects
        """
        all_trait_data = []
        # Draws info in black if unit has enough TP, gray if it doesn't
        for trait_data in all_learnable_traits:
            
            # 3rd entry in list is a T/F flag to determine if the unit has enough TP
            # Case: unit has enough TP
            if trait_data[2]:
                all_trait_data.append([self.engine.sfont.render(trait_data[0],True,(0,0,0)),
                                   self.engine.sfont.render(str(trait_data[1]),True,(0,0,0))])
            # Case: unit doesn't have enough
            else:
                all_trait_data.append([self.engine.sfont.render(trait_data[0],True,(100,100,100)),
                                   self.engine.sfont.render(str(trait_data[1]),True,(100,100,100))])

        return all_trait_data
            
        text_TP = self.engine.bfont.render("Trait Points: "+str(unit.trait_points),True,(0,0,0))


class Trait(object):




    def __init__(self, name, variation, desc, statmods, movemod_add, movemod_mult, flight, expmod, spiritmod,restrictions,cost,properties=[]):

        """
        # Function Name: __init__
        # Purpose: Creates a trait class
        # Inputs: Name = Trait's name (e.g. "Magic Attack +", "Flight"
        #         Variation = Support or Action (Support Types are mainly stat boosters, Action types affect battle events)
        #         Desc = Short description of the traitibute
        #         Statmods = A list containing 6 numbers for STR, DEF, MAG, MDEF, ACC, AGL percent bonuses
        #         Movemod_add = A modifier that adds to a units movement points. Must be a non-negative integer
        #         Movemod_mult = A multiplier modifier to a units allowed movement spots. Must be a non-negative integer
        #         Flight = If True, it grants the unit flight
        #         Expmod = A positive rational number that multiplies experience gains in battle.
        #         Spiritmod = A positive rational number that multiplies spirit gains (and losses) in battle.
        """
        
        # Name
        self.name = name
        
        # Variation
        self.variation = variation
        self.desc = desc
        
        # Base stat modifiers
        self.statmods = statmods

        # Movement modifiers (Addition and subtraction are mutually exclusive)
        self.movemod_add = movemod_add
        self.movemod_mult = movemod_mult

        # If flight is TRUE the unit is granted flying ability while this is equipped.
        self.flight = flight

        # EXPmod is multiplicative
        self.expmod = expmod

        # Spirit Mod is multiplicative of total received at the end of the battle
        self.spiritmod = spiritmod
        
        # Additional properties of the traitibute
        self.properties = properties

        # Trait Learning Information
        self.cost = cost
        # Restrictions Format:
        # {'class':[LIST],'character':[LIST],'prereqs':[LIST]}
        self.restrictions = restrictions

        
    def __str__(self):
        """
        # Function Name: __str__
        # Purpose: String interpretation of a trait
        # Output: Trait's name
        """
    
        return self.name

    def check_restrictions(self,unit):

        """
        # Function Name: check restrictions
        # Purpose: Determines if a unit can learn this trait
        # Input: Unit - unit to check restrictions on 
        # Output: True if unit can learn this trait
        #         False if unit cannot learn this trait
        """
    
        # Constructs a list of all known traits
        known_traits = [] 
        [known_traits.append(trait.name) for trait in unit.traits[0] if trait]
        [known_traits.append(trait.name) for trait in unit.traits[1] if trait]
        [known_traits.append(trait.name) for trait in unit.reserve_traits[0] if trait]
        [known_traits.append(trait.name) for trait in unit.reserve_traits[1] if trait]
        
        # Character has already learned this trait
        if self.name in known_traits:
            return False

        # Character is not at a high enough level
        if self.restrictions['level'] and unit.level < self.restrictions['level']:
            return False
            
        # Case character is not in the list of traits who can learn this
        if self.restrictions['class'] and unit.unitclass not in self.restrictions['class']:
            return False
        
        # Case character is not in the list of units who can learn this
        if self.restrictions['character'] and unit.name not in self.restrictions['character']:
            return False

        # Checks prerequisite traits
        if self.restrictions['prereq'] and not set(self.restrictions['prereq']).issubset(set(known_traits)):
            return False
            
        return True
        

    def turn_execute(self,user):
        
        """
        # Function Name: turn_execute
        # Purpose: Does whatever the trait is designed to do at the beginning of the turn
        # Inputs: user - The person who is using this trait
        """
    
        # Regeneration HP
        if ('Regen Lv.1' in self.properties or 'Regen Lv.2' in self.properties or 'Regen Lv.3' in self.properties):
            # Regen Lv.1 - 10% max HP
            if 'Regen Lv.1' in self.properties:
                recovery = int(user.maxHP*0.10)
                
            # Regen Lv.2 - 15% max HP
            elif 'Regen Lv.2' in self.properties:
                recovery = int(user.maxHP*0.15)
            
            # Regen Lv.3 - 20% max HP
            elif 'Regen Lv.3' in self.properties:
                recovery = int(user.maxHP*0.20)
                
            user.map_heal(user,self.name,recovery,'orb2')
            print user.name + " regen: " + str(recovery)
            
    def turn_check(self,user):
        
        """
        # Function Name: turn_execute
        # Purpose: Checks if the trait is needed to be executed on this turn
        # Inputs: user - The person who is using this trait
        # Outputs: True if trait needs to be executed, False if trait does not need to be executed
        """
        
        if ('Regen Lv.1' in self.properties 
            or 'Regen Lv.2' in self.properties 
            or 'Regen Lv.3' in self.properties) and user.HP < user.maxHP:
            return True
        else:
            return False
    