from lostsky.worldmap.event import BattleEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, ArrivalTrigger, CustVarTrigger

class Mission(BattleEvent):
    
    def __init__(self):
        # Event Data
        name = 'Marisa\'s Arrival'
        location = 'Central Forest'
        id_string = 'CH1ST2'
        prereqs = ['CH1ST1']
        auto_signup = True
        cancellable = False
        show_rewards = True
        desc = "Early this morning, I spoke with one Marisa Kirisame who was out hunting for mushrooms when the trees began their rampage. She was a little anxious to get back to her house. I wonder what the intent of those walking trees is. This reporter will do all she can to find out!"
        
        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch1st2b.txt'
        mission_type = 'battle'
        objective = {'type':'Defeat Boss',
                     'target':'Kodama Lord',
                     'desc':'Defeat the Kodama Lord!'
                     }   
        
        deploy_data = {'enable':False,
                       'max_units':None,
                       'preset_units':None,
                       'boxes':[]
                       }

        reward_list = [('spell_action','Rice Cake'), 
                       ('treasure','synth_fire'), 
                      ]
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree A',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree C',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree D',
                                    'level': 4},     
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy A',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy B',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy C',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy D',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy E',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy F',
                                    'level': 4},
                           {'template_name': 'Kodama Lord',
                                'unit_name': 'Kodama Lord',
                                    'level': 5},
                           
                           #intro scene enemies         
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree B',
                                    'level': 2},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball A',
                                    'level': 1},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball B',
                                    'level': 1},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball C',
                                    'level': 1},
                          ]
        
        initial_spells = {'Walking Tree A':['Leaf Crystal'],
                          'Walking Tree B':['Leaf Crystal'],
                          'Walking Tree C':['Leaf Crystal'],
                          'Walking Tree D':['Leaf Crystal'],
                          'Fairy A':['Holy Amulet'],
                          'Fairy B':['Holy Amulet'],
                          'Fairy C':['Fireball'],
                          'Fairy D':['Fireball'],
                          'Fairy E':['Fireball'],
                          'Fairy F':['Holy Amulet'],
                          'Kodama Lord':['Leaf Crystal'],
                          'Fuzzball A':['Dagger Throw'],
                          'Fuzzball B':['Holy Amulet'],
                          'Fuzzball C':['Holy Amulet']}
                          
        initial_traits = {}
        initial_ai_states = {'Walking Tree A':'Attack',
                          'Walking Tree B':'Attack',
                          'Walking Tree C':'Attack',
                          'Walking Tree D':'Attack',
                          'Fairy A':'Attack',
                          'Fairy B':'Attack',
                          'Fairy C':'Attack',
                          'Fairy D':'Attack',
                          'Fairy E':'Attack',
                          'Fairy F':'Attack',
                          'Kodama Lord':'Attack',
                          'Fuzzball A':'Attack',
                          'Fuzzball B':'Attack',
                          'Fuzzball C':'Attack'}
                          
        initial_locations = {'Ran':(0,22),
                             'Chen':(0,24),
                             'Youmu':(1,23),
                             
                             'Walking Tree B':(40,20),
                             'Fuzzball A':(40,20),
                             'Fuzzball B':(40,20),
                             'Fuzzball C':(40,20),
                             
                             'Walking Tree A':(24,23),
                             'Walking Tree C':(28,26),
                             'Walking Tree D':(27,12),
                             
                             'Fairy A':(12,12),
                             'Fairy B':(10,9),
                             'Fairy C':(16,11),
                             'Fairy D':(19,9),
                             'Fairy E':(12,8),
                             'Fairy F':(14,10),
                             'Kodama Lord':(18,9),
                             
                             }
        reserve_units = []#[list of unit names to deploy later in mission]
        all_landmarks = [{'name':'M1',
                          'id_string':'mushroom',
                          'location':(10,11)},
                         {'name':'M2',
                          'id_string':'mushroom',
                          'location':(8,8)},
                         {'name':'M3',
                          'id_string':'mushroom',
                          'location':(23,8)},
                         {'name':'M4',
                          'id_string':'mushroom',
                          'location':(18,10)},
                         {'name':'M5',
                          'id_string':'mushroom',
                          'location':(24,21)},
                         {'name':'M6',
                          'id_string':'mushroom',
                          'location':(11,18)},
                         {'name':'M7',
                          'id_string':'mushroom',
                          'location':(14,22)},
                         {'name':'M8',
                          'id_string':'mushroom',
                          'location':(13,27)},
                         {'name':'CB1',
                          'id_string':'cherryblossom_tree',
                          'location':(27,23)},
                         ]
    
        required_starters = ['Youmu','Chen','Ran']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [DirectPathMAE(),
                                AlternatePathMAE(),
                                TreasureAlertMAE(),
                                TreasureFoundMAE(),
                                ]
        post_mission_MAE = PostMissionMAE()
        required_survivors = ['Youmu', 'Chen', 'Ran', 'Marisa']
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Marisa arrives, tutorial part 2
        """
        
        self.set_cursor_state(False)
        self.set_stats_display(False)
 
        # Set up Marisa
        self.add_to_party('Marisa')
        self.assign_trait('Marisa','Flight')
        self.assign_spell('Marisa','Fireball')
        self.assign_spell('Marisa','Master Spark')
        self.set_spell_uses('Marisa','Master Spark',2)
        self.set_spirit_charge('Marisa',505)
        self.set_unit_pos('Marisa',(30,22))
        
        # Heroes arrive on the scene
        self.center_on('Youmu')
        self.move_unit('Youmu',(3,23))
        self.move_unit('Ran',(2,22))
        self.move_unit('Chen',(2,24))
 
        
        self.say("Are we heading the right way?",
                'Chen',
                'Chen')
        self.say("We must be!  We can't afford to be getting lost!",
                'Youmu',
                'Youmu')
        self.say("Wait, something's coming.",
                'Ran',
                'Ran')
        
        # Setup intro scene
        self.set_unit_pos('Fuzzball A',(13,6))
        self.set_unit_pos('Fuzzball B',(13,6))
        self.set_unit_pos('Fuzzball C',(13,6))
        self.set_unit_pos('Walking Tree B',(13,6))
        
        # Three Fuzzballs move in.
        # One of them tries to go towards Youmu, but she blocks the way
        # All Fuzzballs retreat
        self.move_unit('Fuzzball A',(13,23))
        self.move_unit('Fuzzball B',(13,22))
        self.move_unit('Fuzzball C',(13,21))
        self.move_unit('Walking Tree B',(13,18))
        self.move_unit('Fuzzball A',(7,23))
        self.move_unit('Walking Tree B',(13,20))
        self.move_unit('Fuzzball B',(13,31))
        self.move_unit('Fuzzball C',(13,31))
        self.move_unit('Fuzzball A',(5,23))
        self.move_unit('Youmu',(4,23))
        self.startle('Youmu')
        self.move_unit('Fuzzball A',(13,23))
        self.move_unit('Fuzzball A',(13,31))
        self.move_unit('Walking Tree B',(13,22))
        
        
        # Remove the three fuzzballs in the intro scene
        self.kill_unit('Fuzzball A')
        self.kill_unit('Fuzzball B')
        self.kill_unit('Fuzzball C')
        
        # Marisa's entrance
        self.play_music('battle01')
        self.say("Outta my way!",
                'Marisa',
                'Marisa')
        self.move_unit('Marisa',(17,22))
 
        # Marisa uses MS on Tree
        self.script_battle('Marisa',
                           'Walking Tree B',
                           {'lhs_hit':True,
                            'lhs_crit':True,
                            'lhs_equip':1,
                            'rhs_hit':True,
                            'rhs_crit':False,
                            'rhs_equip':0
                            }
                           )
        self.say("Isn't that...",
                'Youmu',
                'Youmu')

        self.move_unit('Marisa',(15,22))

        # Hero group moves up
        self.move_unit('Youmu',(6,23))
        self.move_unit('Ran',(5,22))
        self.move_unit('Chen',(5,24))
        self.say("Marisa?",
                'Youmu',
                'Youmu')
        self.move_unit('Youmu',(11,23))
        self.move_unit('Ran',(11,22))
        self.move_unit('Chen',(11,24))
        self.center_on('Youmu')
        
        self.say("You here for trouble?",
                'Marisa',
                'Marisa')
        self.say("We're having the same troubles, actually.",
                'Ran',
                'Ran')
        self.say("Our Masters haven't come back yet after going to the forest last night.",
                'Chen',
                'Chen')
        self.say("So two troublemakers disappear and the forest goes crazy the next morning?  Everything makes sense now.",
                'Marisa',
                'Marisa')
        self.say("Are you suggesting that we're behind all of this?",
                'Youmu',
                'Youmu')
        self.say("You saying they aren't?",
                'Marisa',
                'Marisa')
        self.emote('Youmu','dotdotdot')
        self.emote('Marisa','dotdotdot')
        self.say("... Um, aren't we in a hurry?",
                'Chen',
                'Chen')
        self.say("How 'bout I help you out?",
                'Marisa',
                'Marisa')
        self.say("And what did you need from us?",
                'Ran',
                'Ran')
        self.say("They caught me as I was going to gather mushrooms to use in spell casting, and they've cut me off from my house. I've only got enough for one more Master Spark spell.",
                'Marisa',
                'Marisa')
        self.say("My house is up ahead. You help me get my stuff back and you have what magic I have back you up.",
                'Marisa',
                'Marisa')
        self.say("That sounds like a reasonable deal. Very well, to Marisa's house then.",
                'Youmu',
                'Youmu')
        self.say("Ah... Hush everyone, I hear voices coming from over there.",
                'Ran',
                'Ran')
 
        # Spying on the actions of the Kodama Lord and her servants       
        self.move_unit('Ran',(10,20))
        self.center_on('Ran')
        self.pause(1.0)
        self.center_on('Kodama Lord')
        self.set_cursor_state(True)
        self.pause(0.5)
        
        self.move_unit('Fairy F',(17,9))
        self.startle('Fairy F')
        
        self.say("I see. I see. Let's keep the pressure up on them. We need to control every bit of the forest.",
                '???',
                None)
        self.startle('Fairy F')
        self.move_unit('Fairy F',(14,10))
        self.say("Ugh, what a mess. We didn't expect the forest residents to be this stubborn.",
                '???',
                None)
        self.say("But this setup is perfect!  That magician won't make it past this trap!  Eheheh!",
                '???',
                None)
        
        # Heroes plan their next actions
        self.set_cursor_state(False)
        self.center_on('Ran')
        self.move_unit('Ran',(11,22))
        self.center_on('Ran')
        self.say("This explains some things. That's a Kodama. They're are both the protectors and the leaders of the tree youkai.",
                'Ran',
                'Ran')
        self.center_on('Marisa')
        self.say("You'd think they'd go setting traps a little more quietly. I say we blast our way through!",
                'Marisa',
                'Marisa')
        self.say("You said you only had one Spark spell right? We shouldn't waste time with the trap if possible. If we could just get to their leader...",
                'Ran',
                'Ran')
        self.say("It's the brains behind the others. Without it, they'll likely be disoriented and scatter, leaving us a way through.",
                'Ran',
                'Ran')
        self.set_cursor_state(True)
        self.center_on('Walking Tree A')
        self.pause(1.0)
        self.say("Well, there's a a small path that circles around nearby.  We can use that and get to the loud one.",
                'Marisa',
                'Marisa')
        self.center_on('Walking Tree D')
        self.pause(1.0)
        self.center_on('Marisa')
        self.set_cursor_state(False)
        self.say("The longest road is the shortest path, but we'll trust in your decision.",
                'Ran',
                'Ran')
        self.say("Marisa joins the party!",
                None,
                None)
        self.say("As a witch, my specialty is moving around. I may not be able to take many hits, but I can cover a lot of ground fast.",
                'Marisa',
                'Marisa')
        self.set_cursor_state(True)
        self.set_stats_display(True)
        
        tutorial_selection = self.choice('Do you wish to see a tutorial about Elemental Relations and Traits?',['Yes','No'])
        if tutorial_selection == 'Yes':
                        
            self.set_stats_display(False)
            self.say("We still face a lot of tough enemies on the road ahead. Is there anything that can give us an advantage?",
                    'Youmu',
                    'Youmu')
            self.say("Youmu, you're not a sorcerer so you probably don't know this. Marisa, are you familiar with the concept of Magic Flow?",
                    'Ran',
                    'Ran')
            self.say("Of course, that's something that every magician needs to know. Ah, let us explain.",
                    'Marisa',
                    'Marisa')
            
            # Tutorial part 1 - Spell Relations
            self.show_image('tutorial_spell_type','tutorial_spell_type.png', (490,210))
            self.say("Spell actions are divided up into four different schools, linked in a cycle that represents the natural flow of magic in the world: Nature to Elemental to Spiritual to Force. Force cycles back around to Nature.",
                    'Marisa',
                    'Marisa')
            self.say("In moving with the flow of magic, your attacks will be stronger. For instance, Youmu's Dagger Throw spell will work more effectively on an enemy equipped with a Nature type spell such as Leaf Crystal.",
                    'Ran',
                    'Ran')
            self.emote('Chen','lightbulb')
            self.say("So that means my Leaf Crystal spell won't be effective against any enemies that are using the Spiritual type Holy Amulet spell, right?",
                    'Chen',
                    'Chen')
            self.say("Right! You got that, Youmu?",
                    'Ran',
                    'Ran')
            self.say("I think so. We just have to follow the flow of magic to get the best out of our spells.",
                    'Youmu',
                    'Youmu')
            self.hide_image('tutorial_spell_type')
 
            # Tutorial Part 2 - Traits
            self.say("Let's move on to Traits now. They come in two flavors: Support and Action. Support traits are mostly focused on stat improvement while Action traits are more battle focused.",
                    'Ran',
                    'Ran')
            
            self.show_image('tutorial_flight','tutorial_traits_flight.png',(490,210))
            self.show_image('tutorial_noflight','tutorial_traits_noflight.png',(210,210))
            self.say("For instance, my Action trait Flight which allows me to travel easily over any terrain, very convenient for traveling in rough regions of Gensokyo.",
                    'Marisa',
                    'Marisa')
            self.hide_image('tutorial_flight')
            self.hide_image('tutorial_noflight')
            self.say("Each character has an active Support and Action trait and up to four standby traits for each type.",
                    'Tutorial',
                    None)
            self.say("With every level up, two Trait Points are earned which can be spent on buying traits in the world map party management menu.",
                    'Tutorial',
                    None)
            self.say("Some traits are unique to characters and some are not learned until a certain level or until other traits have been learned.",
                    'Tutorial',
                    None)
            self.say("Any additional traits need to be swapped in on the world map in the main menu.",
                    'Tutorial',
                    None)
 
            self.show_image('tutorial_save','tutorial_traits_save.png',(490,210))
            self.say("Remember to save when you swap in traits in mid-battle.",
                    'Tutorial',
                    None)
            self.hide_image('tutorial_save')
            self.set_stats_display(False)
        
        # Custom mission flag to see if player has chosen a strategy
        self.set_cust_var('player_path_chosen', False)

class DirectPathMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [# Triggers if any unit from the player team moves into a set of coordinates
                    ArrivalTrigger((8,6,11,15),1),
                    
                    # Verifies that the player has not already shosen a path
                    CustVarTrigger('player_path_chosen', False)
                    
                    ]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        Triggers if player chooses a direct attack up the middle of the forest
        """
 
        self.set_stats_display(False)
        self.center_on('Kodama Lord')
        self.emote('Kodama Lord','exclamation')
        self.say("Just as I predicted!",
                'Kodama Lord',
                None)
        self.say(" Attack!",
                'Kodama Lord',
                None)
        self.center_on_coords((27,14))
        
        # Makes sure the alternate path event is not triggered
        self.set_cust_var('player_path_chosen',True)
        
        self.set_ai_state('Kodama Lord','Pursuit')
        self.set_stats_display(True)


class AlternatePathMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [# Triggers if any unit from the player team moves into a set of coordinates
                    ArrivalTrigger((19, 8, 11, 9), 1),
                    
                    # Verifies that the player has not already shosen a path
                    CustVarTrigger('player_path_chosen',False)
                    ]
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Case if player decides to go for a sneak attack
        """
        self.set_stats_display(False)
        self.center_on('Kodama Lord')
        self.emote('Kodama Lord','questionmark')
        self.startle('Kodama Lord')
        self.say("How did you get over there?",
                'Kodama Lord',
                None)
        self.center_on_coords((27,14))
        
        # Makes sure the alternate path event is not triggered
        self.set_cust_var('player_path_chosen',True)
        
        self.set_ai_state('Kodama Lord','Pursuit')
        self.set_stats_display(True)

class TreasureAlertMAE(MapActionEvent):

    def __init__(self):
        triggers = [ArrivalTrigger((18, 20, 7, 4), 1)]
        
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        Alerts player to presence of treasure under the tree
        """
        self.center_on_coords((27,23))
        self.say("There seems to be something shiny sitting under of the cherry blossom tree.",
                None,
                None)
        self.center_on_coords((18,22))

class TreasureFoundMAE(MapActionEvent):

    def __init__(self):
        triggers = [ArrivalTrigger((27, 23, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        Player has discovered treasure under cherry blossom tree
        """
        
        self.say("Buried at the foot of the cherry tree is a small rusted box.",
                None,
                None)
        self.say("Acquired Treasure Item: Wooden Statue!",
                None,
                None)
        self.add_item('treasure','000_statue',1)

class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.remove_all_enemies()
        self.stop_music()
        self.set_unit_pos('Youmu',(14,3))
        self.set_unit_pos('Ran',(13,3))
        self.set_unit_pos('Chen',(16,4))
        self.set_unit_pos('Marisa',(15,3))

    def execute(self):
        """
        Victory Scene
        """

        self.center_on('Marisa')
        self.say("Not very loyal for an army, are they? They scattered as soon as the Kodama Lord went down.",
                'Marisa',
                'Marisa')
        self.say("Hey, what's this? One of those fairies left it behind",
                'Chen',
                'Chen')
        
        self.show_image('fire_crystal',
                        'fire_crystal.png',
                        (350,210))
        
        self.say("It's definitely a magical item.",
                'Ran',
                'Ran')
        self.say("I bet if we take it to Kourin, he can find out. That is, if his place hasn't already been trampled by these trees as well.",
                'Marisa',
                'Marisa')
 
        self.hide_image('fire_crystal')
        self.say("The path ahead looks clear. Which way is it to your house, Marisa? ",
                'Chen',
                'Chen')
        self.say("Next stop on the forest tour: my place. Let's go!",
                'Marisa',
                'Marisa')
        self.say("Lead the way, Marisa.",
                'Youmu',
                'Youmu')
        self.say("I swear, if they've wrecked my house, I'm going to turn these walking trees into firewood!",
                'Marisa',
                'Marisa')
        
        self.set_cursor_state(True)
        self.set_stats_display(True)
    