from lostsky.worldmap.event import BattleEvent 
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, ArrivalTrigger
class Mission(BattleEvent):
    
    def __init__(self):
        # Event Data
        name = 'Lost Lantern'
        location = 'Marisa\'s House'
        id_string = 'CH1ST3'
        prereqs = ['CH1ST2']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "After she united with Youmu and her party, Marisa led them north towards her house. At the same time, I spotted one of the Kodama Lords up at Marisa's place. She said that she was here to stop that \"tree wrecking witch who would blow up the whole forest if given the chance.\" There are rumors floating about that some of the bug youkai have also allied themselves with the Kodama Lords!"

        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch1st3v2.txt'
        mission_type = 'battle'
        objective = {'type':'Arrive and Defeat Boss',
                     'target': 'Wriggle',
                     'location_name': 'Marisa\'s House',
                     'location_box': (19,6,3,3),
                     'desc': "Defeat Wriggle and get any party member to Marisa's house (3x3 box)!"}
        
        deploy_data = {'enable':False,
                       'max_units':None,
                       'preset_units':None,
                       'boxes':[]
                       }
        
        reward_list = [('treasure','synth_wood'), 
                       ('treasure','synth_metal'), 
                      ]
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree A',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree B',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree C',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree D',
                                    'level': 4},     
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree E',
                                    'level': 4},
                                     
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy A',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy B',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy C',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy D',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy E',
                                    'level': 4},         
                           
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly A',
                                    'level': 5},
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly B',
                                    'level': 5},
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly C',
                                    'level': 5},
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly D',
                                    'level': 5},
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly E',
                                    'level': 5},
                                    
                           {'template_name': 'Kodama Lord',
                                'unit_name': 'Kodama Lord',
                                    'level': 10},
                                    
                           {'template_name': 'Wriggle',
                                'unit_name': 'Wriggle',
                                    'level': 6},
                          ]
        
        initial_spells = {'Walking Tree A':['Leaf Crystal'],
                          'Walking Tree B':['Leaf Crystal'],
                          'Walking Tree C':['Leaf Crystal'],
                          'Walking Tree D':['Leaf Crystal'],
                          'Walking Tree E':['Leaf Crystal'],
                          'Fairy A':['Holy Amulet'],
                          'Fairy B':['Fireball'],
                          'Fairy C':['Dagger Throw'],
                          'Fairy D':['Holy Amulet'],
                          'Fairy E':['Dagger Throw'],
                          'Firefly A':['Fireball'],
                          'Firefly B':['Fireball'],
                          'Firefly C':['Fireball'],
                          'Firefly D':['Fireball'],
                          'Firefly E':['Fireball'],
                          'Wriggle':['Fireball']}
                          
        initial_traits = {'Firefly A':['Flight'],
                          'Firefly B':['Flight'],
                          'Firefly C':['Flight'],
                          'Firefly D':['Flight'],
                          'Firefly E':['Flight'],
                          'Wriggle':['Flight','Mirage']}
                          
        initial_ai_states = {'Walking Tree A':'Attack',
                             'Walking Tree B':'Attack',
                             'Walking Tree C':'Attack',
                             'Walking Tree D':'Attack',
                             'Walking Tree E':'Attack',
                             'Fairy A':'Attack',
                             'Fairy B':'Attack',
                             'Fairy C':'Attack',
                             'Fairy D':'Attack',
                             'Fairy E':'Attack',
                             'Firefly A':'Attack',
                             'Firefly B':'Attack',
                             'Firefly C':'Attack',
                             'Firefly D':'Defend',
                             'Firefly E':'Defend',
                             'Wriggle':'Attack'}
                             
        initial_locations = {'Ran':(15,23),
                             'Chen':(16,23),
                             'Youmu':(17,23),
                             'Marisa':(16,22),
                             
                             'Walking Tree A':(4,14),
                             'Walking Tree B':(10,17),
                             'Walking Tree C':(1,7),
                             'Walking Tree D':(10,4),
                             'Walking Tree E':(9,5),
                             
                             'Fairy A':(6,16),
                             'Fairy B':(3,9),
                             'Fairy C':(4,6),
                             'Fairy D':(19,8),
                             'Fairy E':(21,8),
                             
                             'Firefly A':(17,12),
                             'Firefly B':(21,13),
                             'Firefly C':(15,3),
                             'Firefly D':(13,2),
                             'Firefly E':(17,2),
                             
                             'Kodama Lord':(14,1),
                             'Wriggle':(15,1),
                             }
        reserve_units = []#[list of unit names to deploy later in mission]
        all_landmarks = [{'name':'House1',
                          'id_string':'house_1',
                          'location':(20,7)},
                         {'name':'LP 1',
                          'id_string':'lilypad',
                          'location':(22,3)},
                         {'name':'CB 1',
                          'id_string':'cherryblossom_tree',
                          'location':(6,6)},
                         {'name':'CB 2',
                          'id_string':'cherryblossom_tree',
                          'location':(15,12)},
                         ]
    
        required_starters = ['Youmu','Chen','Ran','Marisa']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [MarisaHouseMAE(), NorthTreasureMAE(), SouthTreasureMAE()]
        required_survivors = ['Youmu', 'Chen', 'Ran', 'Marisa', 'Wriggle']
        post_mission_MAE = PostMissionMAE()
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Arriving at Marisa's house, the heroes find an ambush set up by Kodama Lord
        and Wriggle.
        """
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.center_on('Wriggle')
        self.say("So the little human's still causing a ruckus?",
                'Wriggle',
                'Wriggle')
        self.say("And after we went to such trouble to get her cornered.  Make sure she doesn't make it back.",
                'Kodama Lord',
                None)
        self.say("But she's already here...",
                'Wriggle',
                'Wriggle')
        self.say("WHAT!?",
                'Kodama Lord',
                None)

        # Scene with hero party discussing what to do next
        self.center_on('Marisa')
        self.say("Just a bit farther!",
                'Marisa',
                'Marisa')
        self.say("I managed to scrap together enough spell fuel to fully recharge my Master Spark from the mushrooms I collected, but that's all.",
                'Marisa',
                'Marisa')
        self.say("So that's why you were tinkering with those vials on the way here.",
                'Chen',
                'Chen')
        self.play_music('battle01')
        self.say("There's a large presence nearby.",
                'Ran',
                'Ran')
        self.say("Are we prepared for it?",
                'Youmu',
                'Youmu')
        self.move_unit('Marisa',(16,21))
        self.startle('Marisa')
        self.say("Just give me a path to my house, and I'll roast whatever \"boss\" still lingers.",
                'Marisa',
                'Marisa')
        self.say("I have a hidden spell fuel stash in my house up there.",
                'Marisa',
                'Marisa')
        self.set_cursor_state(True)
        self.center_on_coords((20,7))
        self.pause(0.5)
        self.say("If I can get back, I should be able to refuel my magic reactor to full power!",
                'Marisa',
                'Marisa')
        self.set_cursor_state(False)
        self.center_on('Wriggle')
        self.say(" W-what are you waiting for?!  S-stop that human!",
                'Kodama Lord',
                None)
        self.say("What exactly do you want us to do?",
                'Wriggle',
                'Wriggle')
        self.say("Either defeat them or chase them out of the forest! I'll let you use these fairies and walking trees.",
                'Kodama Lord',
                None)
        self.say("And remember our promise. If we agree to help you, the other insects and I get to stay.",
                'Wriggle',
                'Wriggle')
        self.say("I'll leave it to you then. That magician isn't the only troublemaker in the forest.",
                'Kodama Lord',
                None)
        self.move_unit('Kodama Lord',(14,0))
        self.kill_unit('Kodama Lord')
        self.center_on('Youmu')
        
        self.set_cursor_state(True)
        self.set_stats_display(True)
        
        
class MarisaHouseMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [ArrivalTrigger((19, 6, 3, 3), 1, 'Marisa')]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        If Marisa arrives at her house, she gets a fully charged Master Spark and 900 SC
        """

        self.say("Looks like we got here before they did.",
                'Marisa',
                'Marisa')
        self.say("Okay, now it's time to tear right through them with my signature Master Spark!",
                'Marisa',
                'Marisa')
        self.set_spirit_charge('Marisa', 900)
        self.say("Marisa's SC is maxed out!",
                None,
                None)
        self.say("Master Spark Spell Card fully charged!",
                None,
                None)
        self.set_spell_uses('Marisa','Master Spark', 5)
 
        
class NorthTreasureMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [ArrivalTrigger((15, 12, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        self.say("Buried at the foot of the cherry tree is a small rusted box.",
                None,
                None)
        self.say("Acquired Treasure Item: Rain Tear!",
                None,
                None)
        self.add_item('treasure','synth_water',1)
       
class SouthTreasureMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [ArrivalTrigger((6, 6, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        self.say("Buried at the foot of the cherry tree is a small rusted box.",
                None,
                None)
        self.say("Acquired Treasure Item: Ancient Amber!",
                None,
                None)
        self.add_item('treasure','synth_wood',1)
       
class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):

        self.set_cursor_state(False)
        self.set_stats_display(False)

        self.remove_all_enemies()
        self.stop_music()
        self.set_unit_pos('Youmu',(18,7))
        self.set_unit_pos('Ran',(19,7))
        self.set_unit_pos('Chen',(19,8))
        self.set_unit_pos('Marisa',(19,6))
        self.set_unit_pos('Wriggle',(16,7))

    def execute(self):

        # Conversation with Wriggle and hero party
        self.center_on('Wriggle')
        self.say("Okay! Okay! I surrender!",
                'Wriggle',
                'Wriggle')
        self.say("Mind telling us what you were doing with those things?",
                'Marisa',
                'Marisa')
        self.say("There's not a lot that I'm aware of. The youkai behind this is an ancient forest spirit that's become active suddenly again, retaking the forest from those who she thinks would harm it.",
                'Wriggle',
                'Wriggle')
        self.say("That explains targeting Marisa.",
                'Youmu',
                'Youmu')
        self.say("I try my best not to blow up the forest...",
                'Marisa',
                'Marisa')
        self.say("She's got support from a lot of the animal youkai. Bugs like me and beasts of the forest.",
                'Wriggle',
                'Wriggle')
        self.say("In a short time she's managed to organize a strong force that's got most of the youkai shaking in their boots. Even the fuzzballs have mostly fled the forest.",
                'Wriggle',
                'Wriggle')
        self.say("Look, I'm leaving the forest too. The forest will have its share of bugs, but they won't always need me.",
                'Wriggle',
                'Wriggle')
        self.say("Is there anything you can tell us that might help us find our masters?",
                'Youmu',
                'Youmu')
        self.say("Your masters?  Well, there were a lot of ghosts this morning...",
                'Wriggle',
                'Wriggle')
        self.say("Did you encounter Milady Yuyuko?",
                'Youmu',
                'Youmu')
        self.say("The only thing I found aside from all those flittering spirits would be this.",
                'Wriggle',
                'Wriggle')
        self.say(" Ah, the Lantern of Souls!",
                'Youmu',
                'Youmu')

        self.add_item('treasure','broken_lantern',1)
        self.say("Finally, a clue!",
                'Chen',
                'Chen')
        self.say("Which part of the forest did you find it?",
                'Marisa',
                'Marisa')
        self.say("Around here...",
                'Wriggle',
                'Wriggle')
        self.say("Then, we're on the right track, right?",
                'Chen',
                'Chen')
        self.say("Actually, they coulda gone any-which-way from here.",
                'Marisa',
                'Marisa')
        self.say("They're likely long gone. There was a big fight here last night. This got knocked out from the scene, and I picked it up.",
                'Wriggle',
                'Wriggle')
        self.say("Anyways, the walking tree youkai are active at night too and the more aggressive ones will be on the hunt for you. You four'd best get out of here.",
                'Wriggle',
                'Wriggle')
        self.say("I agree. By sunset, they'll likely have gotten us surrounded. For now, let's retreat.",
                'Ran',
                'Ran')
        self.say("But where to next? We need to know where our masters might have gone after that fight.",
                'Youmu',
                'Youmu')
        self.say("Let's see if we can find out first about this ancient tree spirit that's causing the mess. Those two are likely on the same trail.",
                'Marisa',
                'Marisa')
        self.say("I don't know too much about the forest's past, but I bet those history buffs in the human village do. Maybe they'll give you a clue as to where to search next.",
                'Marisa',
                'Marisa')
        self.say("Marisa, you should probably grab whatever items you need from your house. There's not going to be much left of this place once the other youkai arrive.",
                'Ran',
                'Ran')
        self.say("Right.",
                'Marisa',
                'Marisa')

        # Marisa goes back to her house to get stuff
        self.move_unit('Marisa', (20, 7))
        self.startle('Marisa')
        self.move_unit('Marisa', (19, 6))
        
        self.say("Well, I'm going now. The rest of the gang is waiting for me.",
                'Wriggle',
                'Wriggle')
 
        self.move_unit('Wriggle', (16,0))
        self.set_cursor_state(True)
        self.set_stats_display(True)