from lostsky.worldmap.event import ConversationEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent 

class Mission(ConversationEvent):
    
    def __init__(self):
        # Event Data
        name = 'Onward to the Village'
        location = 'Western Village Path'
        id_string = 'CH1ST4'
        prereqs = ['CH1ST3']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "Just before sunset, I spotted Reimu and Rinnosuke fleeing together from the Forest of Magic. Rinnosuke had his whole shop trampled to bits by those walking trees and was carrying what he could salvage in a small wooden box. They were both headed for the human village."
        
        ConversationEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch1st4.txt'
        mission_type = 'conversation'
        objective = None
        
        deploy_data = {'enable':False,
                       'max_units':None,
                       'preset_units':None,
                       'boxes':[]
                       }
        
        reward_list = []
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Rinnosuke',
                                'unit_name': 'Rinnosuke',
                                    'level': 11},
                          ]
        
        initial_spells = {}
        
        initial_traits = {}
        
        initial_ai_states = {}
        
        initial_locations = {'Ran':(4,18),
                             'Chen':(4,20),
                             'Youmu':(5,19),
                             'Marisa':(4,19),
                             
                             'Rinnosuke':(9,19),
                             }
        
        reserve_units = []#[list of unit names to deploy later in mission]
        
        all_landmarks = []
    
        required_starters = ['Youmu','Chen','Ran','Marisa']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = []
        required_survivors = []
        post_mission_MAE = None
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Prologue event 
        """

        self.set_cursor_state(False)
        self.set_stats_display(False)

        # Use the sunset color overlay
        self.set_bg_overlay('Sunset')
        self.play_music('event01')
        self.add_to_party('Reimu')
        
        self.set_unit_pos('Reimu',(8,19))
        self.assign_spell('Reimu','Holy Amulet')
        self.assign_spell('Reimu','Fantasy Seal')
        self.assign_spell('Reimu','Weakening Amulet')
        self.assign_spell('Reimu','Barrier Buster')
        self.center_on('Marisa')
 
        self.say("Reimu!",
                'Marisa',
                'Marisa')
        self.startle('Marisa')
        self.say("Aren't you late to the party?",
                'Marisa',
                'Marisa')
        self.say("I was making sure Rinnosuke got out of the forest safely since I was over there when this whole commotion started.",
                'Reimu',
                'Reimu')
        self.say("Geez, they kicked you out too?",
                'Marisa',
                'Marisa')
        self.center_on('Rinnosuke')
        self.say("Unfortunately, my antique shop's been reduced to a pile of rubble.",
                'Rinnosuke',
                'Rinnosuke')
        self.say("What are those three doing with you, Marisa?",
                'Reimu',
                'Reimu')
        
        # Fade to black as they tell their story
        self.fade_to_color('black',0.5)
        self.pause(0.5)
        self.fade_from_color('black',0.5)
 
        self.center_on('Reimu')
        self.say("Knowing Yukari, I wouldn't be surprised if she knew exactly what was going on or had a hand in this.",
                'Reimu',
                'Reimu')
        self.say(" I don't think so. She isn't the type to stir up trouble that upsets the balance this badly.",
                'Ran',
                'Ran')
        self.say("No, I'm sure that's exactly the kind of thing she would do to keep herself amused.",
                'Reimu',
                'Reimu')
        self.say("Hey, Kourin, we found this earlier today, can you tell us what it does?",
                'Marisa',
                'Marisa')
        
        self.show_image('fire_crystal','fire_crystal.png',(350,210))
        self.say("It's called a Lava Rock, and it's used in the synthesis of spells. I'm sure with your knowledge of magic, you can figure out how to use it to craft your own spells.",
                'Rinnosuke',
                'Rinnosuke')
        self.hide_image('fire_crystal')
        self.say("So what are your plans, Mr. Morichika?",
                'Chen',
                'Chen')
        self.say("Hopefully, set up shop in the village. I'd be willing to make trades for any useful items for antiques that you might happen to bring in.",
                'Rinnosuke',
                'Rinnosuke')
        self.say("And I'll trade extra fine items for some of the artifacts from my old shop that are now guaranteed to be scattered around the forest and beyond after all that chaos.",
                'Rinnosuke',
                'Rinnosuke')
        self.say("Well, looks like we have quite a party.",
                'Reimu',
                'Reimu')
        self.say(" You want to join us? Think of all the donations you'll get if you have a hand in solving the Great Kodama Lord Mystery.",
                'Marisa',
                'Marisa')
        self.say("When you put it that way... All right, I'm in.",
                'Reimu',
                'Reimu')
        self.say("Reimu joins the party!",
                None,
                None)
        self.say("Spell Synthesis is now available!",
                None,
                None)
        self.say("Be sure to check out the Spell Synthesis Tutorials if you don't know how to use it.",
                None,
                None)
        self.say("We should consider using Spell Synthesis to replenish our spell supplies.",
                'Youmu',
                'Youmu')
        self.say("Fireball, Dagger Throw, Leaf Crystal, and Holy Amulet can now be synthesized!",
                None,
                None)
        self.add_recipe('Fireball')
        self.add_recipe('Dagger Throw')
        self.add_recipe('Leaf Crystal')
        self.add_recipe('Holy Amulet')
        self.say("Item Trading is now available!",
                None,
                None)
        self.say("Shall we head on to the human village, then?",
                'Reimu',
                'Reimu')
        
        # Hide all units except for Ran and Youmu
        self.fade_to_color('black',0.5)
        self.set_unit_pos('Marisa',(-1,-1))
        self.set_unit_pos('Reimu',(-1,-1))
        self.set_unit_pos('Chen',(-1,-1))
        self.set_unit_pos('Rinnosuke',(-1,-1))
        self.fade_from_color('black',0.5)
        
        # Youmu and Ran's private conversation
        self.move_unit('Youmu',(8,19))
        self.move_unit('Ran',(7,18))
        self.move_unit('Youmu',(9,19))
        self.pause(1.0)
        self.move_unit('Youmu',(7,19))
        self.center_on('Youmu')
        
        self.say("Hey, Ran...",
                'Youmu',
                'Youmu')
        self.say("You seem a little troubled. What's wrong?",
                'Ran',
                'Ran')
        self.say("Ran, can we really do this? I don't know if we'll be able to make it through this crisis.",
                'Youmu',
                'Youmu')
        self.say("Youmu...",
                'Ran',
                'Ran')
        self.move_unit('Ran',(7,18))
        self.say("So much has already happened today. Lady Yuyuko's gone... For Reimu and Marisa, this is stuff they tackle every day. I don't know if I can do this.",
                'Youmu',
                'Youmu')
        self.say("This is the first chance we've had to catch our breaths. ",
                'Ran',
                'Ran')
        self.say("I'm afraid of what's going to happen if we don't succeed. ",
                'Youmu',
                'Youmu')
        self.say("Youmu, I'm not going to pretend the journey ahead is going to be easy. It'll likely be so difficult that there will be times when you want to turn around. But we have to have the courage and strength to go on. Not only for Lady Yuyuko and Lady Yukari, but also for Gensokyo.",
                'Ran',
                'Ran')
        self.say("What do you do in those times when you need to turn around, Ran? Am I strong enough?",
                'Youmu',
                'Youmu')
        self.say("Youmu, we've already started the journey. We've found clues that'll help us along the adventure's way. Strength... That's something you gain along the way.",
                'Ran',
                'Ran')
        self.say("After all, what's the purpose of an adventure if it's nothing more than a pleasant flower viewing? We'll make it, Youmu. We'll be fine.",
                'Ran',
                'Ran')
        self.say("Ran... Thank you...",
                'Youmu',
                'Youmu')
        self.set_cursor_state(True)
        self.set_stats_display(True)
        self.done = True