from lostsky.worldmap.event import BattleEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, TurnNumTrigger, UnitAliveTrigger

class Mission(BattleEvent):
    
    def __init__(self):
        # Event Data
        name = 'To the Aid of Alice'
        location = "Alice's House"
        id_string = 'CH2SQ2'
        prereqs = ['CH2SQ1']
        auto_signup = True
        cancellable = True
        show_rewards = False
        desc = "The Kodama Lord Kotone marches towards Alice's house in the Forest of Magic. Will our heroes be able to stop her in time?"
        
        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch2sq2.txt'
        mission_type = 'battle'
        objective = {'type':'Defend and Defeat Boss',
                     'target':'Kotone',
                     'location_box': [21,13,3,2],
                     'location_name': 'Alice\'s House',
                     'desc':'Defeat Kotone and prevent any enemy units from reaching the (3x2) region about Alice\'s house'
                     } 
        
        deploy_data = {'enable':True,
                       'max_units':9,
                       'preset_units':{'Alice':(22,13),
                                       'Marisa':(21,13)},
                       'boxes':[(16,15,3,3), (25,15,3,3)]
                       }
        reward_list = [('spell_action','Feather Pin'),
                       ('treasure','007_spellbook')
                      ]
                
        # Enemy Unit Data
        enemy_unit_data = [

                            # Initial enemy group
                            {'template_name': 'Kotone',
                                'unit_name': 'Kotone',
                                    'level': 9
                                },
                            {'template_name': 'Fairy',
                                'unit_name': 'Fairy A',
                                    'level': 6
                                },

                            {'template_name': 'Fairy',
                                'unit_name': 'Fairy B',
                                    'level': 6
                                },

                            {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree A',
                                    'level': 6
                                },

                            # Three Dolls for the intro scene
                            {'template_name': 'Doll',
                                'unit_name': 'Doll A',
                                    'level': 6
                            },
                            {'template_name': 'Doll',
                                'unit_name': 'Doll B',
                                    'level': 6
                            },
                            {'template_name': 'Doll',
                                'unit_name': 'Doll C',
                                    'level': 6
                            },


                            # Squad 1 - Close units approaching from the east
                            {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree B',
                                    'level': 5
                            },
                            {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree C',
                                    'level': 5
                            },
                            {'template_name': 'Fairy',
                                'unit_name': 'Fairy C',
                                    'level': 6
                                },

                            {'template_name': 'Fairy',
                                'unit_name': 'Fairy D',
                                    'level': 6
                                },
                            {'template_name': 'Healer Fairy',
                                'unit_name': 'Healer Fairy A',
                                    'level': 6
                                },

                            # Squad 2 - Close units approaching from the west.
                            {'template_name': 'Fairy',
                                'unit_name': 'Sealing Fairy',
                                    'level': 6
                                },
                            {'template_name': 'Fairy',
                                'unit_name': 'Dazzling Fairy',
                                    'level': 6
                                },
                            {'template_name': 'Fairy',
                                'unit_name': 'Ninja Fairy',
                                    'level': 6
                                },
                            {'template_name': 'Fairy',
                                'unit_name': 'Fairy Captain',
                                    'level': 7
                                },

                            # Squad 3 - Firefly unit from the northwest

                            {'template_name': 'Firefly',
                                'unit_name': 'Firefly A',
                                    'level': 5
                                },
                            {'template_name': 'Firefly',
                                'unit_name': 'Firefly B',
                                    'level': 5
                                },
                            {'template_name': 'Firefly',
                                'unit_name': 'Firefly C',
                                    'level': 5
                                },
                            {'template_name': 'Firefly',
                                'unit_name': 'Firefly D',
                                    'level': 5
                                },

                            # Kotone's Squad

                            {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree D',
                                    'level': 7
                            },
                            {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree E',
                                    'level': 7
                            },
                            {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree F',
                                    'level': 7
                            },
                            {'template_name': 'Healer Fairy',
                                'unit_name': 'Healer Fairy B',
                                    'level': 6
                                },

                            ]
        
        initial_spells = {'Fairy A':['Fireball'],
                          'Fairy B':['Fireball'],
                          'Fairy C':['Holy Amulet'],
                          'Fairy D':['Holy Amulet'],
                          'Dazzling Fairy':['Shimmering Stars'],
                          'Sealing Fairy':['Spirit Break'],
                          'Ninja Fairy':['Feather Pin'],
                          'Fairy Captain':['Holy Amulet'],

                          'Walking Tree A':['Leaf Crystal'],
                          'Walking Tree B':['Leaf Crystal'],
                          'Walking Tree C':['Leaf Crystal'],
                          'Walking Tree D':['Leaf Crystal'],
                          'Walking Tree E':['Leaf Crystal'],
                          'Walking Tree F':['Leaf Crystal'],
                          'Healer Fairy A':['Healing Drop'],
                          'Healer Fairy B':['Healing Drop'],

                          'Kotone':['Fireball'],

                          'Firefly A':['Poison Dust'],
                          'Firefly B':['Poison Dust'],
                          'Firefly C':['Poison Dust'],
                          'Firefly D':['Poison Dust'],


                            }
        initial_traits = {'Healer Fairy A':['Move+ Lv.1','Flight'],
                          'Healer Fairy B':['Move+ Lv.1','Flight'],
                          'Firefly A':['Flight'],
                          'Firefly B':['Flight'],
                          'Firefly C':['Flight'],
                          'Firefly D':['Flight'],}
        initial_ai_states = {'Kotone':'Attack',
                             'Fairy A':'Attack',
                             'Fairy B':'Attack',
                             'Fairy C':'Attack',
                             'Fairy D':'Attack',
                             'Walking Tree A':'Attack',
                             'Walking Tree B':'Attack',
                             'Walking Tree C':'Attack',
                             'Walking Tree D':'Attack',
                             'Walking Tree E':'Attack',
                             'Walking Tree F':'Attack',

                             'Ninja Fairy': 'Pursuit',
                             'Sealing Fairy': 'Pursuit',
                             'Dazzling Fairy': 'Pursuit',
                             'Fairy Captain': 'Pursuit',
                             
                             'Firefly A': 'Pursuit',
                             'Firefly B': 'Pursuit',
                             'Firefly C': 'Pursuit',
                             'Firefly D': 'Pursuit',

                             'Healer Fairy A': 'HealerStandby',
                             'Healer Fairy B': 'HealerStandby'



                            }
        initial_locations = {'Fairy A':(21,16),
                             'Fairy B':(23,16),
                             'Walking Tree A':(22,16),
                             'Kotone':(22,15),

                             'Youmu': (13,27),
                             'Marisa': (15,25),
                             'Ran': (12,28),
                             'Chen': (13,28),
                             'Reimu': (14,27),
                             'Keine': (14,28),

                             }
        reserve_units = ['Doll A', 'Doll B', 'Doll C',
                         'Fairy C',
                         'Fairy D',
                         'Walking Tree B',
                         'Walking Tree C',
                         'Healer Fairy A',

                         'Fairy Captain',
                         'Sealing Fairy',
                         'Ninja Fairy',
                         'Dazzling Fairy',

                         'Walking Tree D',
                         'Walking Tree E',
                         'Walking Tree F',
                         'Healer Fairy B',

                         'Firefly A',
                         'Firefly B',
                         'Firefly C',
                         'Firefly D',


                         ]#[list of unit names to deploy later in mission]
        all_landmarks = [{'name':'Alice\'s House',
                          'id_string':'house_2',
                          'location':(22,13)},
                         ]
    
        required_starters = ['Youmu', 'Chen', 'Ran', 'Marisa', 'Reimu', 'Keine']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [Turn5MAE()]
        required_survivors = ['Youmu', 'Chen', 'Ran', 'Marisa', 'Reimu', 'Keine', 'Kotone', 'Alice']
        post_mission_MAE = PostMissionMAE()
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Prologue event 
        """
        self.set_cursor_state(False)
        self.set_stats_display(False)

        # Kotone proposes an alliance with Alice

        self.center_on('Kotone')

        self.move_unit('Kotone', (22,14))
        self.say("... and as a representative of Fuyuhana Touji, Kodama Lord of the Forest of Magic, I ask that you join our cause.",
                'Kotone',
                'Kotone')
        self.emote('Kotone', 'dotdotdot')
        self.say("We did check if someone's home right?",
                'Fairy A',
                'Fairy')
        self.say("I saw something move in the window over there. I know you're in there! Would you come out and give a reply already?",
                'Kotone',
                'Kotone')
        self.emote('Kotone', 'scribble')
        self.say("There are no ways out. We have the house surrounded.",
                'Kotone',
                'Kotone')
        self.say("Walking Tree! We'll have to show her we mean business. Attack through that window!",
                'Kotone',
                'Kotone')

        self.play_music('battle01')

        # Marisa charges into battle

        self.center_on('Marisa')
        self.say("Looks like we got here right in time!",
                'Marisa',
                'Marisa')
        self.move_unit('Marisa', (19,18))
        self.center_on('Marisa')
        self.say("Wait! Marisa!",
                'Ran',
                'Ran')
        self.move_unit('Marisa', (19,14))
        self.startle('Marisa')
        self.move_unit('Marisa', (22,14))

        # Marisa knocks Kotone aside
        self.fade_to_color('white', 0.1)
        self.fade_from_color('white', 0.1)
        self.move_unit('Kotone', (26,14))

        self.emote('Kotone', 'annoyed')
        self.say("You guys again?!",
                'Kotone',
                'Kotone')

        self.say("You thought you could go around and bully my friend, could you?!",
                'Marisa',
                'Marisa')
        self.emote('Marisa', 'questionmark')
        self.say("Ah... do you guys hear something?",
                'Marisa',
                'Marisa')

        # Three dolls deploy around Marisa
        self.say("The sound of something ticking.",
                'Fairy A',
                'Fairy')
        self.emote('Marisa', 'exclamation')
        self.deploy_unit('Doll A', (22,13))
        self.move_unit('Doll A', (21,14))
        self.deploy_unit('Doll B', (22,13))
        self.move_unit('Doll B', (23,14))
        self.deploy_unit('Doll C', (22,13))


        self.center_on('Doll A')
        self.say("3...",
                'Doll A',
                'Doll')

        self.say("Oh no. Oh no. This is bad!",
                'Marisa',
                'Marisa')


        self.center_on('Doll B')
        self.say("2...",
                'Doll B',
                'Doll')


        self.center_on('Doll C')
        self.say("1...",
                'Doll C',
                'Doll')

        # Alice makes her appearance
        self.add_to_party('Alice')
        self.assign_spell('Alice', 'Fireball')
        self.assign_spell('Alice', 'Artful Sacrifice')
        self.assign_trait('Alice', 'Magic+ Lv.1')
        self.assign_trait('Alice', 'Homing Attack')
        self.set_unit_pos('Alice', (23,23))

        self.center_on('Alice')

        self.say("Marisa, you idiot!",
                'Alice',
                'Alice')

        self.center_on('Marisa')

        # Dolls explode, taking themselves and a few nearby units out. Marisa is thrown aside.
        self.play_sfx('explode')
        self.fade_to_color('white', 1.5)
        self.set_hp('Marisa', 1)
        self.kill_unit('Doll A')
        self.kill_unit('Doll B')
        self.kill_unit('Doll C')

        self.kill_unit('Fairy A')
        self.kill_unit('Fairy B')
        self.kill_unit('Walking Tree A')

        self.fade_from_color('white', 0.5)
        self.move_unit('Marisa', (22,21))

        self.center_on('Marisa')

        self.emote('Marisa','scribble')
        self.move_unit('Alice', (21,22))
        self.emote('Marisa','scribble')
        self.startle('Alice')

        # Alice drags Marisa towards the rest of the party
        self.move_unit('Alice', (21,23))
        self.move_unit('Marisa', (21,22))

        self.move_unit('Alice', (20,23))
        self.move_unit('Marisa', (21,23))

        self.emote('Marisa','scribble')
        self.startle('Alice')

        self.move_unit('Alice', (19,23))
        self.move_unit('Marisa', (20,23))


        self.startle('Alice')
        self.say("Ah... Over here!",
                'Alice',
                'Alice')

        self.center_on('Keine')

        self.emote('Keine', 'exclamation')
        self.say("Youmu, I heard someone call for help over there!",
                'Keine',
                'Keine')

        # Player party regroups
        self.move_unit('Keine', (20, 24))

        self.move_unit('Youmu', (20, 26))
        self.move_unit('Ran', (19, 26))
        self.move_unit('Chen', (21, 26))
        self.move_unit('Reimu', (20, 27))

        self.center_on('Keine')

        self.startle('Marisa')
        self.say("Marisa, are you okay?!",
                'Youmu',
                'Youmu')
        self.say("Owowowowowowowowow!",
                'Marisa',
                'Marisa')
        self.say("Hold still, Marisa! You got thrown a huge distance by that blast.",
                'Keine',
                'Keine')
        self.set_hp('Marisa', 10)
        self.emote('Marisa', 'scribble')
        self.say("There. That should get you back on your feet.",
                'Keine',
                'Keine')
        self.say("This is so like you to just charge in without thinking. I had those exploding dolls set up as a trap for that Kodama Lord.",
                'Alice',
                'Alice')
        self.emote('Marisa', 'annoyed')
        self.say("Hey! We came to help you! I thought you were going to be in trouble.",
                'Marisa',
                'Marisa')
        self.say("I don't need your help! I could have taken Kotone all by myself.",
                'Alice',
                'Alice')
        self.say("See? I told you Alice could fend for herself.",
                'Reimu',
                'Reimu')
        self.say("Well, now that we're here, the best we can do is fight together.",
                'Chen',
                'Chen')

        self.say("What are you guys doing here, anyway? I never expected to see all of you in one place",
                'Alice',
                'Alice')
        self.say("It's quite a long story. Now, would you let us help you get out of here?",
                'Marisa',
                'Marisa')
        self.startle('Alice')
        self.say("We can't leave now! All my doll making materials are in there.",
                'Alice',
                'Alice')

        self.move_unit('Ran', (21, 24))
        self.say("Kotone is pulling her trees back to the forest's edge. We can take this chance to position ourselves to defend Alice's house while she gathers her stuff together.",
                'Ran',
                'Ran')
        self.say("Great idea!",
                'Youmu',
                'Youmu')

        # Party gathers at Alice's house
        self.fade_to_color('black',1.0)

        self.set_unit_pos('Alice', (22,13))
        self.set_unit_pos('Marisa', (21,13))
        self.set_unit_pos('Youmu', (22,14))

        self.set_unit_pos('Reimu', (21,14))
        self.set_unit_pos('Keine', (23,13))
        self.set_unit_pos('Ran', (23,14))
        self.set_unit_pos('Chen', (23,15))

        # Deploy all enemy units

        # Eastern Squad
        self.deploy_unit('Walking Tree B', (28,18))
        self.deploy_unit('Walking Tree C', (28,14))
        self.deploy_unit('Fairy C', (30,14))
        self.deploy_unit('Fairy D', (30,17))
        self.deploy_unit('Healer Fairy A', (31,17))

        # Western Squad
        self.deploy_unit('Fairy Captain', (3,15))
        self.deploy_unit('Ninja Fairy', (3,12))
        self.deploy_unit('Dazzling Fairy', (3,17))
        self.deploy_unit('Sealing Fairy', (1,15))

        # Kotone's Squad
        self.set_unit_pos('Kotone', (33, 2))
        self.deploy_unit('Walking Tree D', (32,2))
        self.deploy_unit('Walking Tree E', (32,3))
        self.deploy_unit('Walking Tree F', (33,3))
        self.deploy_unit('Healer Fairy B', (34,1))

        self.fade_from_color('black',1.0)

        self.center_on('Alice')

        self.say("Make sure none of them get near the house!",
                'Alice',
                'Alice')

        self.center_on('Kotone')
        self.say("Listen up! Our strategy is simple. Charge!",
                'Kotone',
                'Kotone')

        self.center_on('Fairy Captain')
        self.say("Kotone! We're sorry we're late. Most of our group was wiped out by... those guys!",
                'Fairy Captain',
                'Fairy')

        self.center_on('Marisa')
        self.say("Heh... We stomped your reinforcements before we got here. Hope that's okay.",
                'Marisa',
                'Marisa')

        self.center_on('Kotone')
        self.emote('Kotone', 'scribble')
        self.say("Argh! This is inexcusable!",
                'Kotone',
                'Kotone')
        self.say("Well... the situation is bad, but we have no choice but to press the attack!",
                'Kotone',
                'Kotone')
        self.say("We are the guardians of the forest! We won't turn back no matter what the odds are!",
                'Kotone',
                'Kotone')

        self.center_on('Alice')

        self.set_cursor_state(True)
        self.set_stats_display(True)

class Turn5MAE(MapActionEvent):

    def __init__(self):
        trigger_list = [
            TurnNumTrigger(5),
            UnitAliveTrigger('Kotone',True),
        ]
        MapActionEvent.__init__(self, trigger_list)

    def execute(self):
        self.center_on('Kotone')
        self.emote('Kotone', 'exclamation')
        self.deploy_unit('Firefly A', (2,4))
        self.deploy_unit('Firefly B', (5,5))
        self.deploy_unit('Firefly C', (7,4))
        self.deploy_unit('Firefly D', (11,4))
        self.center_on_coords((6,4))
        self.say("Ah good. The fireflies are here! They'll never see it coming.",
                 'Kotone',
                 'Kotone')
        self.center_on_coords((22,13))

class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.remove_all_enemies()

        self.set_unit_pos('Kotone', (33,2))

        self.set_unit_pos('Alice', (22,13))
        self.set_unit_pos('Marisa', (21,13))
        self.set_unit_pos('Youmu', (22,15))
        self.set_unit_pos('Reimu', (21,15))
        self.set_unit_pos('Keine', (20,14))
        self.set_unit_pos('Ran', (23,15))
        self.set_unit_pos('Chen', (24,14))


    def execute(self):
        self.center_on('Kotone')

        self.say("I barely made it out of there. Ayaka is NOT going to take this well at all!",
                'Kotone',
                'Kotone')
        self.move_unit('Kotone', (33,-1))
        self.center_on('Alice')

        self.say("I see. Yuyuko is missing and you think all this commotion in the forest has to do with her?",
                'Alice',
                'Alice')

        self.say("The circumstances under which our masters disappeared point to that.",
                'Ran',
                'Ran')

        self.say("Did you hear anything about them?",
                'Youmu',
                'Youmu')

        self.say("Yeah. It seems that I'm the last of the holdouts in the forest. Everyone else has either fled or joined with the Kodama Lords.",
                'Alice',
                'Alice')

        self.say("So what'll you do now, Miss Alice? Are you tagging along with us?",
                'Chen',
                'Chen')

        self.emote('Alice', 'questionmark')

        self.say("We'd love to have you with us.",
                'Marisa',
                'Marisa')

        self.say("I'm not sure yet. Solving Gensokyo's crises? Not really what I do every day.",
                'Alice',
                'Alice')

        self.say("Come on! Remember all the fun we had together last time?",
                'Marisa',
                'Marisa')

        self.emote('Alice', 'dotdotdot')

        self.say("Okay, fine. I do owe you guys, and I'd love to get back at the Kodama Lords.",
                'Alice',
                'Alice')

        self.say("Great! Welcome aboard!",
                'Marisa',
                'Marisa')

        self.say("It'll be really helpful to have you fight alongside us, Alice.",
                'Youmu',
                'Youmu')


        self.say("But let me be clear. I'm not doing this so I can hang around with you, Marisa!",
                'Alice',
                'Alice')

        self.center_on('Reimu')
        self.move_unit('Reimu', (21,16))
        self.move_unit('Youmu', (21,15))

        self.say("Psst. Youmu.",
                'Reimu',
                'Reimu')

        self.say("There they go again with their little lovers quarrel.",
                'Reimu',
                'Reimu')

        self.say("At least having the two of them should make things lively.",
                'Youmu',
                'Youmu')


