from lostsky.worldmap.event import BattleEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, TurnNumTrigger, ArrivalTrigger, UnitAliveTrigger

class Mission(BattleEvent):
    
    def __init__(self):
        # Event Data
        name = 'Lost Sky Crisis'
        location = 'Graveyard'
        id_string = 'CH2ST3'
        prereqs = ['CH2ST2']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "A girl named Asa has gone missing from the human village! The people say that she's a strange person, able to sense the presence of spirits around her and comunicate with them. She was last sighted in the graveyard speaking with the ghost of someone who was buried there. One of the humans in the village even said they saw a Kodama Lord lurking about."
        
        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch2st3v2.txt'
        mission_type = 'battle'
        objective = {'type':'Defeat Boss',
                     'target':'Miu',
                     'desc':'Defeat Miu to escape!'
                     } 
        
        deploy_data = {'enable':True,
                       'max_units':8,
                       'preset_units':{},
                       'boxes':[(6,12,2,2), (17,12,2,2)]
                       }
        reward_list = [('spell_action','Rice Cake'),
                       ('treasure','synth_fire'),
                       ('treasure','synth_water'),
                       ('treasure','synth_earth')
                       ]
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Fuyuhana',
                                'unit_name': 'Fuyuhana',
                                    'level': 30},
                           {'template_name': 'Asa',
                                'unit_name': 'Asa',
                                    'level': 6},
                                    
                           # Kodama Lord Kotone's group
                           {'template_name': 'Kotone',
                                'unit_name': 'Kotone',
                                    'level': 9},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy D',
                                    'level': 6},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy E',
                                    'level': 6},
                           {'template_name': 'Healer Fairy',
                                'unit_name': 'Healer Fairy A',
                                    'level': 5},
                           
                           # Kodama Lord Ayaka's group
                           {'template_name': 'Ayaka',
                                'unit_name': 'Ayaka',
                                    'level': 9},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree A',
                                    'level': 5},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree B',
                                    'level': 5},
                                    
                           # Kodama Lord Haruna's Group
                           {'template_name': 'Haruna',
                                'unit_name': 'Haruna',
                                    'level': 9},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy A',
                                    'level': 5},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy B',
                                    'level': 5},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy C',
                                    'level': 5},
                                    
                                    
                           # Central Island Group
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly A',
                                    'level': 4},       
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly B',
                                    'level': 4}, 
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree C',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree D',
                                    'level': 4},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree E',
                                    'level': 4},
                                    
                           # Kodama Lord Miu's exit guard group
                           {'template_name': 'Miu',
                                'unit_name': 'Miu',
                                    'level': 7},
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly C',
                                    'level': 4},       
                           {'template_name': 'Firefly',
                                'unit_name': 'Firefly D',
                                    'level': 4}, 
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree F',
                                    'level': 5},
                           {'template_name': 'Healer Fairy',
                                'unit_name': 'Healer Fairy B',
                                    'level': 5},
                                    
                           # Reinforcement group
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy F',
                                    'level': 4},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy G',
                                    'level': 4},
                                    
                          ]
        
        initial_spells = {'Fuyuhana':['Leaf Crystal'],
                          'Kotone':['Fireball'],
                          'Ayaka':['Fireball'],
                          'Haruna':['Holy Amulet'],
                          'Miu':['Holy Amulet'],
 
                          'Walking Tree A':['Leaf Crystal'],
                          'Walking Tree B':['Leaf Crystal'],
                          'Walking Tree C':['Leaf Crystal'],
                          'Walking Tree D':['Leaf Crystal'],
                          'Walking Tree E':['Leaf Crystal'],
                          'Walking Tree F':['Leaf Crystal'],
 
                          
                          'Fairy A':['Fireball'],
                          'Fairy B':['Fireball'],
                          'Fairy C':['Holy Amulet'],
                          'Fairy D':['Holy Amulet'],
                          'Fairy E':['Holy Amulet'],
                          'Fairy F':['Dagger Throw'],
                          'Fairy G':['Dagger Throw'],
                          
                          'Healer Fairy A':['Healing Drop'],
                          'Healer Fairy B':['Healing Drop'],
                          
                          'Firefly A':['Fireball'],
                          'Firefly B':['Poison Dust'],
                          'Firefly C':['Poison Dust'],
                          'Firefly D':['Fireball']
                         
                          }
        initial_traits = {'Healer Fairy A':['Move+ Lv.1','Flight'],
                          'Healer Fairy B':['Move+ Lv.1','Flight'],
                          'Firefly A':['Flight'],
                          'Firefly B':['Flight'],
                          'Firefly C':['Flight'],
                          'Firefly D':['Flight'],
 
                          
                          'Ayaka':['Focused Movement','Agility+ Lv.3'],
                          'Haruna':['Focused Movement','Defense+ Lv.3'],
                          'Miu':['Flight']
                          }
        initial_ai_states = {'Kotone':'Pursuit',
                             'Ayaka':'Pursuit',
                             'Haruna':'Pursuit',
                             'Miu':'Defend',
                             
                             'Walking Tree A':'Pursuit',
                             'Walking Tree B':'Pursuit',
                             'Walking Tree C':'Attack',
                             'Walking Tree D':'Attack',
                             'Walking Tree E':'Attack',
                             'Walking Tree F':'Attack',
                             'Fairy A':'Pursuit',
                             'Fairy B':'Pursuit',
                             'Fairy C':'Pursuit',
                             'Fairy D':'Pursuit',
                             'Fairy E':'Pursuit',
                             'Fairy F':'Attack',
                             'Fairy G':'Attack',
                             
                             'Firefly A':'Attack',
                             'Firefly B':'Attack',
                             'Firefly C':'Attack',
                             'Firefly D':'Attack',
                             
                             'Healer Fairy A':'HealerStandby',
                             'Healer Fairy B':'HealerStandby'
                             
                             }
        initial_locations = {'Keine':(12,6),
                             'Youmu':(12,7),
                             'Ran':(11,7),
                             'Chen':(10,7),
                             'Reimu':(13,7),
                             'Marisa':(14,7),
                             
                             'Asa':(12,4),
                             }
        reserve_units = ['Fuyuhana',
                         'Kotone',
                         'Ayaka',
                         'Haruna',
                         'Miu',
                         'Walking Tree A',
                         'Walking Tree B',
                         'Walking Tree C',
                         'Walking Tree D',
                         'Walking Tree E',
                         'Walking Tree F',
                         'Fairy A',
                         'Fairy B',
                         'Fairy C',
                         'Fairy D',
                         'Fairy E',
                         'Fairy F',
                         'Fairy G',
                         'Firefly A',
                         'Firefly B',
                         'Firefly C',
                         'Firefly D',
                         'Healer Fairy A',
                         'Healer Fairy B',
                         ]
        
        
        all_landmarks = [{'name':'GS1',
                          'id_string':'tombstone',
                          'location':(4,10)},
                          {'name':'GS2',
                          'id_string':'tombstone',
                          'location':(6,10)},
                          {'name':'GS3',
                          'id_string':'tombstone',
                          'location':(6,8)},
                          {'name':'GS4',
                          'id_string':'tombstone',
                          'location':(4,8)},
                          {'name':'GS5',
                          'id_string':'tombstone',
                          'location':(4,6)},
                          {'name':'GS6',
                          'id_string':'tombstone',
                          'location':(6,6)},
                          {'name':'GS7',
                          'id_string':'tombstone',
                          'location':(11,6)},
                          {'name':'GS8',
                          'id_string':'tombstone',
                          'location':(13,6)},
                          {'name':'GS9',
                          'id_string':'tombstone',
                          'location':(11,8)},
                          {'name':'GS10',
                          'id_string':'tombstone',
                          'location':(13,8)},
                          {'name':'GS11',
                          'id_string':'tombstone',
                          'location':(11,10)},
                          {'name':'GS12',
                          'id_string':'tombstone',
                          'location':(13,10)},
                          {'name':'GS13',
                          'id_string':'tombstone',
                          'location':(18,6)},
                          {'name':'GS14',
                          'id_string':'tombstone',
                          'location':(20,6)},
                          {'name':'GS15',
                          'id_string':'tombstone',
                          'location':(18,8)},
                          {'name':'GS16',
                          'id_string':'tombstone',
                          'location':(20,8)},
                          {'name':'GS17',
                          'id_string':'tombstone',
                          'location':(18,10)},
                          {'name':'GS18',
                          'id_string':'tombstone',
                          'location':(20,10)},
                          
                          {'name':'Mini Shrine',
                          'id_string':'minishrine',
                          'location':(12,3)},
                          {'name':'CB0',
                          'id_string':'cherryblossom_tree',
                          'location':(11,3)},
                          {'name':'CB1',
                          'id_string':'cherryblossom_tree',
                          'location':(13,3)},
                          
                          {'name':'CB2',
                          'id_string':'cherryblossom_tree',
                          'location':(2,17)},
                          {'name':'CB3',
                          'id_string':'cherryblossom_tree',
                          'location':(20,25)},
                          
                          {'name':'Torii',
                          'id_string':'small_torii',
                          'location':(12,11)},
                         ]
    
        required_starters = ['Youmu','Chen','Ran','Marisa','Reimu','Keine']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [WestTreasureMAE(), EastTreasureMAE(), MiuReinforceMAE(),
                                AyakaHarunaMAE(), AyakaYoumuMAE()]
        required_survivors = ['Youmu', 'Ran', 'Chen', 'Marisa', 'Reimu', 'Keine',
                              'Mokou', 'Kotone', 'Miu', 'Haruna', 'Ayaka']
        post_mission_MAE = PostMissionMAE()
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Big encounter with Fuyuhana
        """
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.center_on('Asa')
        self.say("Is someone there?",
                'Asa',
                'Asa')
        self.say("Asa, what are you doing here?",
                'Keine',
                'Keine')
        self.say("Professor -oh. You're traveling with a ghost? No, not quite a ghost.",
                'Asa',
                'Asa')
        self.say("You can see my ghost half?",
                'Youmu',
                'Youmu')
        self.say("Even though she's blind, she can sense the presence of things that others don't normally see. Is that why you're in the cemetery?",
                'Keine',
                'Keine')
        self.say("Yes, there's a ghost here who asked for this tree branch. I think they share a connection.",
                'Asa',
                'Asa')
        self.say("That branch, bring it back to the forest, and allow my worries to be at peace.",
                '???',
                None)
        self.say("Are you crazy? The forest is no place for someone like her.",
                'Marisa',
                'Marisa')
        self.say("They're right. I can't go into the forest right now.",
                'Asa',
                'Asa')
        self.say("We'll just get her back to the village, and then-",
                'Marisa',
                'Marisa')
        self.startle('Youmu')
        self.play_music('battle04')
        self.say("Wait. There!",
                'Youmu',
                'Youmu')

        # Kotone is discovered by Youmu
        self.deploy_unit('Kotone', (11, 4))
        self.move_unit('Youmu', (11, 4))
        self.fade_to_color('white', 0.1)
        self.fade_from_color('white', 0.1)
        self.move_unit('Kotone', (11, 1))
        self.startle('Kotone')
        self.say("A Kodama Lord!",
                'Ran',
                'Ran')
        
        # Other party members move up
        self.move_unit('Keine', (13, 4))
        self.move_unit('Chen', (10, 4))
        self.move_unit('Ran', (11, 5))
        self.move_unit('Marisa', (14, 4))
        self.move_unit('Reimu', (13, 5))
 
        self.say("That ghost is the one behind this crisis, isn't she?",
                'Youmu',
                'Youmu')
        self.say("This is the best timing. I've been looking forward to fighting the ones who escaped from the forest. What shall we do, Lady Fuyuhana?",
                'Kotone',
                'Kotone')
        
        # Kodama Lord Fuyuhana makes an apperance and attacks
        self.deploy_unit('Fuyuhana', (12, 3))
        self.move_unit('Fuyuhana', (13, 3))
        self.say("We need to take her to the forest with us. I promised to show you the world that you were never able to see, Asa. Now's the time. Please join me.",
                'Fuyuhana',
                'Fuyuhana')
        self.say("Asa, get behind me.",
                'Keine',
                'Keine')
        self.say("Y... yes...",
                'Asa',
                'Asa')

        # Asa moves behind Keine
        self.move_unit('Reimu', (12, 5))
        self.move_unit('Asa', (13, 5))
        self.say("You lay one finger on her, and I'll erase you from existence!",
                'Keine',
                'Keine')
        self.say("I was once known as the master of the forest. My history predates yours and Gensokyo's.",
                'Fuyuhana',
                'Fuyuhana')
        self.say("Humans and youkai alike have defiled our precious home in the Forest of Magic, so the contract I had with your ancestors is severed.",
                'Fuyuhana',
                'Fuyuhana')
        self.say("Gods of rain and storms, skies and wind, hear my call! Invoke Spell Card! Ritual Sign - Gathering Clouds!",
                'Fuyuhana',
             'Fuyuhana')

        
        # Fuyuhana casts a spell and knocks Keine out of the way
        self.fade_to_color('white', 0.1)
        self.fade_from_color('white', 0.1)
        self.move_unit('Keine', (13, 7)) 
        self.move_unit('Fuyuhana', (13, 4))
 
        self.say("Ugh...",
                'Keine',
                'Keine')
        
        self.startle('Asa')
        self.say("Professor!",
                'Asa',
                'Asa')
        self.say("I'm sorry that I had to do that. We need to complete the part of the plan we agreed on. Asa, are you ready to go?",
                'Fuyuhana',
                'Fuyuhana')

        self.say("Um...",
                'Asa',
                'Asa')
        self.say("Ayaka and Haruna are on their way to help you keep them busy. I'll be taking Asa with me back to my tree.",
                'Fuyuhana',
                'Fuyuhana')

        # Fuyuhana teleports away 
        self.fade_to_color('white', 0.5)
        self.kill_unit('Fuyuhana')
        self.kill_unit('Asa')
        self.fade_from_color('white', 0.5)
        self.say("Asa!",
                'Keine',
                'Keine')
        
        self.fade_to_color('white', 0.5)
        self.show_image('clouds001', 'sky001.jpg', (0, 0))
        self.fade_from_color('white', 0.5)
 
        
        self.say("Ah! Look up in the sky!",
                'Chen',
                'Chen')
        
        # Show image of sky with clouds -> sky without clouds
        self.fade_to_color('white', 0.5)
        self.hide_image('clouds001')
        self.show_image('clouds002', 'sky002.jpg', (0, 0))
        self.fade_from_color('white', 0.5)

        self.say("The clouds... they've all disappeared!",
                'Keine',
                'Keine')

        self.fade_to_color('white', 0.5)
        self.hide_image('clouds002')
        self.fade_from_color('white', 0.5)

        self.say("That's quite a disappearing act.",
                'Marisa',
                'Marisa')
        self.say("Then let's show them off stage!",
                '???',
                None)
        
        # Mokou's appearance
        
        self.set_cursor_state(True)
        self.move_unit('Kotone', (12, 1))
        self.center_on('Kotone')
        self.add_to_party('Mokou')
        self.assign_spell('Mokou', 'Dagger Throw')
        self.assign_spell('Mokou', 'Flying Phoenix')
        self.assign_spell('Mokou', 'Rice Cake')
        self.assign_trait('Mokou', 'Regen Lv.1')
        self.assign_trait('Mokou', 'Homing Attack')
        self.set_unit_pos('Mokou', (8,0))
        
        self.show_animation('fire_spell', (11, 0))
        self.move_unit('Mokou', (12, 4))
        self.move_unit('Reimu', (13, 4))
        self.move_unit('Keine', (12, 5))
        
        self.say("Mokou!",
                'Keine',
                'Keine')
        self.say("That fool's going to set the all these trees on fire!",
                'Kotone',
                'Kotone')
        self.say("Fires arise and singe the earth to ashes! Immortal Firebird - Flying Phoenix!",
                'Mokou',
                'Mokou')
        self.show_animation('fire_spell', (12, 1))
        self.startle('Kotone')
        self.say("Aaaagh! I'm on fire! Help!",
                'Kotone',
                'Kotone')
 
        # Kotone's Party Deploys
        self.fade_to_color('white', 0.5)
        self.deploy_unit('Fairy D', (10, 3))
        self.deploy_unit('Fairy E', (14, 3))
        self.deploy_unit('Healer Fairy A', (12, 2))
        self.fade_from_color('white', 0.5)
        
        self.set_hp('Kotone', 1)
        self.startle('Healer Fairy A')
        
        self.say("Healing Fairy Corps, reporting for duty! How may I be of service?",
                'Healer Fairy A',
                'Healer Fairy')
        self.say("Just put out this fire!",
                'Kotone',
                'Kotone')
        self.say("But I have no water...",
                'Healer Fairy A',
                'Healer Fairy')
        self.say("Ack! You are all useless! Just heal these wounds or something with your magic.",
                'Kotone',
                'Kotone')
        self.say("Okay! One healing spell, coming right up!",
                'Healer Fairy A',
                'Healer Fairy')
        self.script_battle('Healer Fairy A', 
                           'Kotone',
                           {'lhs_hit': True,
                            'lhs_crit': False,
                            'lhs_equip': 0,
                            'rhs_hit': False,
                            'rhs_crit': False,
                            'rhs_equip': 0,
                            })
        self.say(" Argh! Is this all we have? Where are the reinforcements that were promised? I should have brought more trees with me.",
                'Kotone',
                'Kotone')
        self.say("Tough luck.",
                'Mokou',
                'Mokou')
        self.say("We have to go after Asa!",
                'Keine',
                'Keine')
        self.say("But what about the skies above? What does the disappearance of the clouds mean?",
                'Youmu',
                'Youmu')

        # Ayaka's Party Deploys        
        self.fade_to_color('white', 0.5)
        self.deploy_unit('Ayaka', (22, 6))
        self.deploy_unit('Walking Tree A', (21, 6))
        self.deploy_unit('Walking Tree B', (23, 6))
        self.deploy_unit('Fairy C', (22, 7))
        self.fade_from_color('white', 0.5)
        self.center_on('Ayaka')
        self.say("You are always so impatient Kotone.",
                'Ayaka',
                'Ayaka')
        self.say("Ayaka! You're here personally?",
                'Kotone',
                'Kotone')
 
        # Haruna's Party Deploys
        self.fade_to_color('white', 0.5)
        self.deploy_unit('Haruna', (4, 6))
        self.deploy_unit('Fairy A', (3, 7))
        self.deploy_unit('Fairy B', (5, 7))
        self.fade_from_color('white', 0.5)
        self.center_on('Haruna')
        self.emote('Haruna', 'zzz')
        self.emote('Ayaka', 'annoyed')
        
        self.say("Haruna!",
                'Ayaka',
                'Ayaka')
        self.emote('Haruna', 'exclamation')
        self.say("*Yawn* It's too early to be up... Haruna, Kodama Lord of the Northern Forests reporting.",
                'Haruna',
                'Haruna')
 
        self.fade_to_color('white', 0.5)

        # Central Island Group
        self.deploy_unit('Firefly A', (10, 22))
        self.deploy_unit('Firefly B', (15, 22))
        self.deploy_unit('Firefly C', (8, 27))
        self.deploy_unit('Firefly D', (17, 27))

        # Eastern Forest
        self.deploy_unit('Walking Tree C', (22, 24))
        # Western Forest
        self.deploy_unit('Walking Tree D', (3, 21))
        
        # Miu's Group
        self.deploy_unit('Miu', (13, 32))
        self.deploy_unit('Healer Fairy B', (13, 33))
        self.deploy_unit('Walking Tree E', (10, 33))
        self.deploy_unit('Walking Tree F', (16, 33))
        self.fade_from_color('white', 0.5)
        self.center_on('Miu')
 
        self.say("Um... um... I'm here too! Miu, Kodama Lord of the Red Maples. Please give me orders.",
                'Miu',
                'Miu')
        self.say("Miu? I thought you were supposed to stay behind and guard the forest.",
                'Kotone',
                'Kotone')
        self.say("Tch... they brought out the whole party?",
                'Marisa',
                'Marisa')
        self.center_on('Ayaka')
        self.say("We have been entrusted with the duty of protecting Fuyuhana's retreat. As guardians of the forest, we must absolutely not fail her.",
                'Ayaka',
                'Ayaka')
        self.center_on('Kotone')
        self.say("I won't, Ayaka. They'll regret trying to harm the forest.",
                'Kotone',
                'Kotone')
        self.center_on('Ayaka')
        self.say("More are on the way from the forest. However, let's defeat these termites before they can gnaw away at our still growing trunk.",
                'Ayaka',
                'Ayaka')
        self.center_on('Haruna')
        self.say("Haruna, I suppose I can count on you to corner them on the other side.",
                'Ayaka',
                'Ayaka')
        self.say("Of course. *yawn* I only wish you hadn't brought us out this early. I have barely gotten out of hibernation, and it has been centuries since I've been in a fight.",
                'Haruna',
                'Haruna')
        self.center_on('Miu')
        self.say("Miu, you may be the youngest of us, but we are counting on you as well. Don't let them escape to the south.",
                'Ayaka',
                'Ayaka')
        self.startle('Miu')
        self.say("Understood!",
                'Miu',
                'Miu')
        
        # Heroes plan their course of action
        self.center_on('Youmu')
        self.say("They've got us surrounded from all sides.",
                'Reimu',
                'Reimu')
        self.say("We've never seen so many Kodama Lords in one place before.",
                'Chen',
                'Chen')
        self.say("Who cares? We'll just blow through them like we've always done!",
                'Marisa',
                'Marisa')
        self.emote('Youmu', 'dotdotdot')
        self.say("Youmu?",
                'Ran',
                'Ran')

        # Everyone gathers around Youmu
        self.move_unit('Youmu', (12, 7))
        self.move_unit('Ran', (11, 7))
        self.move_unit('Reimu', (13, 7))
        self.move_unit('Chen', (11, 6))
        self.move_unit('Marisa', (13, 6))
        self.move_unit('Keine', (12, 8))
        self.move_unit('Mokou', (12, 6))

        self.say("Let's retreat for now.",
                'Youmu',
                'Youmu')
        self.say("What? Why?",
                'Marisa',
                'Marisa')
        self.say("You sensed it too, huh? There's something eerily dangerous about these four.",
                'Mokou',
                'Mokou')
        self.say("More reinforcements will be here soon. We can return behind Keine's veil and regroup.",
                'Ran',
                'Ran')
        self.say("What about Asa? We can't leave her behind. She knows about the human village so Fuyuhana will have its memory as well!",
                'Keine',
                'Keine')
        self.say("But it feels like we've been constantly retreating! What happened to the spirit of throwing caution to the wind and charging right in?",
                'Marisa',
                'Marisa')
        self.say("I have the feeling the Kodama Lords won't play nicely with one on one duels that we've been accustomed to.",
                'Reimu',
                'Reimu')
        self.say("If we are to win, we have to stick together.",
                'Ran',
                'Ran')
        self.say("So... Nobody's answered the obvious question. How are we going to get out of here?",
                'Reimu',
                'Reimu')
        self.emote('Chen', 'lightbulb')
        self.say("When mice are chased by cats, they split up. Maybe we can do the same.",
                'Chen',
                'Chen')
        self.say("They've surrounded us in this area. If we split up and head south, we should be able to make evade the brunt of their attack.",
                'Ran',
                'Ran')
        self.say("The one down there doesn't seem very experienced. Breaking through her should be easier.",
                'Youmu',
                'Youmu')
        self.say("As before, their lackeys are dependent upon the Kodama Lords for guidance. If we knock her out, we will be able to escape.",
                'Ran',
                'Ran')


        
        self.set_stats_display(True)
        
        
class WestTreasureMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [ArrivalTrigger((2, 17, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        self.say("Buried at the foot of the cherry tree is a small rusted box.",
                None,
                None)
        self.say("Acquired Treasure Item: Lava Rock!",
                None,
                None)
        self.add_item('treasure','synth_fire',1)
       
       
class EastTreasureMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [ArrivalTrigger((20, 25, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        self.say("Sitting at the side of the cherry tree is a cardboard box.",
                None,
                None)
        self.say("Acquired Treasure Item: Cell Phone!",
                None,
                None)
        self.add_item('treasure','002_cellphone',1)
       
       
class MiuReinforceMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [TurnNumTrigger(3),
                    # Ayaka must be still alive
                    UnitAliveTrigger('Ayaka', True)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        Ayaka sends reinforcements to back up Miu
        """
        self.set_stats_display(False)
        self.center_on('Ayaka')
        
        self.say("Miu, they're coming your way! Get ready!",
                'Ayaka',
                'Ayaka')
        
        self.center_on('Miu')
        self.say("Got it!",
                'Miu',
                'Miu')
        self.center_on('Ayaka')
        self.say("I'm sending some of our fairies to assist you. These guys are fast, but we'll catch up to them soon.",
                'Ayaka',
                'Ayaka')
        
        self.center_on('Miu')
        self.deploy_unit('Fairy G', (13,34))
        self.move_unit('Fairy G', (11, 33))
        self.deploy_unit('Fairy F', (13,34))
        self.move_unit('Fairy F', (15, 33))
 
        self.say("We'll not fail you, Lady Ayaka!",
                'Fairy',
                'Fairy')
 
        self.say("I have to hand it to them. They're more organized than any troublemakers we've encountered in the past.",
                'Reimu',
                'Reimu')
        self.set_stats_display(True)
        
        
class AyakaHarunaMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [TurnNumTrigger(5),
                    UnitAliveTrigger('Ayaka', True),
                    UnitAliveTrigger('Haruna', True)
                    ]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        Ayaka + Haruna converse about old times
        """
        self.set_stats_display(False)
        self.center_on('Ayaka')
        self.say("Are you okay, Ayaka?",
                'Haruna',
                'Haruna')
        self.say("As best as can be. They're tough, but we've got the strength in numbers.",
                'Ayaka',
                'Ayaka')
        self.center_on('Haruna')
        self.say("You know, it's been long time since we've seen battle. A lot has changed in this world.",
                'Haruna',
                'Haruna')
        self.center_on('Ayaka')
        self.say("Miu and Kotone wouldn't know it. Our world is a lot different from when we went to sleep. Many new youkai have shown up.",
                'Ayaka',
                'Ayaka')
        self.center_on('Haruna')
        self.say("Yeah... we've been cut off from the outside world. I wonder if we'll ever see our old companions again.",
                'Haruna',
                'Haruna')
        self.set_stats_display(True)
 
       
class AyakaYoumuMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [TurnNumTrigger(7),
                    UnitAliveTrigger('Ayaka', True),
                    UnitAliveTrigger('Youmu', True),
                    UnitAliveTrigger('Chen', True)
                    ]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        """
        Ayaka talks with Youmu during their fight 
        """
        self.set_stats_display(False)
        self.center_on('Ayaka')
        self.say("There you are!",
                'Ayaka',
                'Ayaka')
        self.center_on('Chen')
        self.say("Ah! These guys are tough.",
                'Chen',
                'Chen')
        self.center_on('Youmu')
        self.say("Hurry! We need to keep moving!",
                'Youmu',
                'Youmu')
        self.center_on('Ayaka')
        self.say("You're pretty fast and strong as well. I look forward to dueling with you.",
                'Ayaka',
                'Ayaka')
        self.set_stats_display(True)
        
class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):

        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.remove_all_enemies()
        self.set_unit_pos('Youmu', (13, 34))
        self.set_unit_pos('Ran', (12, 33))
        self.set_unit_pos('Chen', (12, 32))
        self.set_unit_pos('Reimu', (13, 33))
        self.set_unit_pos('Marisa', (13, 32))
        self.set_unit_pos('Mokou', (14, 33))
        self.set_unit_pos('Keine', (14, 32))


        self.set_unit_pos('Ayaka', (15, 8))
        self.set_unit_pos('Haruna', (9, 8))
        self.set_unit_pos('Miu', (12, 10))
        self.set_unit_pos('Kotone', (12, 6))

    def execute(self):

        # Heroes escape from the scene 
        self.center_on('Youmu')
        self.say("We made it!",
                'Youmu',
                'Youmu')
        self.say("Everyone all right?",
                'Keine',
                'Keine')
        self.say("A few bruises but I'm okay.",
                'Mokou',
                'Mokou')
        self.say("Let's hurry and get out of here then.",
                'Marisa',
                'Marisa')
        
        self.move_unit('Youmu', (13, 36))
        self.move_unit('Ran', (13, 36))
        self.move_unit('Chen', (13, 36))
        self.move_unit('Marisa', (13, 36))
        self.move_unit('Reimu', (13, 36))
        self.move_unit('Mokou', (13, 36))
        self.move_unit('Keine', (13, 36))

        # Kodana Lords talk about their defeat
        self.center_on('Miu')
        self.emote('Miu', 'scribble')
        
        self.say("I'm sorry. I couldn't stop them. They came at me too fast.",
                'Miu',
                'Miu')
        self.say("Miu, we should have supported you more in that battle. If only I hadn't been so far away.",
                'Kotone',
                'Kotone')
        self.say("It's my fault. I failed to fulfill my duty to the forest.",
                'Miu',
                'Miu')
        self.emote('Kotone', 'annoyed')
        self.say("Ayaka, why didn't you bring your rampaging trees? We could have really used them to stomp that shrine maiden and her companions!",
                'Kotone',
                'Kotone')
        self.emote('Ayaka', 'annoyed')
        self.move_unit('Haruna', (12, 8))
        self.say("That's enough. You both fought hard and bravely. We can't have everything go our way.",
                'Haruna',
                'Haruna')
        self.say("I would consider this a battle won. We forced them to retreat, and Gensokyo's clouds are all gathered above the Forest of Magic as planned.",
                'Ayaka',
                'Ayaka')
        self.say("Until Fuyuhana completes the rain making ceremony to strengthen the forest, it remains in danger so long as we are away from it.",
                'Haruna',
                'Haruna')
        self.emote('Kotone', 'questionmark')
        self.say("We're not going to go after them?",
                'Kotone',
                'Kotone')
        self.say("That's right. Miu, Kotone, return and protect the forest.",
                'Ayaka',
                'Ayaka')

        self.emote('Kotone', 'exclamation')
        self.emote('Miu', 'exclamation')
        self.say("At once!",
                'Miu and Kotone',
                None)
        
        
        self.move_unit('Kotone', (12, -1))
        self.move_unit('Haruna', (13, 8))
        self.move_unit('Miu', (12, -1))
        self.kill_unit('Kotone')
        self.kill_unit('Miu')
 
        self.say("These new Kodama Lords, I can see that working with them will be difficult.",
                'Ayaka',
                'Ayaka')
        self.say("As your mentor for so long, all I can say is to put your trust in them. We were in the same place at one time.",
                'Haruna',
                'Haruna')
        self.say("I suppose you're right.",
                'Ayaka',
                'Ayaka')
        self.say("Ayaka, a question: What's Fuyuhana's plan with the girl?",
                'Haruna',
                'Haruna')
        self.move_unit('Haruna', (12, 9))
        self.say("Asa is quite sensitive to ghosts and spirits. Fuyuhana will likely form a contract to possess her as a vessel. Otherwise, she can't remain far from her tree for long.",
                'Ayaka',
                'Ayaka')
 
        self.emote('Haruna', 'dotdotdot')
        self.say("I'm not quite sure if Asa knows what she's gotten herself into.",
                'Haruna',
                'Haruna')
        self.say("Haruna, I have a mission for you. Go to Youkai Mountain...",
                'Ayaka',
                'Ayaka')
        self.emote('Haruna', 'questionmark')
        self.say("The sanctuary?",
                'Haruna',
                'Haruna')
        self.say("Fuyuhana wants to make sure that that sacred sanctuary is safe.",
                'Ayaka',
                'Ayaka')
        self.say("I'll set out with a handful of fairies immediately.",
                'Haruna',
                'Haruna')
        self.say("Best of luck, Haruna.",
                'Ayaka',
                'Ayaka')
        
        self.stop_music()
        self.set_stats_display(True)
        self.set_cursor_state(False)
        