from lostsky.worldmap.event import ConversationEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent

class Mission(ConversationEvent):
    
    def __init__(self):
        # Event Data
        name = 'Legend of the Forest of Magic'
        location = 'Human Village'
        id_string = 'CH2ST4'
        prereqs = ['CH2ST3']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "Legend has it that Fuyuhana Touji was a great Kodama Lord that once led the trees in the Forest of Magic. Today's special event, a lecture by Akyu will take place on the Kodama Lords who have led the forest in the past. It seems boring. I usually sleep through these things."

        ConversationEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch2st2.txt'
        mission_type = 'conversation'
        objective = None
        deploy_data = {}
        reward_list = []
                
        # Enemy Unit Data
        enemy_unit_data = [ {'template_name': 'Akyu',
                                 'unit_name': 'Akyu',
                                     'level': 5 }, 
                          ]
        
        initial_spells = {}
        initial_traits = {}
        initial_ai_states = {}
        initial_locations = {'Akyu':(15,9),
                             'Keine':(15,8),
                             'Mokou':(14,8),
                             
                             'Youmu':(17,8),
                             'Ran':(18,10),
                             'Chen':(18,9),
                             
                             'Reimu':(17,9),
                             'Marisa':(17,10),
                             }
        reserve_units = []
        all_landmarks = [{'name':'Akyu\'s House',
                          'id_string':'house_1',
                          'location':(14,9)},
                          
                          {'name':'House 2',
                          'id_string':'house_1',
                          'location':(10,8)},
                          {'name':'House 3',
                          'id_string':'house_1',
                          'location':(14,14)},
                          {'name':'House 4',
                          'id_string':'house_2',
                          'location':(10,14)},
                          {'name':'House 5',
                          'id_string':'house_1',
                          'location':(5,8)},
                          {'name':'House 6',
                          'id_string':'house_2',
                          'location':(18,6)},
                          {'name':'House 7',
                          'id_string':'house_2',
                          'location':(24,6)},
                          {'name':'House 8',
                          'id_string':'house_1',
                          'location':(25,14)},
                          {'name':'House 9',
                          'id_string':'house_2',
                          'location':(5,6)},
                          
                          {'name':'CB1',
                          'id_string':'cherryblossom_tree',
                          'location':(10,9)},
                          {'name':'CB2',
                          'id_string':'cherryblossom_tree',
                          'location':(13,8)},
                         ]
    
        required_starters = ['Chen','Ran','Youmu','Marisa','Reimu','Keine','Mokou']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = []
        required_survivors = []
        post_mission_MAE = None
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Meeting with Akyu after the confrontation with the Kodama Lords
        """
        
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.play_music('event01')
        self.center_on('Marisa')
        self.startle('Marisa')
 
        self.say("That was close!",
                'Marisa',
                'Marisa')
        self.say("You guys just rushed back here! Are you all right?",
                'Akyu',
                'Akyu')
        self.say("It was an ambush! We had four Kodama Lords coming at us.",
                'Youmu',
                'Youmu')
        self.say("More importantly, what's so special about the girl that Fuyu-whatever captured?",
                'Reimu',
                'Reimu')
        self.say("Asa's always been a strange girl. She has the ability to speak with spirits as well as manipulate sunlight. Her power of sunlight must be being used to help strengthen the tree youkai.",
                'Keine',
                'Keine')
        self.say("Her sensitivity to the world of the dead must have made her an easy target for Fuyuhana.",
                'Youmu',
                'Youmu')
        self.say("There's another issue. Since the village hiding spell I cast doesn't affect the humans from there, the Kodama Lords will eventually know where to find our village.",
                'Keine',
                'Keine')
        self.say("Not to mention all the clouds vanished from the sky!",
                'Mokou',
                'Mokou')
        self.say("Then we must hurry to solve this crisis!",
                'Youmu',
                'Youmu')
        self.say("That's the spirit, Youmu. You'll be a bona fide crisis resolver like Reimu and me in no time.",
                'Marisa',
                'Marisa')
        self.say("Now that we're all gathered here, let's begin from where everything started, in the ancient times long before the foundation of Gensokyo.",
                'Akyu',
                'Akyu')
        self.play_music('event02')
        self.say("Our story begins with the birth of one maple tree on the Youkai Mountain and the mountain's guardian.",
                'Akyu',
                'Akyu')
        self.say("You mean Princess Iwanaga? She was known in legend as a goddess of immortality who resides on Youkai Mountain.",
                'Keine',
                'Keine')
        self.say("That's a name I'd rather not hear again. Is Fuyuhana an immortal being then?",
                'Mokou',
                'Mokou')
        self.say("You see, the trees that lived on Youkai Mountain were blessed by goddess with extraordinarily long lifetimes. The tree was born on the mountain, but the winds carried it to a vast plain where the Forest of Magic now lies.",
                'Akyu',
                'Akyu')
        self.say("As trees age, they gain a companion guardian spirit. These are the Kodama. Fuyuhana was the kodama that was bound to this first tree.",
                'Akyu',
                'Akyu')
        self.say("All the trees in the Forest of Magic originated from one tree?",
                'Chen',
                'Chen')
        self.say("That's mostly right. Some trees would eventually drift in, but Fuyuhana was also a particularly wise leader and strong protector of the forest. Over the ages, she served as the leader of the Kodama and other flora and fauna in the Forest of Magic...",
                'Akyu',
                'Akyu')
                
        self.stop_music()
        
        # Scene with Reimu and Marisa dozing off at Akyu's explanation
        self.emote('Akyu','dotdotdot')
        self.pause(0.2)
        self.emote('Reimu','zzz')
        self.emote('Marisa','zzz')
        self.pause(0.2)
        
        self.emote('Keine','annoyed')
        self.move_unit('Keine',(16,9))
        self.startle('Keine')
        
        self.say("Hey, you two! Wake up! This is important! Don't make me treat you like one of my students who fall asleep in class!",
                'Keine',
                'Keine')
        self.say("Keine, I could use your help. I have an idea. And Reimu and Marisa and Youmu.",
                'Akyu',
                'Akyu')

        # Set everyone in position for the play
        # Reimu, Youmu, and Marisa are the actors
        self.fade_to_color('black',1.0)
        self.set_unit_pos('Mokou',(10,12))
        self.set_unit_pos('Akyu',(11,11))
        self.set_unit_pos('Keine',(10,10))
        self.set_unit_pos('Ran',(17,10))
        self.set_unit_pos('Chen',(17,12))
        
        self.set_unit_pos('Reimu',(17,11))
        self.set_unit_pos('Youmu',(11,10))
        self.set_unit_pos('Marisa',(14,11))
 
        self.fade_from_color('black',1.0)
        
        # Theater scene
        self.play_music('event01')
        self.emote('Reimu','scribble')
        self.say("So we just read these lines? Ugh... these look so thick and formal. It's not anything like my usual way of talking.",
                'Reimu',
                'Reimu')
        self.say("Just read it as it comes to you. Reimu, you are our great Kodama Lord, Fuyuhana Touji. And Youmu, you are her loyal retainer Ayaka. Marisa, you will be Hotaru, a representative of the humans opposing Fuyuhana.",
                'Akyu',
                'Akyu')
        self.say("Ayaka? I think that was one of the Kodama Lords we fought.",
                'Chen',
                'Chen')
        self.say("But I've never acted before!",
                'Youmu',
                'Youmu')
        self.say("Neither have I, but it seems kind of fun! At least it's more entertaining than just listening to Akyu's lecture.",
                'Marisa',
                'Marisa')
        self.say("All right, we're starting!",
                'Akyu',
                'Akyu')

        self.startle('Marisa')
        self.say("The Gensokyo Players present: Legend of the Forest of Magic by Akyu Hieda!",
                'Marisa',
                'Marisa')
        self.startle('Marisa')
        self.say("Act 5, Scene 3 - End of the Kodama Lord's Reign! The setting is the Forest of Magic thousands of years ago.",
                'Marisa',
                'Marisa')
        self.say("We hope you enjoy this play!",
                'Marisa',
                'Marisa')

        # Fuyuhana and Ayaka talk about being cornered
        self.move_unit('Marisa',(12,13))
        self.move_unit('Reimu',(14,10))
        self.play_music('battle01')
        self.move_unit('Youmu',(13,10))
        self.say("I heard the news. Haruna and the others were just defeated. Does your reign really have to come to an end? Can't you keep fighting?",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("We may be blessed by by Iwanagi, but we are not immortal. Perhaps our continued growth of the forest backfired on us.",
                'Fuyuhana (Reimu)',
                'Reimu')
        self.say("We can still hold them here. They may have the rest of the land, but they'll never make it to the center of the forest.",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("Fighting the humans and the youkai that have allied with them has taken such a dreadful toll on us. I have seen too many of our brothers and sisters, trees and Kodama alike, perish in this struggle.",
                'Fuyuhana (Reimu)',
                'Reimu')
        self.say("I could not have forseen how strong and determined they have been, but please, we can't give up!",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("The humans fear our final retribution. As an elder Kodama Lord, the curse I release if I die will bring floods and plagues upon the village. They are terrified of delivering the final blow. That may be the only thing that will save this forest.",
                'Fuyuhana (Reimu)',
                'Reimu')
        self.say("One of them comes this way!",
                'Ayaka (Youmu)',
                'Youmu')

        # Hotaru arrives to propose an end to the crisis
        self.move_unit('Marisa',(14,12))
        self.say("Why have you come here to this sacred place?",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("We have decided to cut down your tree.",
                'Hotaru (Marisa)',
                'Marisa')
        
        self.startle('Youmu')
        self.move_unit('Youmu', (14,11))
        self.emote('Youmu','annoyed')
        self.say("Let me kill her now! I will die before you lay a hand on my master!",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("Please listen. We do not want to incur the wrath of the forest. I would like to offer a pact with you.",
                'Hotaru (Marisa)',
                'Marisa')
        self.say("What could you possibly offer us?",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("Ayaka, please. I want to hear what they have to say.",
                'Fuyuhana (Reimu)',
                'Reimu')
 
        self.move_unit('Youmu',(13,10))
        self.say("The thing you cherish the most is the protection of the forest. We will allow your ghost to remain and watch over the forest. And we will not harm it.",
                'Hotaru (Marisa)',
                'Marisa')
        self.say("That's insane! You can't just go like that!",
                'Ayaka (Youmu)',
                'Youmu')
 
        self.emote('Reimu','dotdotdot')
        
        self.say("Ayaka, our duty is to the forest. Forever and ever, we are bound to be its protectors. If it means accepting my death in exchange for being able to watch over the forest, I will do so.",
                'Fuyuhana (Reimu)',
                'Reimu')
        self.say("But...",
                'Ayaka (Youmu)',
                'Youmu')
        self.say("All right. I will accept those terms.",
                'Fuyuhana (Reimu)',
                'Reimu')
        self.say("Ayaka, the pact is made. There is nothing more to say. The forest will no longer expand, but we will remain as her protectors.",
                'Fuyuhana (Reimu)',
                'Reimu')

        # Move everyone to the center of the stage
        self.stop_music()
        self.play_music('event02')
        self.move_unit('Youmu',(13,11))
        self.move_unit('Reimu',(14,11))
        self.move_unit('Marisa',(15,11))
        self.say("And so the spirit of Fuyuhana kept watched over the forest for the next several centuries.",
                'Youmu',
                'Youmu')
        self.say("Through the centuries, she was known as ghost who would chase out enemies of the forest and even guide those lost back the forest's edge.",
                'Marisa',
                'Marisa')
        self.say("What will become of her? Why has she started this latest crisis? These are things not yet recorded in our history.",
                'Reimu',
                'Reimu')

        # chen is pleased
        self.emote('Chen','musicnote')
        self.say("*clap* *clap* *clap*",
                'Chen',
                'Chen')
        
        self.play_music('event01')
        # Restore character positions
        self.fade_to_color('black',(1.0))
        self.set_unit_pos('Akyu',(14,8))
        self.set_unit_pos('Youmu',(16,8))
        self.set_unit_pos('Mokou',(17,7))
        self.set_unit_pos('Keine',(18,7))
        self.set_unit_pos('Ran',(17,8))
        self.set_unit_pos('Chen',(18,8))
        self.set_unit_pos('Reimu',(17,9))
        self.set_unit_pos('Marisa',(18,9))
        self.fade_from_color('black',(1.0))
        
        self.say("I'll have to try that in my classes next time.",
                'Keine',
                'Keine')
 
        self.emote('Youmu','lightbulb')
        self.say("I think I have an idea where to head next then. We should pay a visit to Fuyuhana's birthplace on Youkai Mountain.",
                'Youmu',
                'Youmu')
        self.say("There's a legend among the Tengu youkai that live in the mountain. It talks about a Kodama and a tree even older than Fuyuhana at the top of the mountain.",
                'Akyu',
                'Akyu')
        self.say("Maybe our masters went there as well.",
                'Chen',
                'Chen')
        self.say("It's not going to be easy convincing the Tengu to let us up there though.",
                'Reimu',
                'Reimu')
        self.say("I've never been to the Youkai Mountain before, what's it like?",
                'Chen',
                'Chen')
        self.say("It's a very dangerous area for outsiders since the Tengu youkai that live here are fiercely territorial. They are a very proud people. They would rather not involve themselves with matters in the rest of the world.",
                'Ran',
                'Ran')
        self.say("Then we'll just have to bust in there.",
                'Marisa',
                'Marisa')
        self.say("Busting down the front gate is a surefire way to bring the wrath of the Tengu down upon us.",
                'Ran',
                'Ran')
        self.say("Reimu and I got in last time. They sent Aya to fight us, but she wasn't a match for us.",
                'Marisa',
                'Marisa')
        self.say("Surely with the latest crisis, the Tengu are going to be more on edge. If the Kodama Lords are pushing out of the forest, the Tengu are not going to allow visitors in readily.",
                'Mokou',
                'Mokou')
        self.say("There's also the issue of what to do with all the clouds gone since there won't be much rain until this crisis is resolved. This must affect the Tengu too!",
                'Youmu',
                'Youmu')
        self.say("She's right. Some areas like Youkai Mountain where there are rivers will be fine for now, but the human village and other areas won't be especially as we move into the blazing heat of summer.",
                'Keine',
                'Keine')
        self.say("All the more reason to hurry and resolve it then.",
                'Marisa',
                'Marisa')
        self.say("They may be stubborn, but the looming threat of an eventual drought may force the Tengu to bend our way. We may be able to negotiate with them for passage.",
                'Keine',
                'Keine')
        self.say("In any case, we should prepare ourselves for a rough climb. This next part is not going to be easy.",
                'Keine',
                'Keine')
        self.say("Then let's get back on the path right away!",
                'Youmu',
                'Youmu')
        self.say("Youkai Mountain is to the south, right?",
                'Chen',
                'Chen')
        self.say("Yeah, there's a path from the village leading there.",
                'Reimu',
                'Reimu')
        self.say("Oh! Before I forget...",
                'Akyu',
                'Akyu')

        # Akyu goes into her house to pick up stuff
        self.move_unit('Akyu',(14,9))
        self.emote('Akyu','questionmark')
        self.emote('Akyu','dotdotdot')
        self.emote('Akyu','exclamation')
        self.move_unit('Akyu',(14,8))
 
        self.say("I dug this recipe for the spell Healing Drop up in my archives. You might find it useful. Please have a safe journey.",
                'Akyu',
                'Akyu')
        self.add_recipe('Healing Drop')
        self.say("Healing Drop can now be synthesized!",
                None,
                None)
        self.set_stats_display(True)
        self.set_cursor_state(True)