from lostsky.worldmap.event import BattleEvent 
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, ArrivalTrigger

class Mission(BattleEvent):
    
    def __init__(self):
        # Event Data
        name = 'Towards the Youkai Mountain'
        location = 'Southern Village Path'
        id_string = 'CH2ST5'
        prereqs = ['CH2ST4']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "What's this about a secret weapon being prepared by the Fuzzball clans? After being chased out from the forest, the Fuzzballs have gathered at the southern road leading out from the human village. Could they be planning to strike back against Fuyuhana?"

        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch2st5v2.txt'
        mission_type = 'battle'
        objective = {'type':'Defeat Boss',
                      'target': 'Lord Fuzzy',
                      'desc': 'Defeat Lord Fuzzy!'}
        
        deploy_data = {'enable':True,
                       'max_units':8,
                       'preset_units':{},
                       'boxes':[(12,2,4,2)],
                       }
        
        reward_list = [('treasure','003_ibistapestry'), 
                      ]
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Lord Fuzzy',
                                'unit_name': 'Lord Fuzzy',
                                    'level': 12},
                           
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball A',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball B',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball C',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball D',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball E',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball F',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball G',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball H',
                                    'level': 6},
                           {'template_name': 'Fuzzball',
                                'unit_name': 'Fuzzball I',
                                    'level': 6},
                            
                           {'template_name': 'Wind Weasel',
                                'unit_name': 'Wind Weasel A',
                                    'level': 6},
                           {'template_name': 'Wind Weasel',
                                'unit_name': 'Wind Weasel B',
                                    'level': 6},
                           {'template_name': 'Wind Weasel',
                                'unit_name': 'Wind Weasel C',
                                    'level': 6},
                          ]
        
        initial_spells = {'Lord Fuzzy':['Holy Amulet'],
                          'Fuzzball A':['Stardust'],
                          'Fuzzball B':['Dagger Throw'],
                          'Fuzzball C':['Fireball'],
                          'Fuzzball D':['Stardust'],
                          'Fuzzball E':['Dagger Throw'],
                          'Fuzzball F':['Fireball'],
                          'Fuzzball G':['Stardust'],
                          'Fuzzball H':['Dagger Throw'],
                          'Fuzzball I':['Fireball'],
                          'Wind Weasel A':['Leaf Crystal'],
                          'Wind Weasel B':['Leaf Crystal'],
                          'Wind Weasel C':['Leaf Crystal'],
                          }
        initial_traits = {'Lord Fuzzy':['Regen Lv.1','Defense+ Lv.3'],
                         }
        initial_ai_states = {'Lord Fuzzy':'Attack',
                             'Fuzzball A':'Attack',
                             'Fuzzball B':'Attack',
                             'Fuzzball C':'Attack',
                             'Fuzzball D':'Attack',
                             'Fuzzball E':'Attack',
                             'Fuzzball F':'Attack',
                             'Fuzzball G':'Attack',
                             'Fuzzball H':'Attack',
                             'Fuzzball I':'Attack',
                             'Wind Weasel A':'Attack',
                             'Wind Weasel B':'Attack',
                             'Wind Weasel C':'Attack'
                             }
        initial_locations = {'Youmu':(15,4),
                             'Reimu':(14,3),
                             'Marisa':(14,2),
                             'Chen':(15,3),
                             'Ran':(15,2),
                             'Keine':(16,3),
                             'Mokou':(16,2),
                             
                             #top half of map
                             
                             'Fuzzball G':(7,6),
                             'Fuzzball H':(8,5),
                             'Fuzzball I':(6,7),
                             
                             'Wind Weasel A':(3,12),
                             'Wind Weasel B':(2,11),
                             'Wind Weasel C':(4,11),
                             
                             #bottom half of map
                             'Fuzzball D':(20,8),
                             'Lord Fuzzy':(18,17),
                             'Fuzzball E':(21,7),
                             'Fuzzball F':(19,9),
                             
                             'Fuzzball A':(17,16),
                             'Fuzzball B':(16,17),
                             'Fuzzball C':(18,15),
                             }
        reserve_units = []#[list of unit names to deploy later in mission]
        all_landmarks = [{'name':'Mini Shrine',
                          'id_string':'minishrine',
                          'location':(3,10)},
                         {'name':'CB 1',
                          'id_string':'cherryblossom_tree',
                          'location':(2,10)},
                         {'name':'CB 2',
                          'id_string':'cherryblossom_tree',
                          'location':(4,10)},
                         ]
    
        required_starters = ['Youmu','Chen','Ran','Marisa','Reimu','Mokou','Keine']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [TreasureMAE()]
        required_survivors = ['Lord Fuzzy', 'Youmu', 'Reimu', 'Marisa', 'Ran',
                              'Chen', 'Mokou', 'Keine'] 
        post_mission_MAE = PostMissionMAE()
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Prologue event 
        """
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.center_on('Lord Fuzzy')

        # Fuzzballs demonstrating their stun spell
        self.say("Hey, boss! I got something to show you!",
                'Fuzzball A',
                'Fuzzball')
        self.emote('Fuzzball B','dotdotdot')
        self.say("What is it?",
                'Lord Fuzzy',
                None)
        self.say("The Master of the Forest's armies are trees right? Well... this is our secret weapon!",
                'Fuzzball A',
                'Fuzzball')
        self.say("Buddy, could you stand over here for a moment?",
                'Fuzzball A',
                'Fuzzball')
        self.move_unit('Fuzzball B', (16,16))
        self.emote('Fuzzball B', 'dotdotdot')
        
        self.say("Here we go... Dazzling Light! Stardust!",
                'Fuzzball A',
                'Fuzzball')
 
        self.fade_to_color('white', 0.1)
        self.fade_from_color('white', 0.1)
        self.emote('Fuzzball B', 'scribble')
        self.say("Hey! Watch where you're firing that, buddy! Agh! I can't see! I can't move!",
                'Fuzzball B',
                'Fuzzball')
 
        self.emote('Fuzzball B', 'dotdotdot')
        self.say("Oh... I can move again.",
                'Fuzzball B',
                'Fuzzball')
 
        self.move_unit('Fuzzball B', (16, 17))
        self.startle('Fuzzball A')
        self.say("Last night, I put together some element crystals to make these things. This'll stop those walking trees long enough that we can ambush them!",
                'Fuzzball A',
                'Fuzzball')
        self.say("Ohohohoho. Yes. Well done.",
                'Lord Fuzzy',
                None)
 
        self.center_on('Fuzzball G')
        self.play_music('battle02')
        self.say("Boss, heads up! Humans coming our way!",
                'Fuzzball G',
                'Fuzzball')
        self.say("Battle formations everyone! We can't lose twice in one day!",
                'Lord Fuzzy',
                None)

        # Player gets info on how best to deal with it
        self.center_on('Marisa')
        self.say("Those fuzzballs stole my trademark flashy spells!",
                'Marisa',
                'Marisa')
        self.say("Did you see how that one fuzzball was stunned momentarily? We should be careful fighting them.",
                'Ran',
                'Ran')
        self.say("It doesn't look like their stardust can reach far though. Attacking from a distance would be the best strategy.",
                'Keine',
                'Keine')
        self.say("Those pesky fuzzballs may be blocking our way, but I think we can just beat up their leader and they'll let us through.",
                'Youmu',
                'Youmu')
 
        self.set_cursor_state(True)
        self.set_stats_display(True)
        
class TreasureMAE(MapActionEvent):
    
    def __init__(self):
        triggers = [ArrivalTrigger( (3, 10, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)
    
    def execute(self):
        self.say("A lone shrine is here that was guarded by youkai weasels.",
                None,
                None)
        self.say("They were protecting an odd treasure box. Inside was a rusted lamp.",
                None,
                None)
        self.say("Acquired Treasure Item: Magic Lamp!",
                None,
                None)
        self.add_item('treasure', '004_magiclamp', 1)
        
class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):

        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.remove_all_enemies()
        self.stop_music()
        self.set_unit_pos('Youmu', (16, 15))
        self.set_unit_pos('Ran', (15, 14))
        self.set_unit_pos('Chen', (15, 13))
        self.set_unit_pos('Marisa', (16, 14))
        self.set_unit_pos('Reimu', (16, 13))
        self.set_unit_pos('Mokou', (17, 14))
        self.set_unit_pos('Keine', (17, 13))
        self.set_unit_pos('Lord Fuzzy', (16, 17))

    def execute(self):

        # Shows that not all youkai are aligned with Fuyuhana
        self.center_on('Youmu')
        self.say("Defeated twice in one day...",
                'Lord Fuzzy',
                None)
        self.say("Someone got here before us?",
                'Marisa',
                'Marisa')
        self.say("That's right, the Master of the Forest is already pushing beyond the Forest of Magic. She's gathered the clouds all around the forest to nourish her trees with plenty of rain.",
                'Lord Fuzzy',
                None)
        self.say("Where are they moving to next?",
                'Youmu',
                'Youmu')
        self.say("The Master of the Forest's underlings are moving in this direction, but I hear from some of our northern cousins that they are advancing a large group toward the bamboo forest.",
                'Lord Fuzzy',
                None)
        self.say("Fuyuhana's definitely ambitious.",
                'Mokou',
                'Mokou')
    