from lostsky.worldmap.event import BattleEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, ArrivalTrigger, CustVarTrigger

class Mission(BattleEvent):
    
    def __init__(self):
        # Event Data
        name = 'River Basin Rumble'
        location = 'River Basin'
        id_string = 'CH3ST1'
        prereqs = ['CH2ST5']
        auto_signup = True
        cancellable = False
        show_rewards = True
        desc = "Youkai Mountain's Wolf Tengu defense force is on high alert this morning. A recent attack by the Kodama Lords was repelled by Momiji, captain of the Wolf Tengu guards, with many trees cut to pieces by the wolves and aided from the air by our Crow Tengu sorcerers. This surprising turn of events has the elder Tengu deeply concerned about the conflict outside the mountain reaching our territory."
        
        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch3st1.txt'
        mission_type = 'battle'
        objective = {'type':'Defeat Boss',
                     'target':'Momiji',
                     'desc':'Defeat the Momiji the Wolf Tengu!'
                     }   
        
        deploy_data = {'enable':True,
                       'max_units':8,
                       'preset_units':{},
                       'boxes':[(6,35,2,2),(14,35,2,2)]
                       }

        reward_list = [('treasure','synth_metal'),
                       ('treasure','synth_wood'),
                      ]
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Momiji',
                                'unit_name': 'Momiji',
                                    'level': 9},
                         {'template_name': 'Nitori',
                                'unit_name': 'Nitori',
                                    'level': 8},

                         {'template_name': 'Kappa',
                                'unit_name': 'Kappa A',
                                'level':9},
                        {'template_name': 'Kappa',
                                   'unit_name': 'Kappa B',
                                   'level':9},
                        {'template_name': 'Kappa',
                                   'unit_name': 'Kappa C',
                                   'level':9},

                        {'template_name': 'Kappa',
                             'unit_name': 'Kappa Medic',
                                 'level': 9},


                         {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Tengu Ascetic A',
                                    'level': 8},
                         {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Tengu Ascetic B',
                                    'level': 8},
                         {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Tengu Ascetic C',
                                    'level': 8},

                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu A',
                            'level': 8},
                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu B',
                            'level': 8},
                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu C',
                            'level': 8},
                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu D',
                            'level': 8},
                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu E',
                            'level': 8},
                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu F',
                            'level': 8},
                        {'template_name': 'Wolf Tengu',
                            'unit_name': 'Wolf Tengu G',
                            'level': 8},
                        {'template_name': 'Aya',
                            'unit_name': 'Aya',
                            'level': 10},

                        {'template_name': 'Tengu Ascetic',
                            'unit_name': 'Tengu Ascetic E',
                                'level': 8},
                        {'template_name': 'Tengu Ascetic',
                            'unit_name': 'Tengu Ascetic F',
                                'level': 8},


                          ]
        
        initial_spells = {'Momiji':['Dagger Throw'],
                          'Nitori':['Tanabata Festival'],
                          'Tengu Ascetic A':['Weakening Amulet'],
                          'Tengu Ascetic B':['Barrier Buster'],
                          'Tengu Ascetic C':['Weakening Amulet'],
                          'Kappa A':['Illusion Veil', 'Fireball'],
                          'Kappa B':['Encourage', 'Fireball'],
                          'Kappa C':['Life Bless', 'Fireball'],
                          'Wolf Tengu A':['Dagger Throw'],
                          'Wolf Tengu B':['Dagger Throw'],
                          'Wolf Tengu C':['Dagger Throw'],
                          'Wolf Tengu D':['Dagger Throw'],
                          'Wolf Tengu E':['Dagger Throw'],
                          'Wolf Tengu F':['Dagger Throw'],
                          'Wolf Tengu G':['Dagger Throw'],
                          
                          'Kappa Medic':['Healing Drop'],
        }
                          
        initial_traits = {'Momiji':['Attack+ Lv.1'],
                          'Tengu Ascetic A':['Flight'],
                          'Tengu Ascetic B':['Flight'],
                          'Tengu Ascetic C':['Flight'],
                          'Kappa A':['Swimming'],
                          'Kappa B':['Swimming'],
                          'Kappa C':['Swimming'],
                          'Kappa Medic':['Mirage','Swimming'],
                          'Nitori':['Danmaku Sniper', 'Swimming']
        }
        initial_ai_states = {'Momiji':'Attack',
                             'Nitori':'Attack',
                             'Tengu Ascetic A':'Pursuit',
                             'Tengu Ascetic B':'Pursuit',
                             'Tengu Ascetic C':'Pursuit',
                             'Kappa A':'Support',
                             'Kappa B':'Support',
                             'Kappa C':'Support',
                             'Wolf Tengu A':'Attack',
                             'Wolf Tengu B':'Attack',
                             'Wolf Tengu C':'Attack',
                             'Wolf Tengu D':'Attack',
                             'Wolf Tengu E':'Attack',
                             'Wolf Tengu F':'Attack',
                             'Wolf Tengu G':'Attack',
                             'Kappa Medic':'HealerStandby',

                             }

        initial_locations = {
                             # Tengu Lookout


                             'Youmu':(6,35),
                             'Keine':(5,36),
                             'Mokou':(7,36),

                             'Ran':(12,36),
                             'Chen':(12,37),
                             'Marisa':(13,36),
                             'Reimu':(13,37),


                             # Forward scout party
                             'Wolf Tengu F':(9, 25),
                             'Wolf Tengu G':(8, 24),
                             'Tengu Ascetic C':(6, 23),
                             'Kappa A':(7,24),
                             'Kappa B':(14,9),
                             'Kappa C':(15,9),


                             'Wolf Tengu A':(20, 9),
                             'Wolf Tengu B':(19, 9),
                             'Wolf Tengu C':(23, 10),
                             'Wolf Tengu D':(20, 11),
                             'Wolf Tengu E':(21, 11),

                             'Tengu Ascetic A':(20, 14),
                             'Tengu Ascetic B':(21, 14),

                             'Kappa Medic':(18, 9)

                             }
        reserve_units = ['Momiji', 'Nitori', 'Aya', 'Tengu Ascetic E', 'Tengu Ascetic F']#[list of unit names to deploy later in mission]
        all_landmarks = [
                         {'name':'CB1',
                          'id_string':'cherryblossom_tree',
                          'location':(22,31)},

                         # Rocks
                        {'name':'SR1',
                         'id_string':'small_rock',
                         'location':(8,16)},
                        {'name':'SR2',
                         'id_string':'small_rock',
                         'location':(19,19)},
                        {'name':'SR3',
                         'id_string':'small_rock',
                         'location':(23,15)},
                        {'name':'BR1',
                         'id_string':'big_rock',
                         'location':(22,14)},
                        {'name':'BR2',
                         'id_string':'big_rock',
                         'location':(13,11)},
                        {'name':'SR4',
                         'id_string':'small_rock',
                         'location':(14,12)},

                         #Houses
                        {'name':'Island House',
                         'id_string':'house_1',
                         'location':(13,12)},
                        {'name':'East House',
                         'id_string':'house_1',
                         'location':(22,15)},


                         #Lilypads
                         {'name':'LP1',
                          'id_string':'lilypad',
                          'location':(4,12)},
                         {'name':'LP2',
                          'id_string':'lilypad',
                          'location':(8,20)},
                         {'name':'LP3',
                          'id_string':'lilypad',
                          'location':(10,7)},
                         {'name':'LP4',
                          'id_string':'lilypad',
                          'location':(12,21)},


                         ]
    
        required_starters = ['Youmu','Chen','Ran', 'Reimu', 'Marisa', 'Keine', 'Mokou']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [TreasureFoundMAE()
                                ]
        post_mission_MAE = PostMissionMAE()
        required_survivors = ['Youmu','Chen','Ran', 'Reimu', 'Marisa', 'Keine', 'Mokou', 'Nitori', 'Momiji', 'Wolf Tengu A']
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Mission Prologue: Momiji
        """
        self.set_stats_display(False)
        self.set_cursor_state(False)

        # Party approaches the river from the south
        self.center_on('Marisa')
        self.say("Boy, the Tengu are out in force today.",
                 'Marisa',
                 'Marisa')
        self.say("There weren't this many the last time we were here.",
                 'Reimu',
                 'Reimu')
        self.say("I think I see someone coming down from the mountain.",
                 'Youmu',
                 'Youmu')
        self.show_chapter_title(3)

        # Wolf tengu discussing their recent encounters, but the Crow Tengu aren't impressed
        self.center_on('Wolf Tengu A')

        self.startle('Wolf Tengu A')
        self.say("When is Momiji going to get here?",
                 'Wolf Tengu A',
                 'Wolf Tengu')

        self.emote('Wolf Tengu B', 'exclamation')
        self.say("I heard she can bring down one of Fuyuhana's trees with a single swipe. She singlehandedly drove them off when they tried to make their way here.",
                 'Wolf Tengu B',
                 'Wolf Tengu')

        self.move_unit('Tengu Ascetic A', (19,13))
        self.emote('Tengu Ascetic A', 'scribble')
        self.say("Please. You wolves are all brawn and no brains. A single curse like this Weakening Amulet and you'll barely have the strength of a puppy.",
                 'Tengu Ascetic A',
                 'Crow Tengu')

        self.emote('Wolf Tengu A', 'annoyed')
        self.say("What did you just call us?",
                 'Wolf Tengu A',
                 'Wolf Tengu')

        self.emote('Tengu Ascetic A', 'scribble')

        self.say("That's right. We Crow Tengu rule the skies. With our curses, we'll stop any enemy in their tracks. Leave Fuyuhana's trees to us.",
                 'Tengu Ascetic A',
                 'Crow Tengu')

        self.startle('Tengu Ascetic B')
        self.say("If we didn't have orders to help you from Lord Tenma we would be protecting this mountain on our own.",
                 'Tengu Ascetic A',
                 'Crow Tengu')

        self.startle('Wolf Tengu B')
        self.say("Like we'll let you steal all the glory.",
                 'Wolf Tengu B',
                 'Wolf Tengu')

        # Momiji and Nitori arrive, they seem to be talking about something
        self.deploy_unit('Momiji', (15,-1))
        self.deploy_unit('Nitori', (15,-1))

        self.move_unit('Momiji', (13,5))
        self.move_unit('Nitori', (13,4))
        self.emote('Nitori', 'heart')
        self.emote('Momiji', 'questionmark')

        self.move_unit('Momiji', (15,7))
        self.move_unit('Nitori', (14,7))
        self.emote('Nitori', 'lightbulb')
        self.startle('Momiji')

        self.move_unit('Momiji', (17,8))
        self.move_unit('Nitori', (16,8))
        self.emote('Momiji','annoyed')

        self.say("Enough bickering! Wolf Tengu, form up! Crow Tengu, form up!",
                 'Momiji',
                 'Momiji')

        self.say("You heard her!",
                 'Wolf Tengu C',
                 'Wolf Tengu')

        # Tengu units form up
        self.move_unit('Wolf Tengu C', (21, 9))
        self.move_unit('Wolf Tengu D', (20, 10))
        self.move_unit('Tengu Ascetic A',(20,11))
        self.startle('Tengu Ascetic A')
        self.move_unit('Wolf Tengu D', (19,10))
        self.move_unit('Tengu Ascetic A', (20,10))
        self.move_unit('Wolf Tengu E', (21,10))
        self.move_unit('Tengu Ascetic B', (21,11))
        self.move_unit('Kappa Medic', (19,11))

        self.say("Listen up! We're to protect the waterworks that the Kappa are building from intruders. Anyone that trespasses on the mountain is an enemy!",
                 'Momiji',
                 'Momiji')

        self.center_on('Tengu Ascetic C')
        self.emote('Tengu Ascetic C', 'exclamation')
        self.say("Momiji, a bunch of humans are approaching from downstream!",
                 'Tengu Ascetic C',
                 'Crow Tengu')

        self.play_music('battle02')

        self.center_on('Wolf Tengu A')
        self.say("Speak of the devil!",
                 'Wolf Tengu C',
                 'Wolf Tengu')

        self.emote('Momiji','exclamation')
        self.say("Everyone in position! We'll block them from both sides of the river.",
                 'Momiji',
                 'Momiji')

        self.fade_to_color('black', 0.5)

        # Forward two groups
        self.set_unit_pos('Tengu Ascetic A',(6, 23))
        self.set_unit_pos('Tengu Ascetic B',(16, 23))
        self.set_unit_pos('Tengu Ascetic C',(17, 23))
        self.set_unit_pos('Wolf Tengu A',(9, 25))
        self.set_unit_pos('Wolf Tengu B',(8, 24))
        self.set_unit_pos('Wolf Tengu C',(20, 25))

        # Nitori's Group
        self.set_unit_pos('Nitori',(11, 14))
        self.set_unit_pos('Kappa B',(10, 15))
        self.set_unit_pos('Kappa C',(12, 15))
        self.set_unit_pos('Wolf Tengu D',(10, 14))
        self.set_unit_pos('Wolf Tengu E',(12, 14))

        # Momiji's group
        self.set_unit_pos('Momiji',(20 ,8))
        self.set_unit_pos('Wolf Tengu F',(19, 7))
        self.set_unit_pos('Wolf Tengu G',(21, 9))
        self.set_unit_pos('Kappa Medic',(19, 9))

        self.fade_from_color('black', 0.5)

        self.center_on('Mokou')
        self.say("Here they come!",
                 'Mokou',
                 'Mokou')

        self.emote('Reimu', 'lightbulb')
        self.say("Momiji seems to be keeping them in line. Defeating her should do the trick.",
                 'Reimu',
                 'Reimu')


        self.set_stats_display(True)
        self.set_cursor_state(True)

class TreasureFoundMAE(MapActionEvent):

    def __init__(self):
        triggers = [ArrivalTrigger((22, 31, 1, 1), 1)]
        MapActionEvent.__init__(self,triggers)

    def execute(self):
        """
        Player has discovered treasure under cherry blossom tree
        """

        self.say("Buried at the foot of the cherry tree is a small rusted box.",
                None,
                None)
        self.say("Acquired Treasure Item: Gondolier's Glove!",
                None,
                None)
        self.add_item('treasure','005_gondglove',1)

class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):
        self.remove_all_enemies()
        self.stop_music()
        self.set_unit_pos('Momiji', (16,6))
        self.set_unit_pos('Nitori', (17,5))
        self.set_unit_pos('Wolf Tengu A', (17,6))

        self.set_unit_pos('Ran', (17,8))
        self.set_unit_pos('Youmu', (18,8))
        self.set_unit_pos('Keine', (19,8))

        self.set_unit_pos('Marisa', (16,9))
        self.set_unit_pos('Chen', (17,10))
        self.set_unit_pos('Mokou', (19,10))
        self.set_unit_pos('Reimu', (20,9))

    def execute(self):
        """
        Victory Scene
        """
        self.set_stats_display(False)
        self.set_cursor_state(False)

        # Hero group confronts Momiji
        self.center_on('Momiji')
        self.say("Don't take another step forward!",
                'Momiji',
                'Momiji')
        self.say("Easy there, Momiji. We're not with Fuyuhana's trees.",
                 'Keine',
                 'Keine')

        self.say("You certainly acted like you are, what with you wiping out our entire mountain defense force.",
                 'Momiji',
                 'Momiji')

        self.emote('Marisa', 'annoyed')
        self.say("You attacked us first!",
                 'Marisa',
                 'Marisa')

        self.say("It doesn't matter! You intruded on the mountain!",
                 'Momiji',
                 'Momiji')

        self.emote('Youmu', 'annoyed')
        self.emote('Momiji', 'annoyed')

        # Aya arrives with more Crow Tengu
        self.deploy_unit('Aya', (23, -1))

        self.deploy_unit('Tengu Ascetic E', (19, -1))
        self.deploy_unit('Tengu Ascetic F', (20, -1))

        self.move_unit('Aya', (21,2))
        self.center_on('Aya')
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        self.say("How about this for a headline? 'Former heroes invade Tengu Homeland!'",
                 'Aya',
                 'Aya')

        self.move_unit('Tengu Ascetic E', (19,6))
        self.move_unit('Tengu Ascetic F', (20,7))

        # Aya circles around and takes pictures
        self.move_unit('Aya', (21,12))
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        self.move_unit('Aya', (15,12))
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        self.move_unit('Aya', (15,6))
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        self.say("I was wondering when we'd see that paparazzi.",
                 'Reimu',
                 'Reimu')

        self.say("More reinforcements?",
                 'Youmu',
                 'Youmu')

        self.say("Actually, today I'm on official business from the Great Tengu.",
                 'Aya',
                 'Aya')

        self.emote('Wolf Tengu A', 'scribble')
        self.say("Ugh. The higher ups are calling the shots.",
                 'Wolf Tengu A',
                 'Wolf Tengu')

        self.say("Don't tell me you're going to invite these outsiders into the mountain.",
                 'Momiji',
                 'Momiji')
        self.say("This comes straight from Lord Tenma. We're to meet an emissary at the foot of Moriya Shrine.",
                 'Aya',
                 'Aya')

        self.say("Moriya Shrine? Are they involved now?",
                 'Marisa',
                 'Marisa')
        self.say("Not as far as I know, but it's the closest we have on this mountain to neutral ground.",
                 'Aya',
                 'Aya')
        self.say("Then let's go and hear what the Tengu have to say.",
                 'Youmu',
                 'Youmu')

        self.fade_to_color('black', 0.5)
        self.kill_unit('Youmu')
        self.kill_unit('Reimu')
        self.kill_unit('Mokou')
        self.kill_unit('Ran')
        self.kill_unit('Chen')
        self.kill_unit('Keine')

        self.kill_unit('Momiji')
        self.kill_unit('Aya')
        self.kill_unit('Tengu Ascetic E')
        self.kill_unit('Tengu Ascetic F')
        self.kill_unit('Wolf Tengu A')

        # Nitori and Marisa have a chat alone
        self.set_unit_pos('Marisa', (15,6))
        self.set_unit_pos('Nitori', (16,6))
        self.fade_from_color('black', 0.5)

        self.emote('Nitori', 'musicnote')
        self.say("By the way, Marisa, that was a heck of an impressive fight. Your Master Spark far outranges my rockets.",
                 'Nitori',
                 'Nitori')

        self.say("It's the first time I've seen you use it.",
                 'Marisa',
                 'Marisa')

        self.say("It's just a prototype, but you can have it if it'll help you. ",
                 'Nitori',
                 'Nitori')

        self.emote('Marisa', 'exclamation')
        self.say("You're really giving it to me?",
                 'Marisa',
                 'Marisa')

        self.say("I've heard about what's going on outside. We kappa will do our best to protect this mountain, so I want to help in whatever way I can.",
                 'Nitori',
                 'Nitori')


        self.say("Acquired Nitori's latest invention: Tanabata Festival",
                None,
                None)
        self.add_item('spell_action','Tanabata Festival',1)
        
        self.set_stats_display(True)
        self.set_cursor_state(True)