from lostsky.worldmap.event import ConversationEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent 

class Mission(ConversationEvent):
    
    def __init__(self):
        # Event Data
        name = 'Emissary of the Tengu'
        location = "Wind God's Lake"
        id_string = 'CH3ST2'
        prereqs = ['CH3ST1']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "Following the unexpected arrival of Youmu and company at the basin of Youkai Mountain, Emissary of Lord Tenma Tsubaki Akahane has agreed to meet with their group. There have been rumors being passed around the higher ranking Tengu that some deal may be made with them, but the majority of the Tengu leaders are strongly opposed to getting involved with the affairs of the outside world."
        
        ConversationEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'ch3st2.txt'
        mission_type = 'conversation'
        objective = None
        
        deploy_data = {'enable':False,
                       'max_units':None,
                       'preset_units':None,
                       'boxes':[]
                       }
        
        reward_list = []
                
        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Tsubaki',
                                'unit_name': 'Tsubaki',
                                    'level': 25},
                        {'template_name': 'Aya',
                            'unit_name': 'Aya',
                            'level': 10},
                        {'template_name': 'Momiji',
                                'unit_name': 'Momiji',
                                    'level': 8},
                         {'template_name': 'Nitori',
                                'unit_name': 'Nitori',
                                    'level': 7},
                        {'template_name': 'Yuyuko',
                                'unit_name': 'Yuyuko',
                                    'level': 10
                                },

                            {'template_name': 'Yukari',
                            'unit_name': 'Yukari',
                                'level': 16
                            }

        ]
        
        initial_spells = {}
        
        initial_traits = {}
        
        initial_ai_states = {}

        initial_locations = {'Ran':(4,13),
                             'Chen':(5,13),
                             'Youmu':(6,14),
                             'Marisa':(4,14),
                             'Reimu':(5,14),
                             'Mokou':(4,15),
                             'Keine':(5,15),

                             'Nitori':(8,12),
                             'Aya':(8,13),
                             'Momiji':(8,14),


                             'Tsubaki':(18,17)
                             }

        reserve_units = ['Yukari', 'Yuyuko']#[list of unit names to deploy later in mission]

        all_landmarks = [
                        {'name':'Main House',
                         'id_string':'house_2',
                         'location':(18,16)},

                        {'name':'Cherry Tree 1',
                         'id_string':'cherryblossom_tree',
                         'location':(17,16)},
                        {'name':'Cherry Tree 2',
                         'id_string':'cherryblossom_tree',
                         'location':(19,16)},

                        {'name':'West House',
                         'id_string':'house_1',
                         'location':(20,17)},
                        {'name':'East House',
                         'id_string':'house_1',
                         'location':(16,17)},

        ]
    
        required_starters = ['Youmu', 'Chen', 'Ran','Marisa', 'Reimu', 'Keine', 'Mokou']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = []
        required_survivors = []
        post_mission_MAE = None
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Prologue event 
        """
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.set_bg_overlay('Sunset')
        self.play_music('event01')

        # Aya motions for party to approach the tengu outpost
        self.center_on('Aya')
        self.say('This way! This way!', 'Aya', 'Aya')


        # Set everyone in front of Tsubaki
        self.fade_to_color('black', 0.5)


        self.set_unit_pos('Aya', (17, 18))
        self.set_unit_pos('Momiji', (19, 18))
        self.set_unit_pos('Youmu', (18, 19))
        self.set_unit_pos('Keine', (19, 19))
        self.set_unit_pos('Mokou', (16, 19))
        self.set_unit_pos('Ran', (16, 20))
        self.set_unit_pos('Chen', (17, 20))

        self.set_unit_pos('Nitori', (20, 19))
        self.set_unit_pos('Marisa', (20, 20))
        self.set_unit_pos('Reimu', (19, 20))

        self.fade_from_color('black', 0.5)

        # Conference with the Tengu leader
        self.center_on('Aya')

        self.say("Tsubaki, I've brought them as you requested. This'll make a great morning special report.",
                 'Aya',
                 'Aya')

        self.move_unit('Aya', (17,17))
        self.move_unit('Momiji', (19,17))

        self.say("It's a shame we couldn't have met under better circumstances.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("Ah... should I speak?",
                 'Youmu',
                 'Youmu')
        self.say("Let me handle this. I've had my share of dealing with the youkai.",
                 'Keine',
                 'Keine')
        self.say("Tsubaki requested to speak with Youmu. She's the leader of your group anyway.",
                 'Aya',
                 'Aya')
        self.say("Allow me to introduce myself. I am Tsubaki Akahane, representative of Lord Tenma and emissary from the Order of the Red Feathers.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("Youmu Konpaku, retainer of Lady Yuyuko Saigyouji.",
                 'Youmu',
                 'Youmu')


        self.say("I want to first know your reasons for approaching the mountain. At these times, you can understand that we are rightfully anxious about intruders.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("Let me explain.",
                 'Youmu',
                 'Youmu')

        self.fade_to_color('black', 1)
        self.fade_from_color('black', 1)

        self.say("And thus, we humbly request that you allow us passage to the sanctuary of the tree guardian at the top of your mountain.",
                 'Youmu',
                 'Youmu')
        self.say("No way! That's sacred ground for us! Only the Great Tengu priests are allowed in there.",
                 'Momiji',
                 'Momiji')

        self.emote('Tsubaki', 'dotdotdot')


        self.say("Unfortunately for you, It is as Momiji says. We don't let outsiders into Misaki's Sanctuary",
                 'Tsubaki',
                 'Tsubaki')
        self.say("Even if I could support you, I'll never be able to convince Lord Tenma or others in the Order. Our society simply doesn't believe in interfering with the rest of Gensokyo's affairs.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("You heard her answer. Now get out of here!",
                 'Momiji',
                 'Momiji')
        
        self.say("This crisis doesn't just affect us. Without the clouds over Gensokyo, we'll slowly run out of water, and so will you.",
                 'Youmu',
                 'Youmu')
        self.emote('Marisa', 'lightbulb')
        self.say("Nitori, how much water does the Youkai Mountain have?",
                 'Marisa',
                 'Marisa')
        self.say("I estimate about a month's worth.",
                 'Nitori',
                 'Nitori')
        
        self.say("We'll find a way to resolve this ourselves. We don't need...",
                 'Tsubaki',
                 'Tsubaki')
        self.say("And how do you propose to do that?",
                 'Marisa',
                 'Marisa')
        self.say("How dare you speak to Lady Akahane that way!",
                 'Momiji',
                 'Momiji')

        self.play_music('battle01')

        # Marisa pushes her way to the front
        self.move_unit('Marisa', (20,21))
        self.move_unit('Marisa', (18,21))
        self.move_unit('Marisa', (18,20))
        self.startle('Marisa')
        self.emote('Youmu', 'exclamation')
        self.move_unit('Youmu', (17,19))
        self.move_unit('Marisa', (18,19))
        self.emote('Marisa', 'annoyed')
        
        self.say("You know what? You Tengu are all cowards! The outside world is facing an imminent drought, and rather than help us, you hide in the mountain like scared baby birds.",
                 'Marisa',
                 'Marisa')
        self.say("Marisa... I don't think that you should antagonize her.",
                 'Youmu',
                 'Youmu')
        self.say("Resolve the crisis? Ha, you Tengu are supposed to be feared warriors. The crows rule the sky! The wolves rule the land! How can you possibly resolve the crisis if you never step foot outside of this mountain?",
                 'Marisa',
                 'Marisa')
        self.say("Marisa, wait!",
                 'Youmu',
                 'Youmu')
        self.say("Come on, Youmu. Come on, Reimu. We're busting our way up there whether these Tengu like us or not.",
                 'Marisa',
                 'Marisa')
        self.say("You're not helping us at all, Marisa!",
                 'Keine',
                 'Keine')
        self.say("Hold on.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("She's right.",
                 'Tsubaki',
                 'Tsubaki')

        self.play_music('event01')

        self.say("What?",
                 'Keine',
                 'Keine')
        self.say("Precisely because we fear getting entangled with the outside is why we'll never be able to resolve the crisis on our own.  We're at an impasse.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("Ah... I have a suggestion.",
                 'Chen',
                 'Chen')
        self.say("Chen, let your elders figure this out.",
                 'Ran',
                 'Ran')
        self.say("Let her speak, Ran.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("Only the Great Tengu priests can enter the holy sanctuary, right?",
                 'Chen',
                 'Chen')
        self.say("That's correct.",
                 'Aya',
                 'Aya')
        self.say("Then... how do we become Great Tengu priests?",
                 'Chen',
                 'Chen')
        self.emote('Marisa', 'exclamation')
        self.say("... what did you ask?",
                 'Marisa',
                 'Marisa')
        self.say("Aya, a word?",
                 'Tsubaki',
                 'Tsubaki')

        # Tsubaki and Aya move aside to have a private chat
        self.move_unit('Tsubaki', (15,17))
        self.move_unit('Aya', (14,17))
        self.emote('Aya', 'questionmark')
        self.emote('Tsubaki', 'heart')
        self.emote('Aya', 'exclamation')
        self.emote('Tsubaki', 'dotdotdot')
        self.emote('Tsubaki', 'lightbulb')
        self.emote('Aya', 'heart')

        self.say("No way. Are they seriously considering Chen's suggestion?",
                 'Reimu',
                 'Reimu')

        # Tsubaki and Aya return to the party
        self.move_unit('Tsubaki', (18,17))
        self.move_unit('Aya', (17,17))
        

        self.say("I agree. And we can't afford to devote our warriors to anything but protecting the mountain from Fuyuhana's cronies.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("They're not going to leave this mountain without trying. We might as well let them have a crack at it.",
                 'Aya',
                 'Aya')
        self.say("Chen, you may be the youngest among us, but perhaps the youth don't have their eyes clouded by our prejudices. I'll speak with the Order of the Red Feathers and Lord Tenma.",
                 'Tsubaki',
                 'Tsubaki')
        self.say("But the trial has never been given to outsiders before!",
                 'Momiji',
                 'Momiji')
        self.say("It's precisely why the others will agree to it. They'll see it as a showing of our resistance, a way of saving face in light of this being the only way to resolve the lost sky crisis.",
                 'Aya',
                 'Aya')
        self.say("You can stay here tonight at this outpost. I'll return in the morning with the word from Lord Tenma. Momiji, please come with me. We'll need to report on the battle at the river tonight.",
                 'Tsubaki',
                 'Tsubaki')

        self.say("I'll look after them and make sure they get all their arrangements to stay here in order.",
                 'Aya',
                 'Aya')

        self.say("We look forward to your answer, Miss Tsubaki.",
                 'Youmu',
                 'Youmu')

        # Tsubaki and Momiji withdraw
        self.move_unit('Tsubaki', (10,17))
        self.move_unit('Momiji', (11,17))
        self.move_unit('Tsubaki', (10,13))
        self.move_unit('Momiji', (11,13))
        self.move_unit('Tsubaki', (-1,13))
        self.move_unit('Momiji', (-1,13))


        self.say("Chen, I can't believe that you got us out of this sticky situation.",
                 'Mokou',
                 'Mokou')
        self.say("Don't celebrate yet. We have no idea how difficult this trial is.",
                 'Reimu',
                 'Reimu')
        self.say("One out of every hundred are able to pass. The Order of Red Feathers is composed exclusively of those who have passed this trial.",
                 'Aya',
                 'Aya')

        # Switch to nighttime scene with Yukari and Yuyuko
        self.fade_to_color('black', 1)
        self.set_unit_pos('Youmu', (-1,-1))
        self.set_unit_pos('Reimu', (-1,-1))
        self.set_unit_pos('Marisa', (-1,-1))
        self.set_unit_pos('Ran', (-1,-1))
        self.set_unit_pos('Chen', (-1,-1))
        self.set_unit_pos('Mokou', (-1,-1))
        self.set_unit_pos('Keine', (-1,-1))
        self.set_unit_pos('Aya', (-1,-1))
        self.set_unit_pos('Tsubaki', (-1,-1))
        self.set_unit_pos('Momiji', (-1,-1))
        self.set_unit_pos('Nitori', (-1,-1))
        self.set_bg_overlay('Night')
        self.fade_from_color('black', 1)

        # Yuyuko and Yukari make an appearance
        self.play_music('event02')
        self.center_on_coords((7,10))
        self.deploy_unit('Yuyuko', (7,9))
        self.move_unit('Yuyuko', (7,10))
        self.deploy_unit('Yukari', (7,9))
        self.move_unit('Yukari', (8,10))

        self.say("The last light in their camp just went out.",
                'Yukari',
                'Yukari')
        self.say("I hope that Youmu has been getting enough rest.",
                'Yuyuko',
                'Yuyuko')
        self.say("She's doing fine. Youmu's already organized together a rather impressive squad.",
                'Yukari',
                'Yukari')

        # Ran discovers the two of them
        self.set_unit_pos('Ran', (9,8))
        self.emote('Ran', 'exclamation')
        self.move_unit('Ran', (9,10))

        self.say("Oh! Ran! I suppose we've been discovered!",
                'Yuyuko',
                'Yuyuko')
        self.say("I thought it was your spiritual energy that I sensed.",
                'Ran',
                'Ran')
        self.say("I suppose I can't cloak my presence from my own shikigami.",
                'Yukari',
                'Yukari')
        self.say("Ran, your shikigami Chen has certainly grown. ",
                'Yuyuko',
                'Yuyuko')
        self.say("We were both impressed that she suggested that to the emissary. I was afraid I would have to call in some favors to bend Tenma to your side, but it seems you did well enough on your own.",
                'Yukari',
                'Yukari')
        self.say("Lady Yukari, is this some kind of trial for us as well?",
                'Ran',
                'Ran')
        self.emote('Yukari', 'dotdotdot')

        self.say("In a sense. I won't tip my hand just yet though. Don't get the wrong impression, Ran.",
                'Yukari',
                'Yukari')
        self.say("We've been fairly active behind the scenes setting some things in motion. This isn't a vacation for Yuyuko and me.",
                'Yukari',
                'Yukari')
        self.say("I see. I won't question it then.",
                'Ran',
                'Ran')
        self.say("Oh, and Ran? Let's keep our meeting a secret.",
                'Yukari',
                'Yukari')
        self.say("Yes, milady.",
                'Ran',
                'Ran')
        self.say("Good luck tomorrow, Ran!",
                'Yuyuko',
                'Yuyuko')

        # Remove Yuyuko and Yukari, set scene to daytime
        self.fade_to_color('black', 1)
        self.kill_unit('Yuyuko')
        self.kill_unit('Yukari')
        self.set_unit_pos('Ran', (-1,-1))
        self.set_unit_pos('Tsubaki', (14, 20))
        self.set_unit_pos('Aya', (14, 19))
        self.set_bg_overlay(None)
        self.fade_from_color('black', 1)
        self.play_music('event01')
        self.center_on('Aya')

        self.say("Good morning, Youmu!",
                'Aya',
                'Aya')

        # Move Youmu out from the house
        self.set_unit_pos('Youmu', (16, 17))
        self.move_unit('Youmu', (16, 18))
        
        self.emote('Youmu', 'exclamation')

        # Aya snaps a photo
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        self.say("I have a special delivery for you!",
                'Aya',
                'Aya')

        # Rest of the party moves out of the house
        self.set_unit_pos('Reimu', (16,17))
        self.move_unit('Reimu', (17,17))
        self.set_unit_pos('Marisa', (16,17))
        self.move_unit('Marisa', (17,18))
        self.set_unit_pos('Mokou', (20,18))
        self.move_unit('Mokou', (17,19))
        self.set_unit_pos('Keine', (20,18))
        self.move_unit('Keine', (18,19))
        self.set_unit_pos('Ran', (20,18))
        self.move_unit('Ran', (18,17))
        self.set_unit_pos('Chen', (20,18))
        self.move_unit('Chen', (18,18))

        self.emote('Youmu', 'questionmark')

        self.say("'Lord Tenma to Youmu's Party: The First Ever Trial of the Red Feathers for Outsiders!'",
                'Youmu',
                'Youmu')

        # Aya snaps another photo
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)


        self.say("Tenma and the Order of the Red Feathers have agreed to your proposal. I've been tasked with running the Tengu trial, and if you succeed, you'll be on your way to the sanctuary. Aya and Momiji will be my assistants.",
                'Tsubaki',
                'Tsubaki')
        self.say("What does this trial consist of?",
                'Youmu',
                'Youmu')
        self.say("Come to the Nine Heavens Waterfall to find out.",
                'Tsubaki',
                'Tsubaki')
        self.say("This is sure to be a great front page article on the evening edition!",
                'Aya',
                'Aya')

        self.set_cursor_state(True)
        self.set_stats_display(True)
        self.done = True