from lostsky.worldmap.event import BattleEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, TurnNumTrigger, ArrivalTrigger, UnitAliveTrigger
from lostsky.battle.mapobj import SpiritSourcePoint

class Mission(BattleEvent):

    def __init__(self):
        # Event Data
        """

        """
        name = 'The Tengu Trial'
        location = 'Nine Heavens Waterfall'
        id_string = 'CH3ST3'
        prereqs = ['CH3ST2']
        auto_signup = True
        cancellable = True
        show_rewards = True
        desc = "In a surprise announcement, Lord Tenma has offered a challenge to Youmu\'s clan to take the Trial of the Red Feathers. \"It\'s just going to be a show. Youmu doesn't stand a chance. We'll never let outsiders set foot in the sanctuary at the mouontain summit.,\" Momiji was quoted as saying as her Wolf Tengu prepared for the morning's activities."
        
        BattleEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)

        ssp_list = [SpiritSourcePoint('Nitori 1',(1,30),0),
                    SpiritSourcePoint('Nitori 2',(26,30),0),
                    SpiritSourcePoint('Aya 1',(2,24),0),
                    SpiritSourcePoint('Aya 2',(6,18),0),
                    SpiritSourcePoint('Momiji 1',(27,21),0),
                    SpiritSourcePoint('Momiji 2',(18,14),0)
                    ]

        # Map Data
        map_name = 'ch3st3.txt'
        mission_type = 'battle'
        objective = {'type':'Capture Spirit Sources',
                     'ssps': ssp_list,
                     'number': 6,
                     'desc':'Capture all Spirit Source Points in N turns'
                     }
        
        deploy_data = {'enable':True,
                       'max_units':9,
                       'preset_units':{},
                       'boxes':[(12,24,3,3)]
                       }
        reward_list = [('treasure','tengu_feather')
                   ]
                
        # Enemy Unit Data
        enemy_unit_data = [  {'template_name': 'Momiji',
                                'unit_name': 'Momiji',
                                    'level': 10},
                            {'template_name': 'Wolf Tengu',
                                'unit_name': 'Wolf Tengu A',
                                'level': 9},
                            {'template_name': 'Wolf Tengu',
                                'unit_name': 'Wolf Tengu B',
                                'level': 9},
                            {'template_name': 'Wolf Tengu',
                                'unit_name': 'Wolf Tengu C',
                                'level': 9},
                            {'template_name': 'Wolf Tengu',
                                'unit_name': 'Wolf Tengu D',
                                'level': 9},

                            {'template_name': 'Wolf Tengu',
                             'unit_name': 'Wolf Tengu E',
                             'level': 9},


                             {'template_name': 'Nitori',
                                    'unit_name': 'Nitori',
                                        'level': 10},

                            {'template_name': 'Kappa',
                                'unit_name': 'Kappa A',
                                'level': 9},
                            {'template_name': 'Kappa',
                                'unit_name': 'Kappa B',
                                'level': 9},
                            {'template_name': 'Kappa',
                                'unit_name': 'Kappa C',
                                'level': 9},
                            {'template_name': 'Kappa',
                                'unit_name': 'Kappa D',
                                'level': 9},

                            {'template_name': 'Aya',
                                'unit_name': 'Aya',
                                'level': 10},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Newspaper Corps A',
                                'level': 10},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Newspaper Corps B',
                                'level': 10},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Newspaper Corps C',
                                'level': 10},


                             {'template_name': 'Tsubaki',
                                    'unit_name': 'Tsubaki',
                                        'level': 15},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Red Feathers A',
                                'level': 13},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Red Feathers B',
                                'level': 13},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Red Feathers C',
                                'level': 13},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Red Feathers D',
                                'level': 13},
                            {'template_name': 'Tengu Ascetic',
                                'unit_name': 'Red Feathers E',
                                'level': 13},
#
#
                            
                            ]
        
        initial_spells = {'Momiji':['Dagger Throw'],
                          'Wolf Tengu A':['Dagger Throw'],
                          'Wolf Tengu B':['Dagger Throw'],
                          'Wolf Tengu C':['Dagger Throw'],
                          'Wolf Tengu D':['Dagger Throw'],

                          'Nitori':['Tanabata Festival'],
                          'Kappa A':['Illusion Veil','Fireball'],
                          'Kappa B':['Tanabata Festival'],
                          'Kappa C':['Fireball'],
                          'Kappa D':['Tanabata Festival'],

                          'Aya':['Holy Amulet'],
                          'Newspaper Corps A':['Shimmering Stars'],
                          'Newspaper Corps B':['Feather Pin'],
                          'Newspaper Corps C':['Spirit Break'],

                          'Tsubaki':['Leaf Crystal'],
                          'Red Feathers A':['Spirit Break'],
                          'Red Feathers B':['Fireball'],
                          'Red Feathers C':['Spirit Break'],
                          'Red Feathers D':['Fireball'],
                          'Red Feathers E':['Spirit Break'],

                            }
        initial_traits = {'Aya':['Flight'],
                          'Newspaper Corps A':['Flight'],
                          'Newspaper Corps B':['Flight'],
                          'Newspaper Corps C':['Flight'],
                          'Nitori':['Danmaku Sniper', 'Swimming'],
                          'Kappa A':['Swimming'],
                          'Kappa B':['Swimming'],
                          'Kappa C':['Swimming'],
                          'Kappa D':['Swimming'],
                          'Red Feathers A':['Flight'],
                          'Red Feathers B':['Flight'],
                          'Red Feathers C':['Flight'],
                          'Red Feathers D':['Flight'],
                          'Red Feathers E':['Flight'],


                          }
        initial_ai_states = {'Momiji':'Defend',
                             'Wolf Tengu A':'Defend',
                             'Wolf Tengu B':'Defend',
                             'Wolf Tengu C':'Defend',
                             'Wolf Tengu D':'Defend',

                             'Nitori':'Defend',
                             'Kappa A':'Support',
                             'Kappa B':'Attack',
                             'Kappa C':'Attack',
                             'Kappa D':'Attack',

                             'Aya':'Pursuit',
                             'Newspaper Corps A':'Pursuit',
                             'Newspaper Corps B':'Pursuit',
                             'Newspaper Corps C':'Pursuit',


                             'Tsubaki':'Defend',
                             'Red Feathers A':'Defend',
                             'Red Feathers B':'Defend',
                             'Red Feathers C':'Defend',
                             'Red Feathers D':'Defend',
                             'Red Feathers E':'Defend',


                            }
        initial_locations = {

                             'Youmu': (13,22),
                             'Marisa': (11,23),
                             'Reimu': (11,24),
                             'Ran': (15,23),
                             'Chen': (15,24),
                             'Keine': (12,25),
                             'Mokou': (14,25),


                             'Momiji': (19, 15),
                             'Wolf Tengu A': (20, 14),
                             'Wolf Tengu B': (18, 16),
                             'Wolf Tengu C': (26, 21),
                             'Wolf Tengu D': (28, 21),

                             'Nitori':(3,33),

                             'Kappa A': (4, 28),
                             'Kappa B': (6, 33),
                             'Kappa C': (22, 27),
                             'Kappa D': (20, 30),

                             'Aya': (13,19),
                             'Newspaper Corps A': (8, 18),
                             'Newspaper Corps B': (7, 20),
                             'Newspaper Corps C': (5, 22),

                             'Tsubaki': (25,4),
                             'Red Feathers A':(2,6),
                             'Red Feathers B':(9,3),
                             'Red Feathers C':(19,4),
                             'Red Feathers D':(24,8),
                             'Red Feathers E':(32,10),
                             }
        reserve_units = ['Wolf Tengu E']
        all_landmarks = [{'name':'Tengu Shrine 1',
                          'id_string':'minishrine',
                          'location':(18,13)
                          },
                        {'name':'Tengu Shrine 2',
                          'id_string':'minishrine',
                          'location':(27,20)
                          },
                        {'name':'Tengu Shrine 3',
                          'id_string':'minishrine',
                          'location':(6,17)
                          },
                        {'name':'Tengu Shrine 4',
                          'id_string':'minishrine',
                          'location':(2,23)
                          },
                        {'name':'Tengu Shrine 5',
                          'id_string':'minishrine',
                          'location':(1,29)
                          },
                        {'name':'Tengu Shrine 6',
                          'id_string':'minishrine',
                          'location':(26,29)
                          },
                        {'name':'Tengu Outpost',
                          'id_string':'shrine',
                          'location':(25,3)
                          },
                        {'name':'Tengu House 1',
                          'id_string':'house_2',
                          'location':(26,3)
                          },
                        {'name':'Tengu House 2',
                          'id_string':'house_2',
                          'location':(24,3)
                          },


        ]

        required_starters = ['Youmu', 'Marisa', 'Reimu', 'Chen', 'Ran', 'Keine', 'Mokou']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [ActivateRedFeathers()]
        required_survivors = ['Aya', 'Tsubaki', 'Momiji', 'Youmu', 'Marisa', 'Reimu', 'Chen', 'Ran', 'Keine', 'Mokou', 'Nitori']
        post_mission_MAE = PostMissionMAE()
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Prologue event 
        """

        self.set_cursor_state(False)
        self.set_stats_display(False)

        self.play_music('battle02')

        self.center_on('Aya')
        self.say('Welcome to Trial of the Red Feathers!',
                 'Aya',
                 'Aya')

        # Aya flies around taking photos
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)


        self.say('Lady Tsubaki will explain the rules of this game.',
                 'Aya',
                 'Aya')

        self.center_on('Tsubaki')

        self.say('You stand at the top of the Waterfall of Nine Heavens. Seek out the six shrines that lie in the region and claim their spiritual energy as your own.',
                 'Tsubaki',
                 'Tsubaki')

        self.center_on('Aya')

        self.say('That sounds simple enough.',
                 'Youmu',
                 'Youmu')
        self.say("Don't be silly, we're going to be trying to stop you, and some of them are going to be really difficult to reach. Lady Tsubaki, we're free to deactivate the shrines too, right?",
                 'Aya',
                 'Aya')
        self.say("Of course.",
                 'Tsubaki',
                 'Tsubaki')
        self.say('The Kappa and I are ready! Your newspaper corps friends better be up to the task, Aya!',
                 'Momiji',
                 'Momiji')
        self.say("Ah, so they got the Kappa to help with this too? This will be fun!",
                 'Aya',
                 'Aya')

        # One last photo
        self.move_unit('Aya', (18,15))
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        # Tsubaki gets annoyed
        self.center_on('Tsubaki')
        self.emote('Tsubaki', 'scribble')
        self.say("This is a serious trial, Miss Shameimaru. And don't get too distracted taking all those pictures.",
                 'Tsubaki',
                 'Tsubaki')

        # All enemy units report their readiness
        self.center_on('Aya')
        self.move_unit('Aya', (6,20))
        self.center_on('Aya')
        self.say("Right. Right. Ready, everyone?",
                  'Aya',
                  'Aya')
        self.center_on('Newspaper Corps A')
        self.say("Ready to go!",
                  'Newspaper Corps A',
                  'Crow Tengu')

        self.center_on('Nitori')
        self.say("And over here as well!",
                  'Nitori',
                  'Nitori')

        self.center_on('Momiji')
        self.say("Wolf Tengu, we'll show those crows that we're the best warriors on this mountain!",
                  'Momiji',
                  'Momiji')

        self.center_on('Youmu')
        self.say("We'll do our best too!",
                  'Youmu',
                  'Youmu')

        self.set_cursor_state(True)
        self.set_stats_display(True)


class ActivateRedFeathers(MapActionEvent):

    def __init__(self):
        triggers = [TurnNumTrigger(12),
                    UnitAliveTrigger('Tsubaki', True)]

        MapActionEvent.__init__(self, triggers)

    def execute(self):

        # Higher level Tengu change AI state into pursuit mode
        self.center_on('Tsubaki')
        self.say('How about you give it a shot, Red Feathers.', 'Tsubaki', 'Tsubaki')

        switch_list = ['Red Feathers A', 'Red Feathers B', 'Red Feathers C', 'Red Feathers D',
                       'Red Feathers E']
        for unit in self.map.team2:
            if unit.name in switch_list:
                self.set_ai_state(unit.name, "Pursuit")

class PostMissionMAE(MapActionEvent):
    def __init__(self):
        # Triggers on turn 2
        triggers = []
        MapActionEvent.__init__(self,triggers)

    def pre_exec(self):
        self.remove_all_enemies()

        # Move the party, Tsubaki, Aya, Momiji, and Nitori together
        self.set_unit_pos('Tsubaki', (13,27))
        self.set_unit_pos('Aya', (12,28))
        self.set_unit_pos('Nitori', (13,28))
        self.set_unit_pos('Momiji', (14,28))
        self.set_unit_pos('Youmu', (13,24))
        self.set_unit_pos('Marisa', (11,23))
        self.set_unit_pos('Reimu', (11,24))
        self.set_unit_pos('Ran', (15,23))
        self.set_unit_pos('Chen', (15,24))
        self.set_unit_pos('Keine', (12,25))
        self.set_unit_pos('Mokou', (14,25))

    def execute(self):

        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.stop_music()
        self.center_on('Youmu')

        self.say("You've done it. Congratulations, Youmu.",
            'Tsubaki',
            "Tsubaki")

        # Aya takes a picture
        self.startle('Aya')
        self.play_sfx('camera')
        self.fade_to_color('white', 0.2)
        self.fade_from_color('white', 0.2)

        self.say("That was definitely one of the toughest battles yet.",
            'Marisa',
            "Marisa")

        self.say("Youmu, step forward.",
            'Tsubaki',
            "Tsubaki")

        # Youmu moves closer to Tsubaki
        self.move_unit('Youmu', (13,25))
        self.move_unit('Tsubaki', (13,26))

        self.say('From this day forward, you are acknowledged as a part of the Order of the Red Feathers.',
            'Tsubaki',
            "Tsubaki")
        self.say("So that means we'll get to visit the mountain's summit, right?",
            'Reimu',
            'Reimu')
        self.say("We'll uphold our end of the deal. Aya, show them to the top of the mountain.",
            'Tsubaki',
            "Tsubaki")
        self.say("Youmu, this red feather amulet should permit you to pass.",
            'Tsubaki',
            "Tsubaki")
        self.say("Just what I wanted to hear! This journalist is going right into the middle of the action!",
            'Aya',
            "Aya")

        self.deploy_unit('Wolf Tengu E', (25,23))
        self.move_unit('Wolf Tengu E', (20, 23))
        self.say("Momiji! Momiji!",
            'Wolf Tengu',
            "Wolf Tengu")
        self.startle('Momiji')
        self.move_unit('Momiji',(14,26) )
        self.move_unit('Momiji',(15,26) )
        self.move_unit('Momiji',(15,25) )
        self.move_unit('Momiji',(16,24) )
        self.move_unit('Momiji',(16,23) )
        self.move_unit('Momiji',(18,23) )

        self.startle('Momiji')
        self.say("What happened?",
            'Momiji',
            'Momiji')

        self.say("The outpost on the mountain path is under attack by a Kodama Lord! We need help!",
            'Wolf Tengu',
            "Wolf Tengu")

        self.emote('Momiji', 'exclamation')
        self.say("Lady Tsubaki, I have to go.",
            'Momiji',
            'Momiji')
        self.move_unit('Wolf Tengu E', (26, 23))
        self.move_unit('Momiji', (26, 23))

        self.say("Wait up, Momiji!",
            'Youmu',
            'Youmu')

        self.set_cursor_state(False)
        self.set_stats_display(False)