from lostsky.worldmap.event import ConversationEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent 

class Mission(ConversationEvent):
    
    def __init__(self):
        # Event Data
        name = 'Prologue'
        location = 'Netherworld Gardens'
        id_string = 'Prologue'
        prereqs = []
        auto_signup = True
        cancellable = True
        show_rewards = True
        desc = 'The prologue to the story.'
        
        ConversationEvent.__init__(self, name, location,
                             id_string, prereqs, 
                             auto_signup, cancellable, 
                             show_rewards, desc)
        
        # Map Data
        map_name = 'prologue.txt'
        mission_type = 'conversation'
        objective = None
        deploy_data = {}
        reward_list = []
                
        # Enemy Unit Data
        enemy_unit_data = [ {'template_name': 'Yuyuko',
                                'unit_name': 'Yuyuko',
                                    'level': 10
                                },
                                
                            {'template_name': 'Yukari',
                            'unit_name': 'Yukari',
                                'level': 16
                            }
                          ]
        
        initial_spells = {}
        initial_traits = {}
        initial_ai_states = {}
        initial_locations = {'Ran':(-1,7),
                             'Chen':(-1,6),
                             'Youmu':(5,5),
                             'Yukari':(6,7),
                             'Yuyuko':(5,15)
                             }
        reserve_units = []
        all_landmarks = [{'name':'CB1',
                          'id_string':'cherryblossom_tree',
                          'location':(2,12)},
                          {'name':'CB2',
                          'id_string':'cherryblossom_tree',
                          'location':(6,10)},
                          {'name':'CB3',
                          'id_string':'cherryblossom_tree',
                          'location':(3,8)},
                          {'name':'CB4',
                          'id_string':'cherryblossom_tree',
                          'location':(8,8)},
                          {'name':'CB5',
                          'id_string':'cherryblossom_tree',
                          'location':(3,4)},
                          {'name':'CB6',
                          'id_string':'cherryblossom_tree',
                          'location':(4,7)},
                          {'name':'CB7',
                          'id_string':'cherryblossom_tree',
                          'location':(8,7)},
                          {'name':'CB8',
                          'id_string':'cherryblossom_tree',
                          'location':(12,1)},
                          {'name':'CB9',
                          'id_string':'cherryblossom_tree',
                          'location':(14,4)},
                          {'name':'CB10',
                          'id_string':'cherryblossom_tree',
                          'location':(10,7)},
                          
                          {'name':'CB11',
                          'id_string':'cherryblossom_tree',
                          'location':(16,0)},
                          {'name':'CB12',
                          'id_string':'cherryblossom_tree',
                          'location':(17,3)},
                          {'name':'CB13',
                          'id_string':'cherryblossom_tree',
                          'location':(18,1)},
                          {'name':'CB14',
                          'id_string':'cherryblossom_tree',
                          'location':(19,4)},
                          {'name':'CB15',
                          'id_string':'cherryblossom_tree',
                          'location':(22,3)},
                          
                         ]
    
        required_starters = ['Chen','Ran','Youmu']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = []
        required_survivors = []
        post_mission_MAE = None
    
        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE) 

class PreMissionMAE(MapActionEvent):
    
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self,triggers)
        
    def execute(self):
        """
        Prologue event 
        """
        self.set_bg_overlay('Night')
        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.play_music('event01')
        
        # Introduction sequence
        self.say("Our world's name is Gensokyo. Long ago, we were separated from the rest of the world by a great magical barrier which has allowed our realm to remain untouched by influences of the outside.",
                 None, None)
        self.say("Our world is home to both humans and youkai, those beings that people would also call playful fairies, energetic spirits, and whimsical ghosts. What the outside world refers to as fantasy and myth are well alive here.",
                 None, None)
        self.say("Within Gensokyo, there exist distinct worlds for the living and the dead. The spirits of the Netherworld are managed by the master of Hakugyokurou, the ghostly princess Yuyuko Saigyouji.",
                 None, None)
        self.say("The ghosts and spirits at Hakugyokurou idle their time with revelry in the majestic garden, which is tended to by the dutiful Saigyouji retainer, a half-ghost by the name of Youmu Konpaku.",
                 None, None)
        self.say("Talented in the arts of swordplay, this Saigyouji retainer is also the vigilant protector of the realm. It is from this world of the dead that our story begins as she speaks with Yukari Yakumo, the youkai of gaps.",
                 None, None)
        
        # Conversation between Yukari, Yuyuko, and Youmu
        self.say("Ah, Youmu, has Yuyuko returned yet?",
                 'Yukari','Yukari')
        self.say('Returned? Did she go someplace?',
                 'Youmu','Youmu')
        self.move_unit('Yuyuko',(5,7))
        self.say('How irresponsible of you to not notice, Youmu.',
                 'Yuyuko','Yuyuko')
        self.say("I'm terribly sorry, Milady. I shall try harder to keep watch over you, always.",
                 'Youmu','Youmu')
        self.say('Oh dear, that might be a little troublesome...',
                 'Yuyuko','Yuyuko')
        self.say("To think that sweet, little Youmu has turned into such a voyeurist.",
                 'Yukari','Yukari')
        self.emote('Youmu','exclamation')
        self.startle('Youmu')
        self.say("I--",
                 'Youmu','Youmu')
        self.say("But that's not important. Yuyuko, did you find it?",
                 'Yukari','Yukari')
        self.say("It's a good thing that it was recovered not long ago. I don't know what I'd say if it was lost again.",
                 'Yuyuko','Yuyuko')
        self.say("Lost what again?",
                 'Youmu','Youmu')
        self.say("My favorite kimono, of course. Don't you think it looks lovely in this light?",
                 'Yuyuko','Yuyuko')
        self.say("Isn't it the same as always?",
                 "Youmu","Youmu")
        self.say("Try looking again. Notice the light shades and how they blend?",
                 "Yukari","Yukari")
        self.say("The only thing that's different from normal is that Milady is holding a paper lantern. Wait, isn't that the Lantern of Souls?",
                 "Youmu","Youmu")
        self.say("And doesn't it really bring out the colors in my kimono?",
                 "Yuyuko","Yuyuko")
        self.say("If you insist, Milady",
                 "Youmu","Youmu")
        self.say("Well, we'd better get going. The night grows short and all.",
                 "Yukari","Yukari")
        self.say("Going? Then I'll just--",
                 "Youmu","Youmu")
        self.say("Stay and be good, okay?",
                 "Yukari","Yukari")
        self.say("We'll be back before long, right?",
                 "Yuyuko","Yuyuko")
        self.say("One portal for two please!",
                 "Yukari","Yukari")
        
        # Fade to white, remove Yukari and Yuyuko from the map, fade back
        self.fade_to_color('white',0.1)
        self.kill_unit('Yuyuko')
        self.kill_unit('Yukari')
        self.fade_from_color('white',0.1)

        self.emote('Youmu','dotdotdot')
        self.say("They left before I could even answer.",
                 "Youmu","Youmu")

        # Next scene, the moring after with Youmu, Ran, and Chen
        self.fade_to_color('black',1.0)
        self.set_bg_overlay(None)
        self.set_unit_pos('Youmu',(5,10))
        self.fade_from_color('black',1.0)
        
        self.say("The next morning...", None, None)
        
        self.move_unit('Youmu',(7,10))
        self.say("I should have gone with them...",
                 "Youmu","Youmu")
        self.move_unit('Youmu',(5,10))
        self.say("Should I go out and search for them?",
                 "Youmu","Youmu")
        
        self.move_unit('Ran',(7,7))
        self.move_unit('Chen',(6,6))
        self.say("Excuse us, Youmu.",
                 "Ran","Ran")
        self.move_unit('Youmu',(7,8))
        self.set_cursor_state(True)
        self.center_on('Youmu')
        self.say("I don't suppose you would know when Lady Yuyuko will be back?",
                 "Youmu","Youmu")
        self.set_cursor_state(False)
        self.say("Wait, your master is not with you?",
                 "Ran","Ran")
        self.say("Lady Yukari told us that she was heading to Hakugyokurou the other day, and hasn't been back yet.",
                 "Ran","Ran")
        self.say("Well, she was here yesterday, but then she left with Lady Yuyuko. I thought they would be with you.",
                 "Youmu","Youmu")
        self.say("Say, if Lady Yukari needed someone to go with her, why did she go all the way here to ask Lady Yuyuko to go, but not bring us, too?",
                 "Chen","Chen")
        self.say("I didn't catch most of it, but they said they were going to the Forest of Magic to meet a ghost. Well, they did take the Lantern of Souls with them. Was that the reason why?",
                 "Youmu","Youmu")
        self.say("There's a lot of commotion in the forest right now. The youkai there are in a panic.",
                 "Chen","Chen")
        self.say("What happened?",
                 "Youmu","Youmu")
        self.say("That's what I'd like to know. I bet it's connected to whatever our masters went after last night.",
                 "Ran","Ran")
        self.say("Then we need to head out now. I'm worried about Miss Yuyuko.",
                 "Youmu","Youmu")
        self.say("Although I do wonder if all of this is an elaborate plot on the part of Lady Yukari. Either way, let's get moving as soon as you're ready.",
                 "Ran","Ran")
        self.say("And so the three set off in search for their missing masters, not knowing it would be the first step in a long journey.",
                 None, None)
        
        self.set_cursor_state(True)
        self.set_stats_display(True)
        