import pygame
from pygame.locals import *
from lostsky.core.utils import split_lines

# spell creation system

class SpellSynthesisSystem(object):


    def __init__(self,engine):
    
        """
        # function name: __init__
        # purpose: creates the spell creation system object
        # inputs: engine - game system engine
        """
    
        self.engine = engine

    def spell_synthesis_menu(self):
        """
        # function name: spell creation menu
        # purpose: allows player to select among menu options within the spell creation menu        
        """
        
        text_menu_header = self.engine.bfont.render("Spell Synthesis",True,(0,0,0))
        text_spellrecipes = self.engine.bfont.render("Spell Recipes",True,(0,0,0))
        text_desc_recipes = self.engine.sfont.render("Create spell items from elemental crystals. Discover new recipes to gain access to new spells.",True,(0,0,0))
        text_alchemy = self.engine.bfont.render("Element Alchemy",True,(100,100,100))
        text_desc_alchemy = self.engine.sfont.render("Use alchemy to combine two separate elemental crystals into a new one.",True,(0,0,0))
        text_treasures = self.engine.bfont.render("Treasures",True,(0,0,0))
        text_desc_treasures = self.engine.sfont.render("View treasures in your possession.",True,(0,0,0))
        text_tutorial = self.engine.bfont.render("Tutorial",True,(0,0,0))
        text_desc_tutorial = self.engine.sfont.render("Read how to use spell synthesis.",True,(0,0,0))
        text_cancel = self.engine.bfont.render("Cancel",True,(0,0,0))
        text_desc_cancel = self.engine.sfont.render("Exit the spell synthesis menu.",True,(0,0,0))
        text_desc = [text_desc_recipes, text_desc_alchemy,text_desc_treasures,text_desc_tutorial,text_desc_cancel]

        menu_flag = True
        menu_pos = 0
        
        while menu_flag:
        
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == K_F4 and bool(event.mod & KMOD_ALT)):
                    exit()
                if event.type == KEYDOWN:
                    
                    if ( event.key == K_UP or event.key == K_LEFT ):
                        if menu_pos > 0:
                            menu_pos -= 1
                        elif menu_pos == 0:
                            menu_pos = 4
                    if ( event.key == K_DOWN or event.key == K_RIGHT ):
                        if menu_pos < 4:
                            menu_pos += 1
                        elif menu_pos == 4:
                            menu_pos = 0
                    if event.key == K_z or event.key == K_RETURN:

                        # Recipes
                        if menu_pos == 0:
                            self.recipe_menu()
                            
                        # Alchemy Menu
                        elif menu_pos == 1:
                            pass
                            
                        # Treasures
                        elif menu_pos == 2:
                            self.engine.treasure_menu()
                        
                        # Tutorial
                        elif menu_pos == 3:
                            self.tutorial_select()
                        
                        # Cancel
                        elif menu_pos == 4:
                            menu_flag = False
                            
                    if event.key == K_x:
                        menu_flag = False
                    
            if menu_flag:
                # Background image
                self.engine.surface.blit(self.engine.shop_bg,(0,0))
                self.engine.surface.blit(self.engine.text_board,(0,490))

                # Header
                self.engine.surface.blit(text_menu_header,(50,50))
                
                # Renders the options
                self.engine.surface.blit(text_spellrecipes,(70,70))
                self.engine.surface.blit(text_alchemy,(70,90))
                self.engine.surface.blit(text_treasures,(70,110))
                self.engine.surface.blit(text_tutorial,(70,130))
                self.engine.surface.blit(text_cancel,(70,150))
 
                # Draws the cursor
                pygame.draw.circle(self.engine.surface, (0,0,0), (60,78+20*menu_pos),5)
                
                # Draw description
                self.engine.surface.blit(text_desc[menu_pos],(30,530))
                
                pygame.display.flip()
                self.engine.clock.tick(60)
        
    ############################################
    # Interactive methods
    ############################################
    def say(self,string,speaker="",portrait=None):
       
        """
        # Function name: say
        # Purpose: Displays a line of text and awaits for the player to press Z to continue
        # Inputs: string = The string of text to be displayed
        #         portrait = portrait to display
        #         speaker = The name of the person saying it
        """
    
        # calls the line splitter function to generate the events
        if portrait:
            lines = split_lines(string,75)
        else:
            lines = split_lines(string)
        
        text_lines = []
        if speaker != "":
            text_speaker = self.engine.bfont.render("- "+speaker+" -",True,(0,0,0))
        else:
            text_speaker = self.engine.bfont.render(speaker,True,(0,0,0))
        
        # generates the text for each line
        for line in lines:
            text_lines.append(self.engine.mfont.render(line,True,(0,0,0)))
        menu_flag = True
        
        while menu_flag:
            
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == K_F4 and bool(event.mod & KMOD_ALT)):
                    exit()
                if event.type == KEYDOWN and (event.key == K_z or event.key == K_RETURN):
                    menu_flag = False
                        
            # Hold down C to skip through the game.
            keys = pygame.key.get_pressed()
            if keys[K_c]:
                menu_flag = False
            self.engine.surface.blit(self.engine.shop_bg,(0,0))
                
            self.engine.surface.blit(self.engine.text_board,(0,490))

            if portrait:
                self.engine.surface.blit(self.engine.portrait_catalog[portrait],(10,500))
                self.engine.surface.blit(text_speaker,(140,505))
                [self.engine.surface.blit(text_line,(140,530+20*index)) for index,text_line in enumerate(text_lines)]
            else:
                self.engine.surface.blit(text_speaker,(30,505))
                [self.engine.surface.blit(text_line,(30,530+20*index)) for index,text_line in enumerate(text_lines)]
            pygame.display.flip()
            self.engine.clock.tick(60)
    
    def recipe_menu(self):
        """
        # Function name: Recipe Menu
        # Purpose: Catalog of known spell recipes and direct access to created a wanted spell
        """
    
        text_menu_header = self.engine.bfont.render("Spell Recipe Library",True,(0,0,0))
        text_empty = self.engine.sfont.render('Empty',True,(0,0,0))
        all_recipe_data = self.get_all_recipe_data()
        
        # Recipe info stuff
        text_ingredients = self.engine.bfont.render("Ingredients:",True,(0,0,0))
        text_ingredient_name = self.engine.sfont.render("Name",True,(0,0,0))
        text_num_required = self.engine.sfont.render("Required",True,(0,0,0))
        text_num_available = self.engine.sfont.render("Available",True,(0,0,0))
        
        menu_flag = True
        menu_pos = 0
        
        while menu_flag:
            
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == K_F4 and bool(event.mod & KMOD_ALT)):
                    exit()
                if event.type == KEYDOWN:
                    
                    if ( event.key == K_UP or event.key == K_LEFT ) and self.engine.player.known_recipes:
                        if menu_pos > 0:
                            menu_pos -= 1
                        elif menu_pos == 0:
                            menu_pos = len(self.engine.player.known_recipes)-1
                    if ( event.key == K_DOWN or event.key == K_RIGHT ) and self.engine.player.known_recipes:
                        if menu_pos < len(all_recipe_data)-1:
                            menu_pos += 1
                        elif menu_pos == len(all_recipe_data)-1:
                            menu_pos = 0
                    if (event.key == K_z or event.key == K_RETURN) and self.engine.player.known_recipes:
                        self.recipe_create_menu(self.engine.spell_recipes_catalog[all_recipe_data[menu_pos]['spell_name']])
                        # Gets Updated Recipe Data
                        all_recipe_data = self.get_all_recipe_data()
                        
                    if event.key == K_x:
                        menu_flag = False
                    
            if menu_flag:
                # Background image
                self.engine.surface.blit(self.engine.stats_bg,(0,0))
                self.engine.surface.blit(self.engine.text_board,(0,490))

                # Header
                self.engine.surface.blit(text_menu_header,(50,50))
                
                # draws the list of recipes known
                [self.engine.surface.blit(recipe_data['name_small'],(70,70+20*index)) for index,recipe_data in enumerate(all_recipe_data)]
                
                
                
                # only draws the cursor if any recipes are known
                if self.engine.player.known_recipes:
                    self.engine.surface.blit(all_recipe_data[menu_pos]['name_big'],(470,50))
                    self.engine.surface.blit(text_ingredients,(470,70))
                    self.engine.surface.blit(text_ingredient_name,(490,90))
                    [self.engine.surface.blit(self.engine.icons[ingredient[0]],(490,110+24*index)) for index,ingredient in enumerate(all_recipe_data[menu_pos]['ingredients'])]
                    [self.engine.surface.blit(ingredient[1],(525,115+24*index)) for index,ingredient in enumerate(all_recipe_data[menu_pos]['ingredients'])]
                    self.engine.surface.blit(text_num_required,(640,90))
                    [self.engine.surface.blit(ingredient[2],(640,115+24*index)) for index,ingredient in enumerate(all_recipe_data[menu_pos]['ingredients'])]
                    self.engine.surface.blit(text_num_available,(720,90))
                    [self.engine.surface.blit(ingredient[3],(720,115+24*index)) for index,ingredient in enumerate(all_recipe_data[menu_pos]['ingredients'])]
                    
                    pygame.draw.circle(self.engine.surface, (0,0,0), (60,78+20*menu_pos),5)
                    
                else:
                    self.engine.surface.blit(text_empty,(70,70))
                    
                    
                pygame.display.flip()
                self.engine.clock.tick(60)
                
    def recipe_create_menu(self,recipe):
        """
        # function name: recipe_create_menu
        # purpose: Player is given the option to create the spell from the recipe
        """
        
        can_create = recipe.check_ingredients(self.engine.player.treasures)
    
        # Set up text objects
        recipe_data = self.get_recipe_data(recipe)
        text_ingredients = self.engine.bfont.render("Ingredients:",True,(0,0,0))
        
        if can_create:
            text_confirm = self.engine.bfont.render("Confirm Creation",True,(0,0,0))
        else:
            text_confirm = self.engine.bfont.render("Confirm Creation",True,(100,100,100))
            
        text_cancel = self.engine.bfont.render("Cancel",True,(0,0,0))
        text_ingredient_name = self.engine.sfont.render("Name",True,(0,0,0))
        text_num_required = self.engine.sfont.render("Required",True,(0,0,0))
        text_num_available = self.engine.sfont.render("Available",True,(0,0,0))
        menu_flag = True # Only two positions, so we use True for Confirm and False for cancel
        menu_pos = True
        
        while menu_flag:
            
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == K_F4 and bool(event.mod & KMOD_ALT)):
                    exit()
                if event.type == KEYDOWN:
                    # Any arrow key press toggles between Confirm and Cancel
                    if ( event.key == K_UP or event.key == K_LEFT or event.key == K_DOWN or event.key == K_RIGHT ):
                        menu_pos = not menu_pos
                        
                    if (event.key == K_z or event.key == K_RETURN) and menu_pos and can_create:
                        # Execute creation
                        self.engine.player.add_item(self.engine.spell_catalog[recipe.spell_action].construct_spell())
                        # Remove used items
                        [self.engine.player.remove_treasure(ingredient,recipe.required_ingredients[ingredient]) for ingredient in recipe.required_ingredients.keys()]

                        self.summary_screen(recipe)

                        # Refresh information about creation ability / number of items
                        can_create = recipe.check_ingredients(self.engine.player.treasures)
                        recipe_data = self.get_recipe_data(recipe)
                          
                    if event.key == K_x or ((event.key == K_z or event.key == K_RETURN) and not menu_pos):
                        menu_flag = False
                        return False
                   
                   
            if menu_flag:
                # Background image
                self.engine.surface.blit(self.engine.stats_bg,(0,0))
                self.engine.surface.blit(self.engine.text_board,(0,490))
                
                self.engine.surface.blit(recipe_data['name_big'],(50,50))
                self.engine.surface.blit(text_ingredients,(60,70))
                
                # Ingredient name
                self.engine.surface.blit(text_ingredient_name,(70,90))
                [self.engine.surface.blit(self.engine.icons[ingredient[0]],(70,110+24*index)) for index,ingredient in enumerate(recipe_data['ingredients'])]
                [self.engine.surface.blit(ingredient[1],(105,115+24*index)) for index,ingredient in enumerate(recipe_data['ingredients'])]
                # Ingredient quantity required
                self.engine.surface.blit(text_num_required,(220,90))
                [self.engine.surface.blit(ingredient[2],(220,115+24*index)) for index,ingredient in enumerate(recipe_data['ingredients'])]
                # Ingredient quantity owned
                self.engine.surface.blit(text_num_available,(300,90))
                [self.engine.surface.blit(ingredient[3],(300,115+24*index)) for index,ingredient in enumerate(recipe_data['ingredients'])]
                
                
                # Draws cursor
                if menu_pos:
                    pygame.draw.circle(self.engine.surface, (0,0,0), (50,418),5)
                else:
                    pygame.draw.circle(self.engine.surface, (0,0,0), (50,438),5)
                    
                self.engine.draw_uncreated_spell(recipe.spell_action)
                
                self.engine.surface.blit(text_confirm,(60,410))
                self.engine.surface.blit(text_cancel,(60,430))
                
                pygame.display.flip()
                self.engine.clock.tick(60)
                
        
    def summary_screen(self,recipe):
        """
        # function name: summary screen
        # purpose: shows the player what they created
        # inputs: ing1,ing2 = ingredient 1 and 2
        #         result = result from attempted synthesis
        """

        # Results screen text surface objects
        text_results_header = self.engine.bfont.render('Spell Synthesis Results:',True,(0,0,0))
        text_success = self.engine.bfont.render('Synthesis Successful!',True,(0,0,0))
        text_created_item = self.engine.sfont.render('Created item: %s'%(recipe.spell_action),True,(0,0,0))
        text_ingredients_header = self.engine.sfont.render('Ingredients:',True,(0,0,0))
        text_ingredients_list = [self.engine.sfont.render("%s: %s"%(self.engine.treasure_catalog[ingredient].name,str(quantity)),True,(0,0,0)) for (ingredient,quantity) in recipe.required_ingredients.iteritems()]
        
        text_instruct = self.engine.sfont.render("Press Z to Continue",True, (0,0,0))
        
        menu_flag = True

        while menu_flag:
            
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == K_F4 and bool(event.mod & KMOD_ALT)):
                    exit()
                if event.type == KEYDOWN:
                    
                    if event.key == K_z or event.key == K_RETURN:
                        menu_flag = False
                
            if menu_flag:
                # Background image
                self.engine.surface.blit(self.engine.stats_bg,(0,0))
                self.engine.surface.blit(self.engine.text_board,(0,490))

                self.engine.surface.blit(self.engine.results_panel,(280,140))
                self.engine.surface.blit(text_results_header,(305,170))
                self.engine.surface.blit(text_success,(315,190))
                self.engine.surface.blit(text_created_item,(315,210))
                
                self.engine.surface.blit(text_ingredients_header,(305,290))
                [self.engine.surface.blit(ingredient,(315,310+count*20)) for (count,ingredient) in enumerate(text_ingredients_list)]
                self.engine.surface.blit(text_instruct,(420,395))
                
                pygame.display.flip()
                self.engine.clock.tick(60)

           
    def add_recipe(self,spell_name):
        """
        # function name: add recipe
        # purpose: Adds a recipe to the list of known recipes
        """
        self.engine.player.known_recipes.append(spell_name)
        self.engine.player.known_recipes.sort()
                
                
    def tutorial_select(self):
        """
        # function name: tutorial select
        # purpose: select among a number of tutorials about the spell synthesis system
        """
        
        text_menu_header = self.engine.bfont.render("Tutorials",True,(0,0,0))
        text_spellsynth = self.engine.bfont.render("Spell Synthesis",True,(0,0,0))
        text_desc_spellsynth = self.engine.sfont.render("Tutorial: Describes the basics about spell synthesis and creating spells from recipes.",True,(0,0,0))

        text_cancel = self.engine.bfont.render("Cancel",True,(0,0,0))
        text_desc_cancel = self.engine.sfont.render("Exit the spell synthesis menu.",True,(0,0,0))
        
        text_desc = [text_desc_spellsynth,text_desc_cancel]

        menu_flag = True
        menu_pos = 0
        
        while menu_flag:
        
            # looks for event type data to select interaction
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == K_F4 and bool(event.mod & KMOD_ALT)):
                    exit()
                if event.type == KEYDOWN:
                    
                    if ( event.key == K_UP or event.key == K_LEFT ):
                        if menu_pos > 0:
                            menu_pos -= 1
                        elif menu_pos == 0:
                            menu_pos = 1
                    if ( event.key == K_DOWN or event.key == K_RIGHT ):
                        if menu_pos < 1:
                            menu_pos += 1
                        elif menu_pos == 1:
                            menu_pos = 0
                    if event.key == K_z or event.key == K_RETURN:

                        # Basic Tutorial
                        if menu_pos == 0:
                            self.tutorial_basic()
                            
                        # Cancel
                        elif menu_pos == 1:
                            menu_flag = False
                            
                    if event.key == K_x:
                        menu_flag = False
                    
            if menu_flag:
                # Background image
                self.engine.surface.blit(self.engine.shop_bg,(0,0))
                self.engine.surface.blit(self.engine.text_board,(0,490))

                # Header
                self.engine.surface.blit(text_menu_header,(50,50))
                
                # Renders the options
                self.engine.surface.blit(text_spellsynth,(70,70))
                self.engine.surface.blit(text_cancel,(70,90))
 
                # Draws the cursor
                pygame.draw.circle(self.engine.surface, (0,0,0), (60,78+20*menu_pos),5)
                
                # Draw description
                self.engine.surface.blit(text_desc[menu_pos],(30,530))
                
                pygame.display.flip()
                self.engine.clock.tick(60)
                
    def tutorial_basic(self):
        self.say("You wanted to learn about spell synthesis, right?","Marisa", "Marisa")
        self.say("Yes, how does it work?","Youmu", "Youmu")
        self.say("Youmu, have you ever done much cooking?","Marisa", "Marisa")
        self.say("Of course. I prepare meals for Lady Yuyuko all the time.","Youmu", "Youmu")
        self.say("Okay then, it should be a snap for you. First off, let's begin with the elemental crystals.","Marisa","Marisa")
        self.say("Elemental crystals are our basic ingredients to synthesize spells. They're like your meats, vegetables, and grains in cooking.","Marisa", "Marisa")
        self.say("What are these elemental crystals anyway?","Youmu", "Youmu")
        self.say("Elemental crystals contain large amounts of pure magical energy, each corresponding to one of the basic classes of magic.","Ran", "Ran")
        self.say("There are also more rare crystals that contain special magical properties. They can't be used directly so we have to use them to form our basic spells. The way we do this is with a spell recipe.","Ran","Ran")
        self.say("Since creating a spell is a complicated process, we need to rely on a recipe to create them. This tells us what steps we need to take to combine the elemental crystals.","Marisa", "Marisa")
        self.say("Some recipes will specify one ingredient like the very basic spells. More advanced spells will specify multiple crystal types you need, so keep an eye out for ways to obtain them.","Marisa", "Marisa")
        self.say("Basic ones are common, but it'll be harder to find the rarer special crystals.","Marisa", "Marisa")
        self.say("For instance to forge a Fireball spell we use the recipe for that spell to create an equippable spell item. Remember to equip the spell after use, and check any new recipes we may get on our travels.","Ran","Ran")
        self.say("That's all there is to it. Select the recipe, and the spell is instantly created for you. This concludes the basic spell creation tutorial.","Tutorial")
              
                
    def get_all_recipe_data(self):
        """
        # function name: get recipe data
        # purpose: constructs the recipe data for all known recipes
        """
    
        if not self.engine.unlock_shops:
            all_recipe_data = [self.get_recipe_data(self.engine.spell_recipes_catalog[recipe_name])
                                for recipe_name in self.engine.player.known_recipes]
        else:
            all_recipe_data = [self.get_recipe_data(recipe) for recipe in 
                               self.engine.spell_recipes_catalog.values()]
        return all_recipe_data
    
    def get_recipe_data(self,recipe):
        """
        # function name: get recipe data
        # purpose: constructs the recipe data for a single recipes
        """
    
        recipe_data = { 'spell_name':recipe.spell_action,
                        'name_small':self.engine.sfont.render(recipe.spell_action,True,(0,0,0)),
                        'name_big':self.engine.bfont.render(recipe.spell_action,True,(0,0,0)),
                        'ingredients':[],
                        }
        # How many of each ingredient is required, as well as how much is in the player's inventory
        for ingredient in recipe.required_ingredients.keys():
            
            # Required ingredients
            # Returns ingredient in black text if player has enough of that item, in red if they don't
            if ingredient in self.engine.player.treasures.keys() and self.engine.player.treasures[ingredient] >= recipe.required_ingredients[ingredient]:
                recipe_data['ingredients'].append(                
                    [self.engine.treasure_catalog[ingredient].icon,
                     self.engine.sfont.render(self.engine.treasure_catalog[ingredient].name,True,(0,0,0)),
                    self.engine.sfont.render(str(recipe.required_ingredients[ingredient]),True,(0,0,0,)),
                    self.engine.sfont.render(str(self.engine.player.treasures[ingredient]),True,(0,0,0))])
                
            else:
                recipe_data['ingredients'].append(                
                    [self.engine.treasure_catalog[ingredient].icon,
                     self.engine.sfont.render(self.engine.treasure_catalog[ingredient].name,True,(255,0,0)),
                    self.engine.sfont.render(str(recipe.required_ingredients[ingredient]),True,(255,0,0,))])
                # Case: Ingredient is not present at all
                if ingredient not in self.engine.player.treasures.keys():
                    recipe_data['ingredients'][-1].append(self.engine.sfont.render("0",True,(255,0,0)))
                # Case: There isn't enough of the ingredient
                else:
                    recipe_data['ingredients'][-1].append(self.engine.sfont.render(str(self.engine.player.treasures[ingredient]),True,(255,0,0)))
                    
               
        return recipe_data
            
class Recipe(object):

    def __init__(self,spell_action,required_ingredients):
        """
        # function name: __init__
        # purpose: creates a spell recipe
        # inputs:   spell_action - The spell produced from this recipe
        #           required_ingredients - the required ingredients for this spell
        """
    
        self.spell_action = spell_action
        # Format: {ingredient:quantity}
        self.required_ingredients = required_ingredients
        
    def check_ingredients(self,treasure_inventory):
        """
        # function name: check_ingredients
        # purpose: checks if player can create this item
        # inputs:  treasure_inventory - player treasure inventory
        # Outputs: True if recipe can be created
        #          False if recipe cannot be created
        """
    
        for ingredient in self.required_ingredients.keys():
        
            if ingredient not in treasure_inventory.keys() or treasure_inventory[ingredient] < self.required_ingredients[ingredient]:
                # At least one condition has not been satisfied to create the item, return False
                return False
                    
        # Condition satisfied, return True
        return True
        